C=======================================================================
C
C                                 S I M M D L
C
C     The program to simulate element test under cyclic loading
C
C
C     Developed        N. Yoshida
C                      Research Institute, Sato Kogyo Co., Ltd.
C                      Sept. 30, 1987     Version 1.0
C     Revised          N. Yoshida
C                      Feb. 1996          Version 2.0
C                      Add Byrne model and modification of M-F-S model
C     Revised          April, 2014 for convenience use
C
C=======================================================================
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION SPARAM(100), RET(2,100), PPARAM(20), GPARAM(20),
     1          TPARAM(20), TAURET(1000)
      CHARACTER*4 FORMT(8), TITLE(20)
      COMMON / PHAILW / IPHI
C
      OPEN(UNIT=11,FILE='Simmdl.out')
C
   10 WRITE(6,6900)
      READ(5,5030) TITLE
      READ(5,5000) ITYP, JTYP, KGTYP, KTTYP, IPRNT, IFILE,
     1             NMAX, ATOM
C
      ITYP = MAX0 (MIN0 (ITYP, 6), 1)
      JTYP = MAX0 (MIN0 (JTYP, 5), 1)
      KGTYP = MAX0 (MIN0 (KGTYP, 5), 1)
      KTTYP = MAX0 (MIN0 (KTTYP, 3), 1)
      IPRNT = MAX0 (MIN0 (IPRNT, 2), 0)
      IFILE = MAX0 (MIN0 (IFILE, 2), 0)
      IF (NMAX .LE. 0) NMAX = 40
C
C     Stress-strain relationships
C
  100 CONTINUE
      GO TO (110, 120, 130, 110, 120, 130), ITYP
C     hyperbolic model
  110 GO TO 190
C     Ramberg-Osgood model
  120 READ(5,5010) (SPARAM(J),J=1,3)
      GO TO 190
C     Ramberg-Osgood model modified by HARA
  130 READ(5,5010) (SPARAM(J),J=1,2)
      GO TO 190
  190 IF (ITYP .LE. 3) GO TO 200
C
C     Here new hysteresis rule
      READ(5,5040) MM,(SPARAM(J),J=7,12)
      IF (MM .GT. 3) READ(5,5010) (SPARAM(J),J=13,2*MM+12)
      SPARAM(4) = MM
C
C     Poerwater pressure generation model
C
  200 GO TO (210, 220, 210, 230, 240), JTYP
C     Martin-Finn-Seed model
  210 READ(5,5010) (PPARAM(J),J=1,7)
      IF (JTYP .EQ. 3) READ(5,5010) (PPARAM(J),J=8,9),
     1                    PPARAM(17)
      GO TO 300
C     Ishihara-Towhata model
  220 CONTINUE
      READ(5,5020) (PPARAM(J),J=1,5), IPHI
      GO TO 300
C     Byrne model
  230 CONTINUE
      READ(5,5010) (PPARAM(J),J=1,2), (PPARAM(J),J=5,7)
      GO TO 300
C     Byrne model (Modified)
  240 CONTINUE
      READ(5,5010) (PPARAM(J),J=1,2), (PPARAM(J),J=5,9),
     1                    PPARAM(17)
      GO TO 300
C
C     Shear modelus model
C
  300 GO TO (310, 320, 330, 340, 350), KGTYP
  310 READ(5,5010) GMAX0, GPARAM(1)
      GO TO 390
  320 READ(5,5010) GMAX0, (GPARAM(J),J=1,3)
      GO TO 390
  330 READ(5,5010) GMAX0, (GPARAM(J),J=1,2)
      GO TO 390
  340 READ(5,5010) GMAX0, (GPARAM(J),J=1,5)
      GO TO 390
  350 READ(5,5010) GMAX0, (GPARAM(J),J=1,2)
      GO TO 390
  390 IF (GPARAM(1) .EQ. 0.) GPARAM(1) = 0.5
C
C     Shear strength model
C
  400 IF (JTYP .EQ. 2) THEN
C     Ishihara-Towhata model in which parameter is alrqady specified
        KTTYP = 3
        TPARAM(1) = 0.
        TPARAM(2) = PPARAM(5)
        GO TO 500
      ENDIF
C
  401 GO TO (410, 420, 430), KTTYP
C
  410 READ(5,5010) TMAX0, TPARAM(1)
      IF (TPARAM(1) .EQ. 0) TPARAM(1) = 1.0
      GO TO 500
  420 READ(5,5010) TMAX0, (TPARAM(J),J=1,3)
      IF (TPARAM(1) .EQ. 0) TPARAM(1) = 1.0
      GO TO 500
  430 READ(5,5010) TMAX0, (TPARAM(J),J=1,3)
      GO TO 500
C
C--------------- Print fundamental data ------------------
C
  500 WRITE(6,6000) TITLE, ITYP, JTYP, KGTYP, KTTYP, IPRNT, IFILE,
     1              NMAX,ATOM
C
      WRITE(6,6100)
      IF (ITYP .EQ. 1 .OR. ITYP .EQ. 4) WRITE(6,6110)
      IF (ITYP .EQ. 2 .OR. ITYP .EQ. 5)
     1                   WRITE(6,6120) (SPARAM(J),J=1,3)
      IF (ITYP .EQ. 3 .OR. ITYP .EQ. 6)
     1                   WRITE(6,6130) (SPARAM(J),J=1,2)
      IF (ITYP .LE. 3 .OR. ITYP .GE. 7) GO TO 540
      MM = SPARAM(4)
      IF (MM .EQ. 1) WRITE(6,6140) SPARAM(5)
      IF (MM .EQ. 1) GO TO 520
      WRITE(6,6150)
      K = (MM - 1) / 10 + 1
      DO 510 I = 1,K
      KK = 20 * K + 6
      IF (KK .GT. 2*MM+6) KK = 2*MM + 6
      WRITE(6,6160) (SPARAM(J),J=20*K-13, KK,2)
      WRITE(6,6170) (SPARAM(J),J=20*K-12, KK,2)
  510 CONTINUE
  520 DO 530 J = 7, 2*MM+6
      SPARAM(J) = SPARAM(J) / 100.
  530 CONTINUE
  540 CONTINUE
C
      WRITE(6,6200)
      IF (JTYP .EQ. 1) THEN
        WRITE(6,6210) (PPARAM(J),J=1,7)
      ELSEIF (JTYP .EQ. 2) THEN
        WRITE(6,6220) (PPARAM(J),J=1,5)
        IF (IPHI .EQ. 1) WRITE(6,*) 
     1     '          Internal friction angle at low stress not used'
      ELSEIF (JTYP .EQ. 3) THEN
        WRITE(6,6230) (PPARAM(J),J=1,9)
      ELSEIF (JTYP .EQ. 4) THEN
        WRITE(6,6240) (PPARAM(J),J=1,2),(PPARAM(J),J=5,7)
      ELSEIF (JTYP .EQ. 5) THEN
        WRITE(6,6250) (PPARAM(J),J=1,2),(PPARAM(J),J=5,9)
      ENDIF
C
      WRITE(6,6300) KGTYP
      IF (KGTYP .EQ. 1) WRITE(6,6310) GMAX0, GPARAM(1)
      IF (KGTYP .EQ. 2) WRITE(6,6320) GMAX0, (GPARAM(J),J=1,3)
      IF (KGTYP .EQ. 3) WRITE(6,6330) (GPARAM(J),J=1,2)
      IF (KGTYP .EQ. 4) WRITE(6,6340) (GPARAM(J),J=1,5)
      IF (KGTYP .EQ. 5) WRITE(6,6350) (GPARAM(J),J=1,2)
      IF (KGTYP .EQ. 5) GPARAM(3) = ATOM
C
      WRITE(6,6400) KTTYP
      IF (KTTYP .EQ. 1) WRITE(6,6410) TMAX0, TPARAM(1)
      IF (KTTYP .EQ. 2) WRITE(6,6420) TMAX0, (TPARAM(J),J=1,3)
      IF (KTTYP .EQ. 3) WRITE(6,6430) (TPARAM(J),J=1,2)
      IF (KTTYP .EQ. 3) TPARAM(2) = DTAN(TPARAM(2)*DATAN(1.D0)/45.)
C
C     Move input parameters
C
  900 IF (ITYP .LE. 2) GO TO 920
      IF (ITYP .GT. 3) GO TO 910
      SPARAM(1) = SPARAM(1) / 100.
      SPARAM(5) = SPARAM(1)
      SPARAM(6) = SPARAM(2)
      GO TO 920
  910 IF (ITYP .EQ. 4) GO TO 920
      IF (ITYP .GT. 6) GO TO 920
      SPARAM(1) = SPARAM(1) / 100.
      SPARAM(5) = SPARAM(1)
      SPARAM(6) = SPARAM(2)
  920 IF (JTYP .NE. 2) GO TO 930
      PPARAM(12) = PPARAM(1)
      PPARAM(13) = PPARAM(2)
      PPARAM(14) = PPARAM(3)
      PPARAM(15) = PPARAM(4)
      PPARAM(16) = PPARAM(5)
  930 CONTINUE
      WRITE(6,6490)
C
C-------------------------------------------------------
C         SIMULATION
C-------------------------------------------------------
C
 1000 READ(5,5100, ERR=2000, END=2000) SIGV0, NSUB, NUNI, NSIG,
     1           ILIQ, GAMLIQ, FACT
      IF (SIGV0 .LE. 0.D0) GO TO 2000
      NSUB = MAX0 (MIN0 (NSUB, 1), 0)
      NUNI = MAX0 (MIN0 (NUNI, 2), 1)
      NSIG = MAX0 (MIN0 (NSIG, 3), 1)
      ILIQ = MAX0 (MIN0 (ILIQ, 3), 0)
      IF (FACT .EQ. 0.) FACT = 1.
      IF (ILIQ .EQ. 1 .AND. GAMLIQ .EQ. 0.) GAMLIQ = 5.
C
      IF (NUNI .EQ. 2) GO TO 1020
C
C     Constant amplitude loading
C
      READ(5,5110) NEND, AMP, TAUINC
      AMPL = AMP
      DO 1010 I = 1, NEND
      IF (TAUINC .NE. 0.) TAURET(I) = AMPL
      IF (TAUINC .EQ. 0.) TAURET(2*I-1) = AMPL
      IF (TAUINC .EQ. 0.) TAURET(2*I ) = 0.D0
      AMPL = -AMPL
 1010 CONTINUE
      IF (TAUINC .EQ. 0.) NEND = NEND * 2
      GO TO 1100
C
 1020 READ(5,5120) NEND, TAUINC, FORMT
      READ(5,FORMT) (TAURET(J),J=1,NEND)
C
 1100 IF (NSIG .EQ. 1) FACT = FACT * SIGV0
      DO 1110 I = 1, NEND
      TAURET(I) = TAURET(I) * FACT
      IF (I .EQ. 1) TAUMAX = DABS(TAURET(I))
      IF (I .GT. 1 .AND. DABS(TAURET(I)) .GT. TAUMAX)
     1               TAUMAX = DABS(TAURET(I))
 1110 CONTINUE
C
      WRITE(6,6520)
      TTT = TAUMAX / SIGV0
      WRITE(6,6500) SIGV0, NSUB, NUNI, NSIG, ILIQ, GAMLIQ, TAUMAX, TTT
      NEND1 = NEND / 2
      IF (I .EQ. 1) WRITE(6,6510) AMP, NEND1
      GAMLIQ = GAMLIQ / 100.
C
      IF (NSIG .EQ. 1 .OR. NSIG .EQ. 3) THEN
        CALL    TAUCON (ITYP, SPARAM, RET, NMAX, JTYP, PPARAM,
     1                  NSUB, KGTYP, GPARAM, KTTYP, TPARAM, GMAX0,
     2                  TMAX0, TAURET, TAUINC, NEND, IPRNT, IFILE,
     3                  ILIQ, GAMLIQ, ATOM, SIGV0, TTT)
      ELSE
        CALL   GAMCON (ITYP, SPARAM, RET, NMAX, JTYP, PPARAM,
     1                  NSUB, KGTYP, GPARAM, KTTYP, TPARAM, GMAX0,
     2                  TMAX0, GAMRET, GAMINC, NEND, IPRNT, IFILE,
     3                  ILIQ, ATOM, SIGV0)
      ENDIF
C
      WRITE(6,6520)
      GO TO 1000
C
 2000 STOP
 5000 FORMAT(6I2, I5, F10.0)
 5010 FORMAT(8F10.0)
 5020 FORMAT(5F10.0, I5)
 5030 FORMAT(20A4)
 5040 FORMAT(I5,15X,6F10.0)
 5100 FORMAT(F10.0,4I2,2X, 2F10.0)
 5110 FORMAT(I5, 2F10.0)
 5120 FORMAT(I5,f10.0, 8A4)
 6000 FORMAT(1H1,80(1H=),/,1X,20A4,/,1X,80(1H=),/,
     1       '     ITYP    JTYP   KGTYP   KTTYP   IPRNT   IFILE',
     1       '    NMAX      ATOM',/,1X,7I8,F12.3)
 6100 FORMAT(1X, 80(1H-),/,' Stress-strain model')
 6110 FORMAT(10X, 'hyperbolic model')
 6120 FORMAT(10X,'Ramberg-Osgoode model    ',/,
     1   15X,'     alpha       r       tau-r',/,15X,3F10.4)
 6130 FORMAT(10X,'Ramberg-Osgoode model (Hara) ',/,
     1   15X,'  gam-f(%)       r ',/,15X,2F10.4)
 6140 FORMAT(15X,'Modified dampling   Dmax=',F5.2,' %')
 6150 FORMAT(15X,'Modified dampling')
 6160 FORMAT(15X,' Damp(%)',10F6.2)
 6170 FORMAT(15X,' Strn(%)',10F6.2)
 6200 FORMAT(1X,80(1H-),/,
     1       ' Excess porewater pressure generation model')
 6210 FORMAT(10X,'Martin-Finn-Seed model',/,10X,'     C1   ',
     1  '     C2        C3        C4        K2         m   ',
     2  '      n   ',/,10X,7F10.4)
 6220 FORMAT(10X,'Ishihara-Towhata model',/,
     1    10x,'     Bp        Bu      Kappa    Phase t   Failure ',/,
     2    10x,'                                Ang(deg)  Ang(deg)',/,
     3    10X,7F10.4)
 6230 FORMAT(10X,'Martin-Finn-Seed model (Modified)',/,
     1 10X,'     C1        C2        C3        C4        K2   ',
     2     '      m         n      Phase t  Failure',/,
     3 80X,'   Ang(deg) Ang(deg)',/,10X, 9F10.4)
 6240 FORMAT(10X,'Byrne model',/,
     1 10X,'     C1        C2        K2         m         n  ',/,
     2 10X, 9F10.4)
 6250 FORMAT(10X,'Byrne model (Modified)',/,
     1 10X,'     C1        C2        K2   ',
     2     '      m         n      Phase t  Failure',/,
     3 60X,'   Ang(deg) Ang(deg)',/,10X, 9F10.4)
 6300 FORMAT(1X,80(1H-),/,' Shear modulus model',10X,'type',I3)
 6310 FORMAT(10X,'      Go        m   ',/,10X, F10.2, F10.4)
 6320 FORMAT(10X,'      Go        m        H1        H2   ',/,
     1       10X,F10.2, 3F10.4)
 6330 FORMAT(10X,'      m       Coef.  ',/, 10X, F10.2, F10.4)
 6340 FORMAT(10X,'      M         A        B         C    ',
     1       '      e   ',/, 10X, 5F10.4)
 6350 FORMAT(10X,'      m       Coef.  ',/, 10X, F10.2, F10.4)
 6400 FORMAT(1X,80(1H-),/,' Shear strength model',10X,'type',I3)
 6410 FORMAT(10X,'      Tmax      m   ',/,10X, 2F10.4)
 6420 FORMAT(10X,'      Tmax      m        H3        H4   ',/,
     1       10X, 4F10.4)
 6430 FORMAT(10X,'  Cohesion   Fric(deg) ',/,10X,2F10.4)
 6490 FORMAT(1X,80(1H-))
 6500 FORMAT(1H0,/,'     SIGV0   NSUB  NUNI  NSIG  ILIQ    GAMLIQ',
     1       '    TAUMAX    T/SIGV',/,1X,F10.3,4I6,2F10.3,F10.5)
 6510 FORMAT('   Amplitude=', F10.3, '      Half cycle=',I6)
 6520 FORMAT(1H0,80(1H=))
 6900 FORMAT('0 SIMMDL   Ver.2.0   Yoshida, N. developed')
C
      END
C=======================================================================
      SUBROUTINE TAUCON (ITYP, SPARAM, RET, NMAX, JTYP, PPARAM,
     1                    NSUB, KGTYP, GPARAM, KTTYP, TPARAM, GMAX0,
     2                    TMAX0, TAURET, TAUINC, NEND, IPRNT, IFILE,
     3                    ILIQ, GAMLIQ, ATOM, SIGV0, TTT)
C=======================================================================
C     This SUBROUTINE compute under stress control.
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION SPARAM(1), RET(2,1), PPARAM(1), GPARAM(1),
     1          TPARAM(1), TAURET(1)
      REAL*8 MXGAM, MXTAU
      DIMENSION DUMSTR(20)
C
C     INITIALIZE
C
      CALL INTSET (GAMB, TAUB, NSUM, IFLG, NCYC, NCALC, LIQ,
     1             GAM, TAU, SIGV, SIGV0, EVP, DYMPP, MXTAU,
     2             MXGAM, KGTYP, GMAX, GMAX0, GPARAM, KTTYP,
     3             TMAX, TMAX0, TPARAM, ITYP, SPARAM,
     4             JTYP, PPARAM, SIGV1)
      IHALF = 0
      IPHASE = 0
      GAMOR = 0.
      IF (IFILE .EQ. 1 .OR. IFILE .EQ. 2) THEN
        WRITE(10,7010)
 7010 FORMAT('      NCYC','     NCALC', '     GAMMA','       TAU',
     1       '      SIGV','     Evp  ', '     PWP  ','     RATIO')
        WRITE(10,7000) 0, 0, 0., 0., SIGV0, 0., 0., 0.
      ENDIF
      IF (IPRNT .GT.0) WRITE(6,6010)
C
  100 NCALC = NCALC + 1
      IF (NCALC .GT. NEND) GO TO 2000
      TAUR   = TAURET(NCALC)
      IF (TAUR .EQ. TAUB) GO TO 1000
C
      TAUDIF = TAUR - TAUB
      IF (TAUINC) 120, 130, 140
  120 DTAU   = TAUDIF / (-TAUINC)
      GO TO 150
  130 DTAU   = TAUDIF * 2.0
      GO TO 150
  140 DTAU   = TAUINC * TAUDIF / ABS(TAUDIF)
  150 CONTINUE
C
      TAU000 = TAUB
C
C     Start of increment computation
C
  200 TAU    = TAU + DTAU
      IF (TAUDIF * (TAU - TAUR) .LE. 0.) GO TO 210
      TAU    = TAUR
C
C     Compute volumetric strain, excess p.w.p.
C
  210 CONTINUE
      R000 = (TAU - TAU000) / TAUDIF
      GO TO (300, 400, 500, 300, 500), JTYP
C
C     Martin-Finn-Seed model and Byrne model
  300 STMC = PPARAM(8)
      EVPMX = PPARAM(9)
      EVPB = EVP
      DYMPPB = DYMPP
      NSUMB = NSUM
C         Iteration
      TAUBB = TAUB
      DO 320 I = 1, 20
      CALL MSGGAM (ITYP, TAU, GAM, GMAX, TMAX, NSUM, TAUBB, GAMB, RET,
     1              NMAX, SPARAM, IFLG)
      IF (IFLG .EQ. 1) GO TO 2100
      IF (NSUB .EQ. 0) GO TO 390
      CALL MAFISE (GAM, GAMB, DYMPP, EVP, NSUB, SIGV0, PPARAM, ATOM,
     1              LIQ, JTYP)
      IF (LIQ .EQ. 1) GO TO 2200
      IF (DABS(DYMPP - DYMPPB) .LE. ATOM/10000.) GO TO 390
C     pore pressure generate
      IF (I .EQ. 1) GO TO 310
      IF (DABS(GAM - GAMTRY) .LE. 1.D-6) GO TO 390
  310 GAMTRY = GAM
      SIGV = SIGV0 - DYMPP
      CALL GMXCAL (KGTYP, GMAX, GMAX0, SIGV, SIGV0, EVP, GPARAM)
      CALL TMXCAL (KTTYP, TMAX, TMAX0, SIGV, SIGV0, EVP, TPARAM)
      NSUM = NSUMB
      PPARAM(8) = STMC
      PPARAM(9) = EVPMX
      EVP = EVPB
      DYMPP = DYMPPB
      CALL FOLLOW (ITYP, GMAX, TMAX, NSUM, RET, SPARAM, GAMB, TAUBB)
  320 CONTINUE
  390 CONTINUE
      GO TO 1000
C
C     Ishihara-Towhata model
  400 CONTINUE
      TAUBB = TAUB
      IF (NSUB .EQ. 0) GO TO 410
      CALL       ISHTOW(TAU, TAUB, DYMPP, NSUB, SIGV1, PPARAM, LIQ,
     1                    SIGV, SIGV0, IPHASE, TAUPHA, SVPHA)
      IF (ILIQ .EQ. 0 .AND. LIQ .EQ. 1) GO TO 2200
      CALL GAMORC (IPHASE, TAUPHA, SVPHA, KGTYP, GMAX0, SIGV0,
     1                    EVP, GPARAM, KTTYP, TMAX0, TPARAM, ITYP,
     2                    GAMOR, NSUM, RET, SPARAM, GAMB, TAUBB, NMAX,
     3                    PPARAM)
      CALL GMXCAL (KGTYP, GMAX, GMAX0, SIGV, SIGV0, EVP, GPARAM)
      CALL TMXCAL (KTTYP, TMAX, TMAX0, SIGV, SIGV0, EVP, TPARAM)
      CALL FOLLOW (ITYP, GMAX, TMAX, NSUM, RET, SPARAM, GAMB, TAUBB)
  410 CALL MSGGAM (ITYP, TAU, GAM, GMAX, TMAX, NSUM, TAUBB, GAMB, RET,
     1              NMAX, SPARAM, IFLG)
      IF (IFLG .EQ. 1) GO TO 2100
      IF (PPARAM(7) .GE. 0.D0) LOC = PPARAM(7) + 0.1
      IF (PPARAM(7) .LT. 0.D0) LOC = PPARAM(7) - 0.1
      IF (ILIQ .EQ. 2 .AND. IABS(LOC) .GE. 2) GO TO 2400
      IF (ILIQ .NE. 1) GO TO 1000
      IF (LIQ .EQ. 1 .AND. DABS(GAM+GAMOR) .LE. GAMLIQ) GO TO 2500
      GO TO 1000
C
C     Martin-Finn-Seed model and Byrne model (Modified)
  500 CONTINUE
      DO 501 I=10, 20
        DUMSTR(I) = PPARAM(I)
  501 CONTINUE
      EVPB = EVP
      DYMPPB = DYMPP
      NSUMB = NSUM
C         Iteration
      TAUBB = TAUB
      DO 520 I = 1, 20
        if(dabs(tau) .ge. tmax) tmax = dabs(tau)*1.02
      CALL MSGGAM (ITYP, TAU, GAM, GMAX, TMAX, NSUM, TAUBB, GAMB, RET,
     1              NMAX, SPARAM, IFLG)
      IF (IFLG .EQ. 1) GO TO 2100
      IF (NSUB .EQ. 0) GO TO 590
      SIGV = SIGV0 - DYMPP
      CALL MAFMOD (GAM, TAU, GAMB, TAUB, DYMPP, EVP, NSUB, SIGV, SIGV0,
     1              PPARAM, ATOM, LIQ, JTYP)
      IF (LIQ .EQ. 1) GO TO 2200
      IF (DABS(DYMPP - DYMPPB) .LE. ATOM/10000.) GO TO 590
C     pore pressure generate
      IF (I .EQ. 1) GO TO 510
      IF (DABS(GAM - GAMTRY) .LE. 1.D-6) GO TO 590
  510 GAMTRY = GAM
      SIGV = SIGV0 - DYMPP
      CALL GMXCAL (KGTYP, GMAX, GMAX0, SIGV, SIGV0, EVP, GPARAM)
      CALL TMXCAL (KTTYP, TMAX, TMAX0, SIGV, SIGV0, EVP, TPARAM)
      NSUM = NSUMB
      DO 511 J = 11, 20
      PPARAM(J) = DUMSTR(J)
  511 CONTINUE
      EVP = EVPB
      DYMPP = DYMPPB
      CALL FOLLOW (ITYP, GMAX, TMAX, NSUM, RET, SPARAM, GAMB, TAUBB)
  520 CONTINUE
  590 CONTINUE
      GO TO 1000
C
 1000 CONTINUE
      IF (TAUB * TAU) 1020, 1010, 1030
 1010 IF (TAUB .EQ. 0. .AND. TAU .EQ. 0.) GO TO 1030
      IF (TAUB .EQ. 0) GO TO 1030
      IHALF = 1
      GO TO 1030
 1020 NCYC = NCYC + 1
 1030 GAMB = GAM
      TAUB = TAU
      SIGV1 = SIGV
      IF (DABS(GAM+GAMOR) .GT. MXGAM) MXGAM = DABS(GAM+GAMOR)
      IF (DABS(TAU) .GT. MXTAU) MXTAU = DABS(TAU)
C
C------------- Print out ------------------
C
      RPP    = DYMPP / SIGV0
      GAMPC = 100. * (GAM+GAMOR)
      EVPPC = 100. * EVP
      IF (IPRNT .EQ. 2) WRITE(6,6000) NCYC, NCALC, GAMPC, TAU,
     1                    SIGV, EVPPC, DYMPP, RPP
      IF (IFILE .EQ. 2) WRITE(10,7000) NCYC, NCALC, GAMPC, TAU,
     1                    SIGV, EVPPC, DYMPP, RPP
C
      IF (ILIQ .EQ. 1 .AND. DABS(GAM+GAMOR) .GE. GAMLIQ) GO TO 2300
      IF (TAU .EQ. TAUR) GO TO 1040
C
      IF (IHALF .EQ. 1) NCYC = NCYC + 1
      IHALF = 0
C
C     Here is the assigned stress
C
 1040 IF (IPRNT .EQ. 1) WRITE(6,6000) NCYC, NCALC, GAMPC, TAU,
     1                    SIGV, EVPPC, DYMPP, RPP
      IF (IFILE .EQ. 1) WRITE(10,7000) NCYC, NCALC, GAMPC, TAU,
     1                    SIGV, EVPPC, DYMPP, RPP
C
      IF (IHALF .EQ. 1) NCYC = NCYC + 1
      IHALF = 0
      IF (TAUR .EQ. 0.D0 .AND. TAU .EQ. 0.D0) GO TO 100
      IF (TAUR .NE. 0.D0 .AND. DABS((TAU-TAUR)/TAUR) .LE. 1.D-7)
     1             GO TO 100
      GO TO 200
C
 2000 CONTINUE
      WRITE(6,6100)
      GO TO 3000
 2100 WRITE(6,6110)
      GO TO 3000
 2200 WRITE(6,6120)
      GO TO 3000
 2300 GG = GAMLIQ * 100.
      WRITE(6,6130) GG
      GO TO 3000
 2400 WRITE(6,6140)
      GO TO 3000
 2500 WRITE(6,6150)
 3000 CONTINUE
      MXGAM = MXGAM * 100.
      CYCL = (NCYC - 1. + R000) / 2.
      WRITE(6,6200) NCYC, CYCL, NCALC, MXGAM, MXTAU
      WRITE(11,6210) CYCL, TTT
      RETURN
C
 6000 FORMAT (1X, 2I8, F10.4, F10.3, F10.3, F10.3, F10.3, F10.3)
 6010 FORMAT (1H0,/,1X,80(1H-),/,
     1 '     HALF    LOAD    STRAIN    STRESS      SIGV',
     2 '     EVP     P.W.P.    P.W.P.',/,
     1 '    CYCLE    STEP     (%)                      ',
     2 '     (%)               RATIO ',/,1X,80(1H-))
 6100 FORMAT(' End of loading')
 6110 FORMAT(' Sample fails because tau .gt. taumax')
 6120 FORMAT(' Sample liquefy completely')
 6130 FORMAT(' Sample liquefy because strain >',F8.3,' %')
 6140 FORMAT(' Initial liquefaction takes place')
 6150 FORMAT(' Sample liquefy completely before specified strain')
 6200 FORMAT(10X, I6,' half cycles    ',
     1 F11.3,' cycles    ',I8,' loading steps',/,
     1   13X, 'max. strain=',F9.4,' %     max. stress',F10.4)
 6210 FORMAT(F15.5, F10.5)
 7000 FORMAT(2I10,1P6E10.3)
C
      END
C=======================================================================
      SUBROUTINE GAMCON (ITYP, SPARAM, RET, NMAX, JTYP, PPARAM,
     1                    NSUB, KGTYP, GPARAM, KTTYP, TPARAM, GMAX0,
     2                    TMAX0, GAMRET, GAMINC, NEND, IPRNT, IFILE,
     3                    ILIQ, ATOM, SIGV0)
C=======================================================================
C     This SUBROUTINE compute under stress control.
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION SPARAM(1), RET(2,1), PPARAM(1), GPARAM(1),
     1          TPARAM(1), GAMRET(1), DUMMY(20)
      REAL*8 MXGAM, MXTAU
C
C     INITIALIZE
C
      CALL       INTSET (GAMB, TAUB, NSUM, IFLG, NCYC, NCALC, LIQ,
     1                    GAM, TAU, SIGV, SIGV0, EVP, DYMPP, MXTAU,
     2                    MXGAM, KGTYP, GMAX, GMAX0, GPARAM, KTTYP,
     3                    TMAX, TMAX0, TPARAM, ITYP, SPARAM,
     4                    JTYP, PPARAM, SIGV1)
      IHALF = 0
      IF (IFILE .EQ. 1 .OR. IFILE .EQ. 2) WRITE(10,7000) 0, 0, 0.,
     1                 0., SIGV0, 0., 0., 0.
      IF (IPRNT .GT.0) WRITE(6,6010)
C
  100 NCALC  = NCALC + 1
      IF (NCALC .GT. NEND) GO TO 2000
      GAMR   = GAMRET(NCALC)
      IF (GAMR .EQ. GAMB) GO TO 1000
C
      GAMDIF = GAMR - GAMB
      IF (GAMINC) 120, 130, 140
  120 DGAM   = GAMDIF / (-GAMINC)
      GO TO 150
  130 DGAM   = GAMDIF * 2.0
      GO TO 150
  140 DGAM   = GAMINC * GAMDIF / ABS(GAMDIF)
  150 CONTINUE
      GAM000 = GAMB
C
C     Start of increment computation
C
  200 GAM    = GAM + DGAM
      IF (GAMDIF * (GAM - GAMR) .LE. 0.) GO TO 210
      GAM    = GAMR
      R000 = (GAM - GAM000) / GAMDIF
C
C     Compute volumetric strain, excess p.w.p.
C
  210 GO TO (300, 400, 220, 300, 220), JTYP
C
  220 write(6,*) ' Cyclic mobility is not allowed for JTYP=3, 5'
      stop
C
C     Martin-Finn-Seed model
  300 CONTINUE
      TAUBB = TAUB
      IF (NSUB .EQ. 0) GO TO 310
      DYMPPB = DYMPP
      CALL MAFISE (GAM, GAMB, DYMPP, EVP, NSUB, SIGV0, PPARAM, ATOM,
     1              LIQ)
      IF (LIQ .EQ. 1) GO TO 2200
      IF (DYMPP .EQ. DYMPPB) GO TO 310
      CALL GMXCAL (KGTYP, GMAX, GMAX0, SIGV, SIGV0, EVP, GPARAM)
      CALL TMXCAL (KTTYP, TMAX, TMAX0, SIGV, SIGV0, EVP, TPARAM)
      CALL FOLLOW (ITYP, GMAX, TMAX, NSUM, RET, SPARAM, GAMB, TAUBB)
  310 CALL MSGTAU (ITYP, TAU, GAM, GMAX, TMAX, NSUM, TAUBB, GAMB, RET,
     1              NMAX, SPARAM)
      GO TO 1000
C
C       Ishihara-towhata model
C
  400 DO 410 J = 1, 11
      DUMMY(J) = PPARAM(J)
  410 CONTINUE
      SIGVB = SIGV
      DYMPPB = DYMPP
      NSUMB = NSUM
      TAUBB = TAUB
C
      DO 450 I = 1, 20
      CALL MSGTAU (ITYP, TAU, GAM, GMAX, TMAX, NSUM, TAUBB, GAMB, RET,
     1              NMAX, SPARAM)
      IF (NSUB .EQ. 0) GO TO 490
      IF (I .EQ. 1) GO TO 420
      IF (DABS(TAUTRY - TAU)/TMAX .LE. 0.00001) GO TO 490
  420 CALL       ISHTOW(TAU, TAUB, DYMPP, NSUB, SIGV1, PPARAM, LIQ,
     1                   SIGV, SIGV0, IPHASE, TAUPHA , SVPHA)
      IF (DABS(DYMPP - DYMPPB) .LE. ATOM/10000.) GO TO 490
C     pore pressure generates
      TAUTRY = TAU
      CALL GMXCAL (KGTYP, GMAX, GMAX0, SIGV, SIGV0, EVP, GPARAM)
      CALL TMXCAL (KTTYP, TMAX, TMAX0, SIGV, SIGV0, EVP, TPARAM)
      DO 430 J = 1, 11
      PPARAM(J) = DUMMY(J)
  430 CONTINUE
      SIGV = SIGVB
      DYMPP = DYMPPB
      NSUM = NSUMB
      CALL FOLLOW (ITYP, GMAX, TMAX, NSUM, RET, SPARAM, GAMB, TAUBB)
  450 CONTINUE
  490 CONTINUE
      IF (ILIQ .EQ. 0 .AND. LIQ .EQ. 1) GO TO 2200
      IF (ILIQ .EQ. 3 .AND.
     1 IABS(IDINT(PPARAM(7)+DSIGN(0.1D0,PPARAM(7)))) .GE. 2) GO TO 2240
      GO TO 1000
C
 1000 IF (GAMB * GAM) 1020, 1010, 1030
 1010 IF (GAMB .EQ. 0.D0 .AND. GAM .EQ. 0.D0) GO TO 1030
      IF (GAMB .EQ. 0.D0) GO TO 1030
      IHALF = 1
      GO TO 1030
 1020 NCYC = NCYC + 1
C
 1030 GAMB = GAM
      TAUB = TAU
      SIGV1 = SIGV
      IF (DABS(GAM) .GT. MXGAM) MXGAM = DABS(GAM)
      IF (DABS(TAU) .GT. MXTAU) MXTAU = DABS(TAU)
C
C------------- Print out ------------------
C
      RPP    = DYMPP / SIGV0
      GAMPC = 100. * GAM
      EVPPC = 100. * EVP
      IF (IPRNT .EQ. 2) WRITE(6,6000) NCYC, NCALC, GAMPC, TAU,
     1                    SIGV, EVPPC, DYMPP, RPP
      IF (IFILE .EQ. 2) WRITE(10,7000) NCYC, NCALC, GAMPC, TAU,
     1                    SIGV, EVPPC, DYMPP, RPP
C
      IF (GAM .EQ. GAMR) GO TO 1040
      IF (IHALF .EQ. 1) NCYC = NCYC + 1
      IHALF = 0
      GO TO 200
C
C     Here is the assigned strain
C
 1040 IF (IPRNT .EQ. 1) WRITE(6,6000) NCYC, NCALC, GAMPC, TAU,
     1                    SIGV, EVPPC, DYMPP, RPP
      IF (IFILE .EQ. 1) WRITE(10,7000) NCYC, NCALC, GAMPC, TAU,
     1                    SIGV, EVPPC, DYMPP, RPP
C
      IF (IHALF .EQ. 1) NCYC = NCYC + 1
      IHALF = 0
      IF (GAMR .EQ. 0.D0 .AND. GAM .LE. 1.D-10) GO TO 100
      IF (GAMR .NE. 0.D0 .AND. DABS((GAM-GAMR)/GAMR) .LE. 1.D-7)
     1          GO TO 100
      GO TO 200
C
 2000 CONTINUE
      WRITE(6,6100)
      GO TO 3000
 2100 WRITE(6,6110)
      GO TO 3000
 2200 WRITE(6,6120)
      GO TO 3000
 2240 WRITE(6,6140)
 3000 CONTINUE
      MXGAM = MXGAM * 100.
      WRITE(6,6200) NCYC, R000, NCALC, MXGAM, MXTAU
      RETURN
C
 6000 FORMAT (1X, 2I8, F10.4, F10.3, F10.3, F10.3, F10.3, F10.3)
 6010 FORMAT (1H0,/,1X,80(1H-),/,
     1 '     HALF    LOAD    STRAIN    STRESS      SIGV',
     2 '     EVP     P.W.P.    P.W.P.',/,
     1 '    CYCLE    STEP     (%)                      ',
     2 '     (%)               RATIO ',/,1X,80(1H-))
 6100 FORMAT(' End of loading')
 6110 FORMAT(' Sample fails because tau .gt. taumax')
 6120 FORMAT(' Sample liquefied completely')
 6140 FORMAT(' Initial liquefaction takes place')
 6200 FORMAT(10X, I6,' half cycles    ',
     1     F8.4,' Process    ',I8,' loading steps',/,
     1   13X, 'max. strain=',F9.4,' %     max. stress',F10.4)
 7000 FORMAT(2I10,1P6E10.3)
C
      END
C=======================================================================
      SUBROUTINE INTSET (GAMB, TAUB, NSUM, IFLG, NCYC, NCALC, LIQ,
     1                    GAM, TAU, SIGV, SIGV0, EVP, DYMPP, MXTAU,
     2                    MXGAM, KGTYP, GMAX, GMAX0, GPARAM, KTTYP,
     3                    TMAX, TMAX0, TPARAM, ITYP, SPARAM,
     4                    JTYP, PPARAM, SIGV1)
C=======================================================================
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION SPARAM(1), GPARAM(1), TPARAM(1), PPARAM(1)
      REAL*8 MXGAM, MXTAU
C
C     INITIALIZE
C
      GAMB   = 0.
      TAUB   = 0.
      NSUM = 0
      IFLG = 0
      NCYC   = 1
      NCALC  = 0
      LIQ = 0
      GAM    = 0.
      TAU    = 0.
      SIGV   = SIGV0
      SIGV1   = SIGV0
      EVP    = 0.
      DYMPP  = 0.
      MXTAU = 0.
      MXGAM = 0.
      CALL GMXCAL (KGTYP, GMAX, GMAX0, SIGV, SIGV0, EVP, GPARAM)
      CALL TMXCAL (KTTYP, TMAX, TMAX0, SIGV, SIGV0, EVP, TPARAM)
  100 GO TO (200, 200, 110, 120, 130, 110), ITYP
C
  110 SPARAM(1) = SPARAM(5) / TMAX * GMAX - 1.D0
      SPARAM(3) = TMAX
      GO TO 200
C
  120 SPARAM(4) = GMAX
      SPARAM(5) = TMAX
      GO TO 200
C
  130 SPARAM(1) = SPARAM(5)
      SPARAM(2) = SPARAM(6)
C
  200 GO TO (210, 220, 230, 210, 230), JTYP
C     Martin-Finn-Seed model and Byrne model
  210 PPARAM(8) = 0.
      PPARAM(9) = 0.
      GO TO 300
C     Ishihara-Towhata model
  220 PPARAM(1) = PPARAM(12)
      PPARAM(2) = PPARAM(13) / 2. / SIGV0**2
      PPARAM(3) = PPARAM(14) * SIGV0
      PPARAM(4) = DTAN(PPARAM(15) * DATAN(1.D0) / 45.)
      PPARAM(5) = DTAN(PPARAM(16) * DATAN(1.D0) / 45.)
      PPARAM(6) = 0.D0
      PPARAM(7) = 0.D0
      PPARAM(8) = 0.D0
      PPARAM(9) = 0.D0
      PPARAM(10) = 0.D0
      PPARAM(11) = 0.D0
      GO TO 300
C     Martin-Finn-Seed model and Byrne model (Modified)
  230 PPARAM(8) = DTAN(PPARAM(8) * DATAN(1.D0) / 45.)
      PPARAM(9) = DTAN(PPARAM(9) * DATAN(1.D0) / 45.)*0.95
      DO 231 I = 10, 16
        PPARAM(I) = 0.D0
  231 CONTINUE
      IF (PPARAM(17) .LE. 0.D0) PPARAM(17) = 0.01
      GO TO 300
C
  300 CONTINUE
      RETURN
      END
C=======================================================================
      SUBROUTINE GMXCAL (KGTYP, GMAX, GMAX0, SIGV, SIGV0, EVP, GPARAM)
C=======================================================================
C
C     This SUBROUTINE computes shear modulus at small strain
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION GPARAM(1)
C
   10 GO TO (100, 200, 300, 400, 500), KGTYP
  100 GMAX = GMAX0 * (SIGV / SIGV0) ** GPARAM(1)
      GO TO 1000
  200 IF (GPARAM(2) .EQ. 0. .AND. GPARAM(3) .EQ. 0.) GO TO 100
      GMAX = GMAX0 * (1. + EVP / (GPARAM(2) + GPARAM(3) * EVP))
     1             * (SIGV / SIGV0) ** GPARAM(1)
      GO TO 1000
  300 GMAX0 = GPARAM(2) * SIGV**GPARAM(1)
      GO TO 1000
  400 GMAX0 = GPARAM(2)*(GPARAM(3)-GPARAM(5))**GPARAM(4)
     1      / (1.+GPARAM(5)) * SIGV**GPARAM(1)
      GO TO 1000
  500 GMAX0 = GPARAM(2) * GPARAM(3) * (SIGV/GPARAM(3))**GPARAM(1)
      GO TO 1000
C
 1000 CONTINUE
      RETURN
      END
C=======================================================================
      SUBROUTINE TMXCAL (KTTYP, TMAX, TMAX0, SIGV, SIGV0, EVP, TPARAM)
C=======================================================================
C
C     This SUBROUTINE computes shear strength at failure
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION TPARAM(1)
C
   10 GO TO (100, 200, 300), KTTYP
C
  100 TMAX = TMAX0 * (SIGV / SIGV0) ** TPARAM(1)
      GO TO 1000
C
  200 IF (TPARAM(2) .EQ. 0. .AND. TPARAM(3) .EQ. 0.) GO TO 100
      TMAX = TMAX0 * (1. + EVP / (TPARAM(2) + TPARAM(3) * EVP))
     1             * (SIGV / SIGV0) ** TPARAM(1)
      GO TO 1000
C
  300 TMAX = TPARAM(1) + SIGV * TPARAM(2)
      GO TO 1000
 1000 CONTINUE
      RETURN
      END
C=======================================================================
      SUBROUTINE MSGTAU (ITYP , TAU , GAM , GMAX , TMAX , NSUM ,
     1                    TAUB , GAMB , RET , NMAX , SPARAM)
C=======================================================================
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION RET(2,NMAX) , SPARAM(1)
      DATA IERR / 0 /
C
C     NMAX   : maximum allowable number for unloading
C     TAU    must be trial value when ITYP=2
C
  100 IF (NSUM .NE. 0) GO TO 200
C
C     Previous point was on the skeleton curve
C
      IF (GAMB .GE. 0.) GO TO 110
      IF (GAM .LE. GAMB) GO TO 400
C                                     to the skeleton curve
      IF (GAM .GE. -GAMB) GO TO 400
C                                     to the skeleton curve
      GO TO 120
C
  110 IF (GAM .GE. GAMB) GO TO 400
C                                     to the skeleton curve
      IF (GAM .LE. -GAMB) GO TO 400
C                                     to the skeleton curve
C
C     Here strain reversal takes place
  120 NSUM   = 1
      RET(1,1) = GAMB
      RET(2,1) = TAUB
      CALL REVPAR(ITYP, GMAX, TMAX, SPARAM, GAMB, TAUB)
      GO TO 420
C
  200 IF (NSUM .NE. 1) GO TO 300
C
C     Previous point was on the first hysteresis curve
C
  210 IF (RET(1,1) .GE. 0.) GO TO 220
      IF (GAM .GE. -RET(1,1)) GO TO 400
      IF (GAM .LE.  RET(1,1)) GO TO 400
C                                          to skeleton curve
      GO TO 420
C                                          to hysteresis curve
  220 IF (GAM .LE. -RET(1,1)) GO TO 400
      IF (GAM .GE.  RET(1,1)) GO TO 400
C                                          to skeleton curve
      GO TO 420
C                                          to hysteresis curve
C
C     Previous point was on the hysteresis curve
C
  300 IF ((GAMB-RET(1,NSUM))*(GAM-GAMB) .GE. 0.) GO TO 310
C
C     Unloading takes place here
C
      IF (NSUM .LT. NMAX) GO TO 305
      IERR = IERR + 1
      IF (IERR .EQ. 1) WRITE (6, 600)
      NSUM = NMAX - 2
  305 NSUM   = NSUM + 1
      RET(1,NSUM) = GAMB
      RET(2,NSUM) = TAUB
C
C     Check to switch to previous hysteresis curve or skeleton curve
C
  310 IF ((RET(1,NSUM)-RET(1,NSUM-1))*(GAM-RET(1,NSUM-1))
     1     .LE. 0.) GO TO 420
C                                          to hysteresis curve
      NSUM   = NSUM - 2
      IF (NSUM .EQ. 0) GO TO 400
C                                          to skeleton curve
      IF (NSUM .EQ. 1) GO TO 210
      GO TO 310
C
C     On the SKELETON curve
C
  400 NSUM   = 0
      CALL TAUC (ITYP , TAU , GAM , GMAX , TMAX , SPARAM)
      GO TO 440
C     On the HYSTERESIS curve
C
  420 GAMMA  = (GAM - RET(1,NSUM)) / 2.
      TAU    = (TAU - RET(2,NSUM)) / 2.
      CALL TAUC (ITYP , STRESS , GAMMA , GMAX , TMAX , SPARAM)
      TAU    = RET(2,NSUM) + STRESS * 2.0
C
  440 CONTINUE
      RETURN
C
  600 FORMAT('0====WARNING==== storage for strain reversal points',
     1       ' becomes full')
      END
C=======================================================================
      SUBROUTINE MSGGAM (ITYP , TAU , GAM , GMAX , TMAX , NSUM ,
     1                    TAUB , GAMB , RET , NMAX , SPARAM , IFLG)
C=======================================================================
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION RET(2,NMAX) , SPARAM(1)
      DATA IERR / 0 /
C
C     NMAX   : maximum allowable number for unloading
C
      IFLG = 0
      IF (DABS(TAU) .GT. TMAX) IFLG = 1
      IF (IFLG .EQ. 1) RETURN
  100 IF (NSUM .NE. 0) GO TO 200
C
C     Previous point was on the skeleton curve
C
      IF (TAUB .GE. 0.) GO TO 110
      IF (TAU .LE. TAUB) GO TO 400
C                                     to the skeleton curve
      IF (TAU .GE. -TAUB) GO TO 400
C                                     to the skeleton curve
      GO TO 120
C
  110 IF (TAU .GE. TAUB) GO TO 400
C                                     to the skeleton curve
      IF (TAU .LE. -TAUB) GO TO 400
C                                     to the skeleton curve
C
C     Here strain reversal takes place.
  120 NSUM   = 1
      RET(1,1) = GAMB
      RET(2,1) = TAUB
      CALL REVPAR(ITYP, GMAX, TMAX, SPARAM, GAMB, TAUB)
      GO TO 420
C
  200 IF (NSUM .NE. 1) GO TO 300
C
C     Previous point was on the first hysteresis curve
C
  210 IF (RET(2,1) .GE. 0.) GO TO 220
      IF (TAU .GE. -RET(2,1)) GO TO 400
      IF (TAU .LE.  RET(2,1)) GO TO 400
C                                          to skeleton curve
      GO TO 420
C                                          to hysteresis curve
  220 IF (TAU .LE. -RET(2,1)) GO TO 400
      IF (TAU .GE.  RET(2,1)) GO TO 400
C                                          to skeleton curve
      GO TO 420
C                                          to hysteresis curve
C
C     Previous point was on the hysteresis curve
C
  300 IF ((TAUB-RET(2,NSUM))*(TAU-TAUB) .GE. 0.) GO TO 310
C
C     Unloading takes place here
C
      IF (NSUM .LT. NMAX) GO TO 305
      IERR = IERR + 1
      IF (IERR .EQ. 1) WRITE (6, 600)
      NSUM = NMAX - 2
  305 NSUM   = NSUM + 1
      RET(1,NSUM) = GAMB
      RET(2,NSUM) = TAUB
C
C     Check to switch to previous hysteresis curve or skeleton curve
C
  310 IF ((RET(2,NSUM)-RET(2,NSUM-1))*(TAU-RET(2,NSUM-1))
     1     .LE. 0.) GO TO 420
C                                          to hysteresis curve
      NSUM   = NSUM - 2
      IF (NSUM .EQ. 0) GO TO 400
C                                          to skeleton curve
      IF (NSUM .EQ. 1) GO TO 210
      GO TO 310
C
C     On the SKELETON curve
C
  400 NSUM   = 0
      CALL GAMC (ITYP , TAU , GAM , GMAX , TMAX , SPARAM)
      GO TO 440
C     On the HYSTERESIS curve
C
  420 GAMMA  = (GAM - RET(1,NSUM)) / 2.
      STRESS = (TAU - RET(2,NSUM)) / 2.
      CALL GAMC (ITYP , STRESS , GAMMA , GMAX , TMAX , SPARAM)
      GAM = RET(1,NSUM) + 2. * GAMMA
C
  440 CONTINUE
      RETURN
C
  600 FORMAT('0====WARNING==== storage for strain reversal points',
     1       ' becomes full')
      END
C=======================================================================
      SUBROUTINE FOLLOW (ITYP, GMAX, TMAX, NSUM, RET, SPARAM, GAMB,
     1                    TAUB)
C=======================================================================
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION SPARAM(1), RET(2,1)
C
C     Modify the parameter acording to the new GMAX and TMAX
C
   10 GO TO (50, 50, 20, 30,50, 20), ITYP
   20 SPARAM(3) = TMAX
      SPARAM(1) = SPARAM(5) * GMAX / TMAX - 1
      GO TO 50
   30 SPARAM(5) = GMAX
      SPARAM(6) = TMAX
      GO TO 50
C
   50 IF (NSUM .EQ. 0) GO TO 200
C
      DO 100 I = 1 , NSUM
      IF (I .GT. 1) GO TO 90
      CALL TAUC (ITYP, RET(2,1), RET(1,1), GMAX, TMAX, SPARAM)
      IF (ITYP .LE. 3 .OR .ITYP .GE. 7) GO TO 100
      CALL       REVPAR(ITYP, GMAX, TMAX, SPARAM, GAM, TAU)
      GO TO 100
   90 GAM = (RET(1,I) - RET(1,I-1)) / 2.
      CALL TAUC (ITYP, TAU, GAM, GMAX, TMAX, SPARAM)
      RET(2,I) = RET(2,I-1) + TAU * 2.
  100 CONTINUE
      GAM = (GAMB - RET(1,NSUM)) / 2.
      CALL TAUC (ITYP, TAUB, GAM, GMAX, TMAX, SPARAM)
      TAUB = RET(2,NSUM) + TAUB * 2.
      RETURN
  200 CONTINUE
      CALL TAUC (ITYP, TAUB, GAMB, GMAX, TMAX, SPARAM)
      RETURN
      END
C=======================================================================
      SUBROUTINE TAUC (ITYP , TAU , GAM , GMAX , TMAX , SPARAM)
C     SUBROUTINE GAMC (ITYP , TAU , GAM , GMAX , TMAX , SPARAM)
C=======================================================================
C
C     This SUBROUTINE compute stress TAU from given strain GAM
C
C     ITYP   : 1 = hyperbolic model
C              2 = Ramberg-Osgood model
C     TAU    : stress
C              If ITYP=2, TAU must be a trial value
C     GAM    : strain
C     GMAX   : modulus at small strain
C     TMAX   : strength
C     SPARAM    : parameters
C                  ITYP           1            1
C                  SPARAM(1)                   alpha
C                  SPARAM(2)                   r
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION SPARAM(1)
C
      GAMMA  = DABS(GAM)
      GO TO (100 , 200, 200, 300, 200, 200), ITYP
C
C     HYPERBOLIC MODEL
C
  100 TAU    = GMAX * GAMMA / (1. + GMAX * GAMMA / TMAX)
      GO TO 900
C
C     RAMBERG-OSGOOD MODEL
C
  200 GAMMA  = GAMMA * GMAX / SPARAM(3)
      TAU    = DABS(TAU) / SPARAM(3)
      IF (TAU .GE. TMAX) TAU = 0.
      DO 210 I = 1 , 50
      GG     = TAU + SPARAM(1) * TAU ** SPARAM(2)
      IF (DABS(GG - GAMMA) .LE. 1.D-6) GO TO 220
      TAU    = TAU + (GAMMA - GG) /
     1        (1. + SPARAM(1) * SPARAM(2) * TAU ** (SPARAM(2) - 1.))
  210 CONTINUE
  220 TAU    = TAU * SPARAM(3)
      GO TO 900
C
C     HYPERBOLIC MODEL (IMPROVED HISTERESIS RULE)
C
  300 TAU = SPARAM(5) * GAMMA / (1. + SPARAM(5) * GAMMA / SPARAM(6))
      GO TO 900
  900 IF (GAM .LT. 0.) TAU = -TAU
      RETURN
C
C
C     Following ENTRY computes strain GAM from given stress TAU
C
      ENTRY      GAMC (ITYP , TAU , GAM , GMAX , TMAX , SPARAM)
C
 1000 STRESS = DABS(TAU)
C
      GO TO (1100, 1200, 1200, 1300, 1200, 1200), ITYP
C
C     HYPERBOLIC MODEL
C
 1100 GAM    = STRESS / GMAX / (1. - STRESS / TMAX)
      GO TO 1900
C
C     RAMBERG-OSGOOD MODEL
C
 1200 GAM    = STRESS / GMAX * (1. + SPARAM(1) *
     1           (STRESS / SPARAM(3)) ** (SPARAM(2) - 1))
      GO TO 1900
C
C     HYPERBOLIC MODEL
C
 1300 GAM    = STRESS / SPARAM(5) / (1. - STRESS / SPARAM(6))
      GO TO 1900
C
 1900 IF (TAU .LT. 0.) GAM = -GAM
      RETURN
      END
C=======================================================================
      SUBROUTINE REVPAR(ITYP, GMAX, TMAX, SPARAM, GAM, TAU)
C=======================================================================
C
C     This subroutine computes new values of the parameter
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION SPARAM(1)
C
   10 IF (ITYP .LE. 3 .OR. ITYP .GE. 7) RETURN
C
C     Compute new damping ratio
C
      MM = SPARAM(4) + 0.1
      GAMMA = DABS (GAM)
      STRESS = DABS (TAU)
      GSEC = STRESS / GAMMA
      IF (MM .EQ. 1) D = SPARAM(7) * (1. - GSEC / GMAX)
      IF (MM .EQ.  1) GO TO 90
C
      IF (GAM .GT. SPARAM(8)) GO TO 20
      D = SPARAM(7)
      GO TO 90
   20 DO 30 I = 2, MM
      IF (GAMMA .LE. SPARAM(2*I+6)) GO TO 40
   30 CONTINUE
      D = SPARAM(2*MM+5)
      GO TO 90
   40 II = 2*I + 4
      D = (SPARAM(II+3) - SPARAM(II+1)) / (DLOG(SPARAM(II+4))
     1   - DLOG(SPARAM(II+2))) * (DLOG(GAMMA) - DLOG(SPARAM(II+2)))
     2     + SPARAM(II+1)
C
   90 II = ITYP - 3
      GO TO (100, 200, 200), II
C
C     Hyperbolic model
C
  100 D = D * DATAN(1.D0)
      X = GAMMA * GMAX / TMAX
      DO 110 I = 1 , 20
      IF (X .GE. 0.001) D1 = (1.D0+X)*(X-DLOG(1.+X))/X/X-0.5D0
      IF (X .LT. 0.001) D1 = X/6.D0 - X*X/12.D0 + X*X*X/4.D0
      IF (DABS(D - D1) .LE. 0.0001) GO TO 120
      X1 = ((2.D0 + X) * DLOG(1.D0 + X) - 2.D0 * X) / X**3
      XNEW = X + (D - D1) / X1
      IF (DABS(XNEW - X) .LT. 0.0001) GO TO 120
      X = XNEW
  110 CONTINUE
  120 SPARAM(5) = GSEC * (1. + X)
      SPARAM(6) = GAMMA / X * SPARAM(5)
      RETURN
C
C     Ramberg-Osgood model
C
  200 PAI = 4.D0 * DATAN(1.0D0)
      R = (1. + PAI * D / 2. - GSEC / GMAX)
     1  / (1. - PAI * D / 2. - GSEC / GMAX)
      SPARAM(2) = R
      SPARAM(1) = (1. - GSEC / GMAX) / (GSEC/GMAX)**R
     1                 / (GAM * GMAX / TMAX)**(R-1.)
      RETURN
      END
C=======================================================================
      SUBROUTINE MAFISE (GAM, GAMB, DYMPP, EVP, NSUB, SIGV0,
     1                    PPARAM, ATOM, LIQ, JTYP)
C=======================================================================
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION PPARAM(1)
C
C     This SUBROUTINE Calculates residual porepressure and
C     residual volumetric strain due to shear for given shear strain.
C
C     GAM    : current strain
C     GAMB   : strain at revious computation
C     C1,C2,C3,C4,XM,XN,XK2 : parameters
C     NSUB   : flag on submerge
C              0 = not submerged       1 = submeged
C     DYMPP  : cumurated excess p.w.p.
C     EVP    : cumurated volumetric strain
C     SIGV0  : initial effective stress
C     SIGV   : current effective strain
C     STMC   : maximum strain in current half cycle
C     ATOM   : atomospheric pressure
C     LIQ    : flag to lequefaction
C     JTYP   : flag on model
C              1=Martin-Finn-Seed  4=Byrne
C
      C1 = PPARAM(1)
      C2 = PPARAM(2)
      C3 = PPARAM(3)
      C4 = PPARAM(4)
      XK2 = PPARAM(5)
      XM = PPARAM(6)
      XN = PPARAM(7)
      STMC = PPARAM(8)
      EVPMX = PPARAM(9)
C
   10 IF (GAM .EQ. GAMB) GO TO 400
      IF (GAM * GAMB) 300, 200, 100
C
C     Here GAM and GAMB are on the same side.
C
  100 IF (DABS(GAM) .LT. DABS(GAMB)) GO TO 150
C
C     Here loading continues, hence no volumetric strain increment.
C
      IF (DABS(GAM) .LE. STMC) GO TO 400
C
C     Here max. strain increases.
C     Max. volumetric strain increment if a half cycle of loading
C     is completed is computed.
C
      STMC = DABS(GAM)
      IF (JTYP .EQ. 1) THEN
        TEMP = -C2 * EVP + C3 * EVP**2 / (STMC + C4 * EVP)
        EVPMX = C1 * (STMC + TEMP) / 2.
        IF (TEMP .GT. 0.) EVPMX = 0.
      ELSE
        EVPMX = 0.5 * C1 * STMC * DEXP(-C2 * EVP / STMC)
      ENDIF
      GO TO 400
C
C     Here unloading takes place, hence pore pressure generates
C
  150 DGAM = DABS(GAM - GAMB)
      DEVP = DGAM * EVPMX / STMC
      EVP = EVP + DEVP
C
C     Compute pore pressure
C
  160 IF (NSUB .EQ. 0) GO TO 400
C
      TEMP = 1. - (ATOM / SIGV0)** XN * EVP * 100. / XK2
      IF (TEMP .LE. 0.) LIQ = 1
      IF (LIQ .EQ. 1) DYMPP = SIGV0
      IF (LIQ .EQ. 0) DYMPP = SIGV0 * (1. - TEMP**(1./XM))
      GO TO 400
C
C     Here either GAM or GAMB equals to zero.
C
  200 IF (GAM .NE. 0.D0) GO TO 100
C
C     Here GAM and GAMB are on the other side
C
  300 DGAM = DABS(GAMB)
      DEVP = DGAM * EVPMX / STMC
      EVP = EVP + DEVP
C
      STMC = DABS(GAM)
      IF (JTYP .EQ. 1) THEN
        TEMP = -C2 * EVP + C3 * EVP**2 / (STMC + C4 * EVP)
        EVPMX = C1 * (STMC + TEMP) / 2.
        IF (TEMP .GT. 0.) EVPMX = 0.
      ELSE
        EVPMX = 0.5 * C1 * STMC * DEXP(-C2 * EVP / STMC)
      ENDIF
      GO TO 160
C
  400 CONTINUE
      PPARAM(8) = STMC
      PPARAM(9) = EVPMX
      RETURN
      END
C=======================================================================
      SUBROUTINE MAFMOD (GAM, TAU, GAMBB, TAUBB, DYMPP, EVP, NSUB,
     1                   SIGV, SIGV0, PPARAM, ATOM, LIQ, JTYP)
C=======================================================================
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION PPARAM(1)
C
C     This SUBROUTINE Calculates residual porepressure and
C     residual volumetric strain due to shear for given shear strain.
C
C     The original model is Martin-Finn-Seed model, but it is modified
C     So that cyclic mobility can be takin into account. Therefore,
C     the code is quite different with MAFISE
C
C     GAM    : current strain
C     TAU    : current strain
C     GAMB   : strain at previous step
C     TAUB   : stress at previous step
C     C1,C2,C3,C4,XM,XN,XK2 : parameters
C     NSUB   : flag on submerge
C              0 = not submerged       1 = submeged
C     DYMPP  : cumurated excess p.w.p.
C     EVP    : cumurated volumetric strain
C     SIGV0  : initial effective stress
C     SIGV   : effective stress at previous step in input and
C              that at final in output
C     ATOM   : atomospheric pressure
C     LIQ    : flag to lequefaction
C     JTYP   : flag on model
C              1=Martin-Finn-Seed  4=Byrne
C
C     PMIN   : PMIN*SIGV0 is minimum mean stress
C
C     Constants
C
      C1 = PPARAM(1)
      C2 = PPARAM(2)
      C3 = PPARAM(3)
      C4 = PPARAM(4)
      XK2 = PPARAM(5)
      XM = PPARAM(6)
      XN = PPARAM(7)
      PHIS = PPARAM(8)
C                            tangent of phase transform angle
      PHI  = PPARAM(9)
C                            tangent of internal friction angle
C     Temporary parameters
C
      GAM0 = PPARAM(10)
C                           shear strain at EVP is fixed, usually 0
C                           unless loading occurred in the same side
      EVP0 = PPARAM(11)
C                           volumetric strain at previous half cycle
      IFLG = PPARAM(12)
C                           flag on corssing phase transform line
C                           0=no   1=yes
      AP = PPARAM(13)
C                           effective stress of hyperbolla, which is
C                           used after phase transform
      SIGVM = PPARAM(14)
C                           target PWP at unloading
      STMC = PPARAM(15)
C                           Maximum shear strain at current half cycle
      EVPT = PPARAM(16)
C                           cumulated volumetric strain
C                           Note EVP is computed from SIGV
      PMIN = PPARAM(17) * SIGV0
C                           Minimum SIGV
C
      TAUB = TAUBB
      GAMB = GAMBB
   10 IF (GAM .EQ. GAMB) GO TO 400
      IF (GAM * GAMB) 300, 200, 100
C
C-----------------------------------------------------------------------
C     Here GAM and GAMB are on the same side. (Normal state)
C-----------------------------------------------------------------------
C
  100 AGAM = DABS(GAM)
      AGAMB = DABS(GAMB)
      IF (AGAM .LT. AGAMB) GO TO 150
C
C     Here loading continues. Porewater generation is conputed
C
  110 CONTINUE
C     Compute volumetric strain increment
C
      IF (DABS(GAM) .LE. STMC) THEN
        AMP = DABS(GAM - GAM0)
      ELSE
        STMC = AGAM
        AMP = AGAM
      ENDIF
C
      IF (JTYP .EQ. 1) THEN
        TEMP = -C2 * EVP0 + C3 * EVP0**2 / (AMP + C4 * EVP0)
        DEPSV = C1 * (AMP + TEMP) / 2.
        IF (TEMP .GT. 0.) DEPSV = 0.
      ELSE
        DEPSV = 0.5 * C1 * AMP * DEXP(-C2 * EVP0 / AMP)
      ENDIF
      DEPSV = DMAX1(DEPSV, 0.D0)
C     Compute excess pore water pressure
      EPSV = EVP0 + DEPSV
      EVPT = EPSV
      TEMP = 1. - (ATOM / SIGV0)** XN * EPSV * 100. / XK2
      IF (TEMP .LE. 0.D0) TEMP = 0.D0
      U = SIGV0 * (1. - TEMP**(1.D0/XM))
      SIGVM = DMAX1(SIGV0 - U, PMIN)
C
C     Check for crossing phase transform line
C
      IF (IFLG .EQ. 1) THEN
C     Already phase transformed
        SIGV = DSQRT(AP**2 + (TAU/PHI)**2)
      ELSE
C     Not phase transformed
        PP = DABS(TAU) / PHIS
C                                sigv if phase transformed
        IF (SIGVM .GE. PP) THEN
C     Still not phase transformed
          SIGV = SIGVM
        ELSE
C     Just phase transformed
C        The intersection is to be computed. Because of this reason,
C        increment is sufficiently small.
          IF (DTAN(DABS(TAUB)/SIGV) .LT. PHIS) THEN
            IF (SIGVM .EQ. SIGV) THEN
              PP = SIGV
            ELSE
              TEMP = PHIS - (DABS(TAU) - DABS(TAUB)) / (SIGVM - SIGV)
              PP = (SIGVM*DABS(TAUB) - DABS(TAU)*SIGV)
     1             / (SIGVM - SIGV) / TEMP
C                           effective stress at intersection
            ENDIF
            AP = PP * DSQRT(1. - (PHIS / PHI)**2)
C                           m-value of hyperbolla
          ENDIF
          IFLG = 1
          SIGV = DSQRT(AP**2 + (TAU/PHI)**2)
        ENDIF
      ENDIF
      GO TO 400
C
C     Here unloading takes place or contiluous process of unloading
C
  150 EVP = EVPT
      GAM0 = GAM
      IF (IFLG .EQ. 0) THEN
C     Not phase transformed
        SIGV = SIGVM
      ELSE
C     Unload after phase transform
        SIGV = SIGVM + TAU / TAUB * (SIGV - SIGVM)
        SIGV = DMAX1(SIGV, SIGVM)
        IF (TAU * TAUB .LE. 0.D0) IFLG = 0
        TMP = SIGV**2 - (TAU/PHI)**2
        TMP = DMAX1(TMP, (PMIN)**2)
        AP = DSQRT(TMP)
      ENDIF
      GO TO 400
C
C-----------------------------------------------------------------------
C     Either one of strain = 0
C-----------------------------------------------------------------------
C
  200 CONTINUE
      IF (GAM .NE. 0.D0) GO TO 100
      GO TO 300
C
C-----------------------------------------------------------------------
C     Both strain are located in opposite side, therefore first unload
C     then loading
C-----------------------------------------------------------------------
C
  300 CONTINUE
C     First, computation for unload path
      IF (TAUB * TAU .LT. 0.D0) IFLG = 0
      IF (TAUB .EQ. 0.D0) IFLG=0
      IF (IFLG .EQ. 0) THEN
        SIGV = SIGVM
      ELSE
        SIGV = SIGVM + TAU / TAUB * (SIGV - SIGVM)
        SIGV = DMAX1(SIGV, SIGVM)
        IF (TAU * TAUB .LE. 0.D0) IFLG = 0
        TMP = SIGV**2 - (TAU/PHI)**2
        TMP = DMAX1(TMP, (PMIN)**2)
        AP = DSQRT(TMP)
      ENDIF
C     Loading calculation
C     Find stress at which strain=0
      TAUB = -GAMB * (TAU-TAUB) / (GAM - GAMB) + TAUB
      GAMB = 0.D0
      GAM0 = 0.
      EVP0 = EVPT
      STMC = 0.D0
      GO TO 110
C
C-----------------------------------------------------------------------
C     End of calculation
C-----------------------------------------------------------------------
C     In the previous section, we computed effective stress change under
C     undrained condition. We need to compute volume change under
C     drained condition, excess porewater generation, etc.
C
  400 CONTINUE
      SIGV = DMAX1(SIGV, PMIN)
      DYMPP = SIGV0 - SIGV
      EVP = XK2/100.D0 * (1.D0-(1.D0-DYMPP/SIGV0)**XM)/(ATOM/SIGV0)**XN
C
      PPARAM(10) = GAM0
      PPARAM(11) = EVP0
      PPARAM(12) = IFLG + 0.1D0
      PPARAM(13) = AP
      PPARAM(14) = SIGVM
      PPARAM(15) = STMC
      PPARAM(16) = EVPT
      RETURN
 500  CONTINUE
      RETURN
      END
C=======================================================================
      SUBROUTINE ISHTOW(TAU, TAUB, DYMPP, NSUB, SIGV1, PPARAM, LIQ,
     1                   SIGV, SIGV0, IPHASE, TAUPHA, SVPHA)
C=======================================================================
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION PPARAM(1)
      COMMON / PHAILW / IPHI
C
      DATA  SVMIN / 0.01 /
C                 RATIO OF MINIMUM SIGMA-V
C
      IF (NSUB .EQ. 0) RETURN
      BP = PPARAM(1)
      ELW = PPARAM(2)
      AKAP = PPARAM(3)
      PHT = PPARAM(4)
      PHAI = PPARAM(5)
      IF (PPARAM(7) .GE. 0.D0) LOC = PPARAM(7) + 0.1
      IF (PPARAM(7) .LT. 0.D0) LOC = PPARAM(7) - 0.1
      TAUEND = PPARAM(8)
      DIF = PPARAM(9)
      SRAMAX = PPARAM(10)
      SRAMIN = PPARAM(11)
      IOM = 0
      IF (IABS(LOC) - 2)  1, 90, 160
    1 IF (ELW .LE. 0.D0)  GO TO 50
      IF (SIGV1 .LE. AKAP)  GO TO 40
C
C     BEFORE INITIAL LIQUEFACTION WITH ENOUGH LARGE EFFECTIVE VERTICAL
C     STRESS
C
      EL = TAU - TAUB
      IF (DIF .EQ. 0.D0) GO TO 11
      IF (EL * DIF .GE. 0.D0)  GO TO 10
   11 TAUEND = TAUB
      DIF = EL
   10 EL = LOG(SIGV1-AKAP) + ELW * (TAUB
     1   -TAUEND)**2
      SIGV = AKAP + EXP(EL-ELW*(TAU-TAUEND)**2)
      EM = TAU / SIGV
      IF (EM .GT. SRAMAX)  GO TO 30
      IF (EM .LT. SRAMIN)  GO TO 20
      LOC = 0
      GO TO 190
C
C     YIELD POINT : Compute stress ratio at yield, EL
C
   20 CALL CROSS(EL, SRAMIN, ELW, SIGV1, TAUEND, AKAP)
      GO TO 70
   30 CALL CROSS(EL, SRAMAX, ELW, SIGV1, TAUEND, AKAP)
      GO TO 60
C
C     BEFORE PHASE TRANSFORMATION
C
   40 ELW = -1.D0
   50 EL = SIGV1
      EM = TAU / EL
      IF (EM .GT. SRAMAX)  GO TO 60
      IF (EM .LT. SRAMIN)  GO TO 70
      LOC = 0
      GO TO 190
C
   60 EM = EL * (0.5D0 + SQRT(0.25D0 + BP * SRAMAX**2))
      EL = BP / EM
      SIGV = EM - EL * TAU**2
      IF (TAU - SIGV*PHT .GE. 0.D0)  GO TO 80
      SRAMAX = TAU / SIGV
      LOC = 1
      GO TO 190
C
   70 EM = EL * (0.5D0 + SQRT(0.25D0+BP*SRAMIN**2))
      EL = BP / EM
      SIGV = EM - EL * TAU**2
      IF (TAU + SIGV*PHT .LE. 0.D0)  GO TO 80
      SRAMIN = TAU / SIGV
      LOC = -1
      GO TO 190
   80 LOC = 2
      IPHASE = 1
      TAUPHA = (SQRT(1. + 4.*BP/PHT**2)-1)/(2.*EL/PHT**2)
      SVPHA = TAUPHA / PHT
      IF (TAU .LE. 0.) TAUPHA = -TAUPHA
      IOM = 1
      EM = PHT **2
      EL = (-0.5D0 + SQRT(0.25D0 + BP * EM)) / (EL * EM)
C%%
      IF (IPHI .EQ. 0) THEN
        PHAI = 1.6 * PHT
      ENDIF
C%%
      EM = (EL**2) * (1.D0-(PHT/PHAI)**2)
      PHT = 1.D0 / PHAI **2
      GO TO 180
C
C     AFTER PHASE TRANSFORMATION
C
   90 IF (TAU * TAUB .GE. 0.D0)  GO TO 130
      IF (LOC .EQ. 2)  BP = TAUB*PHT/SIGV1
      EL = SIGV1 - TAUB * BP
      LOC = 2
      IF (EL .GT. SVMIN*SIGV1)  GO TO 120
C
C     COMPLETELY LIQUEFIED!!
C
  100 IF (LIQ .EQ. 1)  GO TO 110
  110 LOC = 3
      LIQ = 1
      EL = SVMIN*SIGV1
C
  120 EM = EL * EL
      GO TO 180
  130 IF (TAU**2 .GE. TAUB**2)  GO TO 150
      IF (LOC .EQ. -2)  GO TO 140
      LOC = -2
      BP = TAUB * PHT / SIGV1
  140 SIGV = SIGV1 + BP*(TAU-TAUB)
      IF (SIGV .LE. SVMIN*SIGV1)  GO TO 100
      GO TO 190
  150 IF (LOC .EQ. -2)  LOC = 2
  160 EM = SIGV1**2 - PHT * TAUB**2
      EL = (SVMIN*SIGV1) **2
      IF (EM .LE. 1.2D0 * EL .OR. LOC .NE. 3)  GO TO 170
      LOC = 2
      GO TO 90
  170 IF (EM .LE. 0.D0)  EM = EL
  180 SIGV = SQRT(EM + PHT * TAU**2)
C
C
C
  190 DYMPP = SIGV0 - SIGV
      PPARAM(1) = BP
      PPARAM(2) = ELW
      PPARAM(3) = AKAP
      PPARAM(4) = PHT
      PPARAM(5) = PHAI
      PPARAM(7) = LOC
      PPARAM(8) = TAUEND
      PPARAM(9) = DIF
      PPARAM(10) = SRAMAX
      PPARAM(11) = SRAMIN
      RETURN
C     DEBUG TRACE,INIT
C     AT 10
C     TRACE ON
      END
C=======================================================================
      SUBROUTINE CROSS (EL, SRA, ELW, SIGV0, TAUEND, AKAP)
C=======================================================================
      IMPLICIT REAL*8 (A-H, O-Z)
C
C
C     EXPLANATION ABOUT ARGUMENTS IS OMITTED.  SEE SUBROUTINE "DILATE".
C
C     "CROSS" FINDS EFFECTIVE VERTICAL STRESS AT THE YIELD POINT (EL).
C
C
      DATA  EPSRN, MINCR / 1.D-4, 10 /
      U0 = SIGV0
      I = 1
      ERN = EPSRN * SIGV0
   10 X = SRA * U0 - TAUEND
      Y = EXP(EL - ELW * X * X)
      DF = 1.D0 + 2.D0 * ELW * SRA * X * Y
      F = U0 - AKAP - Y
      U1 = U0 - F / DF
      IF (ABS(U1-U0) .LT. ERN)  GO TO 30
      IF (I .GE. MINCR)  GO TO 20
      I = I + 1
      U0 = U1
      GO TO 10
   20 U0 = ABS(U1-U0) / SIGV0
      WRITE (6,600)  U0
   30 EL = U1
      RETURN
C
C
  600 FORMAT (52H0### INSUFFICIENT CONVERGENCE IN FINDING YIELD POINT,
     1          4H  (, 1PD11.4, 27H)  -SUBROUTINE CROSS-  ### /)
      END
C=======================================================================
      SUBROUTINE GAMORC (IPHASE, TAUPHA, SVPHA, KGTYP, GMAX0, SIGV0,
     1                    EVP, GPARAM, KTTYP, TMAX0, TPARAM, ITYP,
     2                    GAMOR, NSUM, RET, SPARAM, GAMB, TAUBB, NMAX,
     3                    PPARAM)
C=======================================================================
C
C     THIS SUBROUTINE COMPUTE SHIFT OF SHEAR STRAIN WHEN PHASE
C     TRANSFORMED FOR ISHIHARA-TOWHATA MODEL
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION GPARAM(1), TPARAM(1), SPARAM(1), RET(2,1), PPARAM(1)
      COMMON / PHAILW / IPHI
C
C     COMPUTE STRESS FOR GIVEN STRAIN OR STRAIN FOR GIVEN STRESS
C
      IF (IPHASE .NE. 1) RETURN
      IPHASE = 0
C
C     COMPUTE SHEAR STRESS AT PHASE TRANSFORMED
C
      CALL GMXCAL(KGTYP, GMAX, GMAX0, SVPHA, SIGV0, EVP, GPARAM)
      CALL TMXCAL(KTTYP, TMAX, TMAX0, SVPHA, SIGV0, EVP, TPARAM)
      CALL FOLLOW(ITYP, GMAX, TMAX, NSUM, RET, SPARAM, GAMB, TAUBB)
      CALL MSGGAM(ITYP, TAUPHA, GAM, GMAX, TMAX, NSUM, TAUBB, GAMB,
     1            RET, NMAX, SPARAM, IFLG)
C     MODIFY SHEAR STRENGTH
      TPARAM(2) = PPARAM(5)
C     MODIFY SHEAR MODULUS AT SMALL STRAIN
      CALL GMXCAL(KGTYP, GMAX, GMAX0, SVPHA, SIGV0, EVP, GPARAM)
      CALL TMXCAL(KTTYP, TMAX, TMAX0, SVPHA, SIGV0, EVP, TPARAM)
      IF (IPHI .EQ. 1) RETURN
C
      NSUM = 0
C
      GO TO (100,200,110,100,200,110), ITYP
C
C     HYPERBOLIC MODEL
C
  100 EM = GMAX * (TMAX - ABS(TAUPHA))
      GAMOR = GAM - TMAX * TAUPHA / EM
      GO TO 200
C
C     RAMBERG-OSGOOD MODEL
C
  110 GAMOR = GAM - TAUPHA / GMAX * (1. + SPARAM(1))
     1      * ABS(TAUPHA / TMAX) ** (SPARAM(2)-1)
C
  200 CONTINUE
      TAUBB = 0.
      GAMB = 0.
C
      RETURN
      END
