C=======================================================================
C
C                    Y U S A Y U S A - 2
C
C     A Computer program for a numerical analysis of vibration and
C     liquefaction of sand deposit
C
C     Origina version     developed by Ikuo Towhata, Tokyo University
C                         in 1978
C
C     Modified by Nozomu Yoshida, Sato Kogyo Co., Ltd. in 1982
C       1) Dynamic allocation system is employed
C       2) Ramberg-Osgood model becomes available.
C       3) Clay deposit in which excess porewater pressure generation
C          does not occur, but it may chenge by seepage can be taken
C          into account.
C       4) Cohesion of material becomes available.
C     Note! Dynamic allocation system here is somewhat complicated,
C           which comes because of the limitation of the total numbers
C           of arguments in a subprogram.
C
C     Modified by N. Yoshida, Sato Kogyo Co., Ltd. in 1991 and
C     renamed to YUSAYUSA-2
C     In the previous modification Hara's model is employed for
C     R-O model, which is found to be inconveninent in some case.
C     Hence ordinary method is employed here. ROSTRN was originally
C     ultimate strain, which is changed to alpha. If the programmer
C     want to use Hara's model, fine line starting C## and activate 
C     them and comment down the following corresponding lines.
C
C     Version 2.01 by Nozomu Yoshida
C       File name can be specified for earthquake input
C     Version 2.02 by Nozomu Yoshida
C       Input in mass density is changed to unit weitht or weight
C       Add new option so that internal friction angle at low pressure
C        equals to input internal friction angle
C     Version 2.03 by Nozomu Yoshida (April 25, 2002)
C       Argument in subroutine was referred such as (1) or (N).
C       However, it may create error when argument becomes greater
C       than specified in some complier. Therefore, they are replaced
C       into (*)
C     Version 2.04 by Nozomu Yoshida (August 7, 2002)
C       Renewal print out format
C       Change dimension Z(*) so that large number of time increment
C       is available.
C       Note that the author moved to Oyo Corporation.
C       yoshida-nozomu@oyonet.oyo.co.jp
C     Version 2.05 by Nozomu Yoshida (October 11, 2002)
C       Warning when volume compressibility is not specified.
C     Version 2.10 by Nozomu Yoshida (Febrary 5, 2003)
C       Bug fix when time increment is subdivided.
C
C=======================================================================
C
C     Notes for programmer
C
C     1) Size of a maste array is temporary set 100000 at statement
C        "REAL*4 Z(100000)". It can be changed depending on the problem
C        and machine capacity. No message appears when required size
C        becomes greater than the specified size, hence it is safe to
C        take sufficient size.
C     2) The program used following unit for input and output. They are
C        related to specific file names in open statements. If the user
C        wants to use othe file name, change open statements.
C                Unit number  File name       Contents
C                    5        ---            input data
C                    6        ---            print output
C                   23        YUSAYUSA.tmp   work to store earthquake
C                   24        YUSAYUSA.tim   time history
C                   25        YUSAYUSA.max   information of analysis
C                                              deposit characteristics
C                                              maximum response
C                   10        ---            input excitaion
C        Note. ---: depend on machine
C        Note. At present, no file name is assigned, therefore default
C        name will be assigned. Usually, they are fort.[unit numer].
C
C=======================================================================
C
      IMPLICIT REAL*8 (A-H, O-Z)
      REAL *8  MS
      COMMON /ITACHI/  UND, UND2, INDELA
      REAL*4 Z(1000000)
      COMMON /TANUKI/ Q70,IRCAL
      COMMON /SARU/ ALPHA,BETA,IDAMP,IRNM
      CHARACTER*64 TITLE
      COMMON /RISU/ TITLE
      COMMON /RISU1/ ACG,RHOW,MS,DT,PMAX,
     1              IRVMAX,IWRMAX,IGRMAX,KUBC,ISTP,NW,NW4,N01,INDIBM
      COMMON /BUTA/  Q80, IPHI
      COMMON / ADRES / LPW1 ,LPW2 ,LPW3 ,LPW4 ,LPW5 ,LPW6 ,LPW7 ,
     1                 LPW8 ,LPW9 ,LPW10,LPW11,LPW12,LPW13,LPW14,
     2                 LPW15,LPW16,LPW17,LPP12,
     3                 LTI1 ,LTI2 ,LTI3 ,LTI4 ,LTI5 ,LTI6 ,
     4                 LWK1 ,LWK2 ,LWK5 ,LWK6 ,LWK7 ,LWK9 ,LWK10,
     5                 LWK11,LWK12,LWK13,LWK14,LMAX
      COMMON / ADRS1 / LPP1 ,LPP2 ,LPP3 ,LPP4 ,LPP5 ,LPP6 ,LPP7 ,
     1                 LPP8 ,LPP9 ,LPP10,LPP11,LPW18,LSP1 ,LSP2 ,
     2                 LSP3 ,LSP4 ,LSP5 ,LSP6 ,LSP7 ,LSP8 ,LSP9 ,
     3                 LSP10,LSP11,LSP12,LSP13,LSP14,LSP15,LSP16,
     4                 LSP17,LSP18,LSP19,LSP20,LSP21,LSP22,LSP23,
     5                 LSP24,LSP25,LSP26,LSP27,LSP28,LSP29,LSP30,
     6                 LSP31,LSP32,LTI7 ,LTI8 ,LTI9 ,LTI10,LTI11,
     7                 LTI12,LTI13,LTI14,LTI15,LTI16,LTI17,LTI18,
     8                 LTI19,LTI20,LTI21
      CHARACTER*24 SMANAL(2),SMDAMP(3),SMKUBC(3),SMIRCA(2)
      DATA  SMANAL /'Total stress','Effective stress'/
      DATA  SMDAMP /'No viscouse damping','Rayleigh damping',
     1              'viscouse damping'/
      DATA  SMKUBC /'No percolation','Zero surface velocity',
     1              'Zero surface pressure'/
      DATA  SMIRCA /'Not perform','Perform' /
C
      OPEN (UNIT=23,FILE='YUSAYUSA.tmp',FORM='UNFORMATTED')
      OPEN (UNIT=24,FILE='YUSAYUSA.tim',FORM='UNFORMATTED')
      OPEN (UNIT=25,FILE='YUSAYUSA.max',FORM='FORMATTED')
C     OPEN (UNIT=5, FILE='FORT.5')
C     OPEN (UNIT=6,FILE='F.6')
      REWIND 24
      REWIND 23
C     REWIND 5
C     23 for earthquake wave, 24 stores step by step result
C     25 store soil properties and maximum response
C
C     TITLE AND BASIC QUANTITIES
C
      READ(5,500) TITLE,IRCAL,IPHI,IRNM,IRVMAX,IWRMAX,IGRMAX
      IRCAL = 1 - IRCAL
      IF(IRNM .LE. 0) IRNM = 1
      IF(IRVMAX .LE. 0) IRVMAX = 1
C
      READ(5,510) ISTP,KUBC,INDELA,IDAMP,N,NW,NREV,ACG,RHOW,
     1            MS, ALPHA, BETA
      WRITE(6,610)  N, NW, NREV
      WRITE(6,620) SMANAL(ISTP+1),SMDAMP(IDAMP)
      IF(IDAMP .EQ. 2) WRITE(6,630) ALPHA,BETA
      IF(ISTP .EQ. 1) WRITE(6,640) SMKUBC(KUBC+1)
      WRITE(6,650) SMIRCA(IRCAL+1)
      IF(ISTP .EQ. 1) WRITE(6,660) IRNM, IRVMAX
      IF(IWRMAX .GT. 0)  WRITE(6,670) IWRMAX
      IF(IGRMAX .GT. 0)  WRITE(6,675) IGRMAX
      IF(IPHI .EQ. 1) WRITE(6,*) ' == Internal friction angle at',
     1 ' low confining stress is not used ==='
      IF(NREV .LE. 0) NREV = 40
      IF(ACG .LE. 0.) ACG = 9.81
C%%
      MS = MS / ACG
      RHOW = RHOW / ACG
C%%
C
      CALL ADRS (N, NW, N01, NW4, NREV)
      DO 100 I = 1,LMAX
      Z(I) = 0.
  100 CONTINUE
      WRITE(24) TITLE
      WRITE(25,600) TITLE
C
C     INTEGRATION OF BASEROCK MOTION RECORD
C
      CALL VIDEFF (Z(1), DT, INDIBM, PMAX ,LMAX)
C
C     PROCESS OF LIQUEFACTION
C
      CALL QUAKE (N, NREV,  Z(LPP1), Z(LPP2), Z(LPP3), Z(LPP4),
     1             Z(LPP5), Z(LPP6), Z(LPP7), Z(LPP8), Z(LPP9),
     2             Z(LPP10), Z(LPP11), Z(LPW18), Z(LSP1), Z(LSP2),
     3             Z(LSP3), Z(LSP4), Z(LSP5), Z(LSP6), Z(LSP7),
     4             Z(LSP8), Z(LSP9), Z(LSP10), Z(LSP11), Z(LSP12),
     5             Z(LSP13), Z(LSP14), Z(LSP15), Z(LSP16), Z(LSP17),
     6             Z(LSP18), Z(LSP19), Z(LSP20), Z(LSP21), Z(LSP22),
     7             Z(LSP23), Z(LSP24), Z(LSP25), Z(LSP26), Z(LSP27),
     8             Z(LSP28), Z(LSP29), Z(LSP30), Z(LSP31), Z(LSP32),
     9             Z(LTI7), Z(LTI8), Z(LTI9), Z(LTI10), Z(LTI11),
     *             Z(LTI12), Z(LTI13), Z(LTI14), Z(LTI15), Z(LTI16),
     1             Z(LTI17), Z(LTI18), Z(LTI19), Z(LTI20), Z(LTI21),
     2             Z(1   ), Z(1   ), NW)
C
  500 FORMAT(A60,2I1,I3,3I5)
  510 FORMAT(4I1,1X,3I5,5F10.0)
  600 FORMAT(A60)
  610 FORMAT(' ******** YUSAYUSA-2    Version 2.10  Febrary 2003',
     1       ' ********',/////,
     2  ' Number of layer -----------------------',I10,/
     3  ' Number of layer under water table -----',I10,/
     4  ' Max. number of strain reversals -------',I10)
  620 FORMAT(' Types of analysis --------------------- ',A24,/,
     1       ' Types of damping ---------------------- ',A24)
  630 FORMAT(31X,'(C) =',G13.5,' * (M) +',G13.5,' * (K)')
  640 FORMAT(' Flag on water ------------------------- ',A24)
  650 FORMAT(' Dynamic equilibrium ------------------- ',A24)
  660 FORMAT(/,' Renew of mass matrix ------------------',I10,' times',
     1       ' Renew of stress history ---------------',I10,' times')
  670 FORMAT(' Print of current value ----------------',I10,' times')
  675 FORMAT(' Print of maximum value ----------------',I10,' times')
      END
C=======================================================================
      SUBROUTINE ADRS(N,NW,N01,NW4,NREV)
C=======================================================================
C
C     "ADRS" determines working area for dynamic allocation
C
C     NOTE !   VARIABLES WITH * IN COMMENT PART ARE USED
C             LIKE "D(###)" IN "QUAKE"
C
C     N  : NUMBER OF LAYER
C     NW : NUMBER OF LAYER UNDER THE UNDERGROUND WATER
C     N01 = N + 1
C     NW4 = NW * 4
C     NREV : NUMBER OF MAXIMUM STRAIN REVERSALS
C
      COMMON / ADRES / LPW1 ,LPW2 ,LPW3 ,LPW4 ,LPW5 ,LPW6 ,LPW7 ,
     1                 LPW8 ,LPW9 ,LPW10,LPW11,LPW12,LPW13,LPW14,
     2                 LPW15,LPW16,LPW17,LPP12,
     3                 LTI1 ,LTI2 ,LTI3 ,LTI4 ,LTI5 ,LTI6 ,
     4                 LWK1 ,LWK2 ,LWK5 ,LWK6 ,LWK7 ,LWK9 ,LWK10,
     5                 LWK11,LWK12,LWK13,LWK14,LMAX
      COMMON / ADRS1 / LPP1 ,LPP2 ,LPP3 ,LPP4 ,LPP5 ,LPP6 ,LPP7 ,
     1                 LPP8 ,LPP9 ,LPP10,LPP11,LPW18,LSP1 ,LSP2 ,
     2                 LSP3 ,LSP4 ,LSP5 ,LSP6 ,LSP7 ,LSP8 ,LSP9 ,
     3                 LSP10,LSP11,LSP12,LSP13,LSP14,LSP15,LSP16,
     4                 LSP17,LSP18,LSP19,LSP20,LSP21,LSP22,LSP23,
     5                 LSP24,LSP25,LSP26,LSP27,LSP28,LSP29,LSP30,
     6                 LSP31,LSP32,LTI7 ,LTI8 ,LTI9 ,LTI10,LTI11,
     7                 LTI12,LTI13,LTI14,LTI15,LTI16,LTI17,LTI18,
     8                 LTI19,LTI20,LTI21
C
      N01 = N + 1
      NW4 = NW * 4
      NW41 = NW4 + 1
      IF(NW .EQ. 0) NW41 = 0
C
C *********** AREA FOR EXCESS PORE PRESSURE GENERATION *************
C
      LPP1  = 1
C                                                 @ BP (NW)
      LPP2  = LPP1  + NW * 2
C                                                 @ CS (NW)
      LPP3  = LPP2  + NW * 2
C                                                 @ PHT (NW)
      LPP4  = LPP3  + NW * 2
C                                                 @ ELW (NW)
      LPP5  = LPP4  + NW * 2
C                                                 @ AKAP (NW)
      LPP6  = LPP5  + NW * 2
C                                                 @ DIF (NW)
      LPP7  = LPP6  + NW * 2
C                                                 @ TAUEND (NW)
      LPP8  = LPP7  + NW * 2
C                                                 @ DU (NW)
      LPP9  = LPP8  + NW * 2
C                                                 @ SRA (2,NW)
      LPP10 = LPP9  + 2 * NW * 2
C                                                 @ LOC (NW)
      LPP11 = LPP10 + NW
C                                                 @ IFL (NW)
      LPP12 = LPP11 + NW
C                                                 @ LQTIME (2,N) *
      LPP13 = LPP12 + 2 * N * 2
C
C *********** AREA FOR PRESSURE WAVE CALCULATION *******************
C
      LPW1 = LPP13
C                                                 @ WD0 (NW41) *
      LPW2  = LPW1  + NW41 * 2
C                                                 @ WV0 (NW41) *
      LPW3  = LPW2  + NW41 * 2
C                                                 @ WA0 (NW41) *
      LPW4  = LPW3  + NW41 * 2
C                                                 @ WA1 (NW41) *
      LPW5  = LPW4  + NW41 * 2
C                                                 @ PK (3,NW4) *
      LPW6  = LPW5  + 3 * NW4 * 2
C                                                 @ PC (3,NW4) *
      LPW7  = LPW6  + 3 * NW4 * 2
C                                                 @ PM (3,NW4) *
      LPW8  = LPW7  + 3 * NW4 * 2
C                                                 @ CPM (NW4)  *
      LPW9  = LPW8  + NW4 * 2
C                                                 @ APM (NW4)  *
      LPW10 = LPW9  + NW4 * 2
C                                                 @ TO (3,NW4) *
      LPW11 = LPW10 + 3 * NW4 * 2
C                                                 @ CTO (NW4)  *
      LPW12 = LPW11 + NW4 * 2
C                                                 @ ATO (NW4)  *
      LPW13 = LPW12 + NW4 * 2
C                                                 @ DD1 (NW4)  *
      LPW14 = LPW13 + NW4 * 2
C                                                 @ DD2 (NW4)  *
      LPW15 = LPW14 + NW4 * 2
C                                                 @ PRD (NW4)  *
      LPW16 = LPW15 + NW4 * 2
C                                                 @ IPM (NW4)  *
      LPW17 = LPW16 + NW4
C                                                 @ ITO (NW4)  *
      LPW18 = LPW17 + NW4
C                                                 @ NL (NW)
      LPW19 = LPW18 + NW
C
C *********** AREA FOR SOIL PROPERTIES *****************************
C
      LSP1  = LPW19
C                                                 @ GMAX (N)
      LSP2  = LSP1  + N * 2
C                                                 @ G (N)
      LSP3  = LSP2  + N * 2
C                                                 @ RHO (N)
      LSP4  = LSP3  + N * 2
C                                                 @ RHOS (N)
      LSP5  = LSP4  + N * 2
C                                                 @ DZ (N)
      LSP6  = LSP5  + N * 2
C                                                 @ PR (N)
      LSP7  = LSP6  + N * 2
C                                                 @ SEP (N)
      LSP8  = LSP7  + N * 2
C                                                 @ PHAI (N)
      LSP9  = LSP8  + N * 2
C                                                 @ TAUF (N)
      LSP10 = LSP9  + N * 2
C                                                 @ GAMR (N)
      LSP11 = LSP10 + N * 2
C                                                 @ K0 (N)
      LSP12 = LSP11 + N * 2
C                                                 @ CA (N)
      LSP13 = LSP12 + N * 2
C                                                 @ CB (N)
      LSP14 = LSP13 + N * 2
C                                                 @ CC (N)
      LSP15 = LSP14 + N * 2
C                                                 @ GAM0 (N)
      LSP16 = LSP15 + N * 2
C                                                 @ GAM1 (N)
      LSP17 = LSP16 + N * 2
C                                                 @ TAU0 (N)
      LSP18 = LSP17 + N * 2
C                                                 @ TAU1 (N)
      LSP19 = LSP18 + N * 2
C                                                 @ SIGV0 (N)
      LSP20 = LSP19 + N * 2
C                                                 @ SIGV00 (N)
      LSP21 = LSP20 + N * 2
C                                                 @ SIGV99 (N)
      LSP22 = LSP21 + N * 2
C                                                 @ GAMOR (N)
      LSP23 = LSP22 + N * 2
C                                                 @ GAMREV (NREV,N)
      LSP24 = LSP23 + NREV * N * 2
C                                                 @ TAUREV (NREV,N)
      LSP25 = LSP24 + NREV * N * 2
C                                                 @ IC (N)
      LSP26 = LSP25 + N
C                                                 @ ICC (N)
      LSP27 = LSP26 + N
C                                                 @ ICGM (N)
      LSP28 = LSP27 + N
C                                                 @ LRFTYP (N)
      LSP29 = LSP28 + N
C                                                 @ ROSTRN (N)
      LSP30 = LSP29 + N * 2
C                                                 @ ROBETA (N)
      LSP31 = LSP30 + N * 2
C                                                 @ COHE (N)
      LSP32 = LSP31 + N * 2
C                                                 @ IFGEF (N)
      LSP33 = LSP32 + N
C
C *********** AREA FOR TIME INTEGRATION ****************************
C
      LTI1  = LSP33
C                                                 @ ALM (N01)  *
      LTI2  = LTI1  + N01 * 2
C                                                 @ CMLM (N01) *
      LTI3  = LTI2  + N01 * 2
C                                                 @ M (3,N01)  *
      LTI4  = LTI3  + 3 * N01 * 2
C                                                 @ MI (3,N01) *
      LTI5  = LTI4  + 3 * N01 * 2
C                                                 @ C (3,N01)  *
      LTI6  = LTI5  + 3 * N01 * 2
C                                                 @ K (3,N01)  *
      LTI7  = LTI6  + 3 * N01 * 2
C                                                 @ U0 (N01)
      LTI8  = LTI7  + N01 * 2
C                                                 @ V0 (N01)
      LTI9  = LTI8  + N01 * 2
C                                                 @ A0 (N01)
      LTI10 = LTI9  + N01 * 2
C                                                 @ U1 (N01)
      LTI11 = LTI10 + N01 * 2
C                                                 @ V1 (N01)
      LTI12 = LTI11 + N01 * 2
C                                                 @ A1 (N01)
      LTI13 = LTI12 + N01 * 2
C                                                 @ CMAX(14,N01)
      LTI14 = LTI13 + 14 * N01 * 2
C                                                 @ TMAX(14,N01)
      LTI15 = LTI14 + 14 * N01 * 2
C                                                 @ FA(N01)
      LTI16 = LTI15 + N01
C                                                 @ FV(N01)
      LTI17 = LTI16 + N01
C                                                 @ FD(N01)
      LTI18 = LTI17 + N01
C                                                 @ FT(N)
      LTI19 = LTI18 + N
C                                                 @ FG(N)
      LTI20 = LTI19 + N
C                                                 @ FS(N)
      LTI21 = LTI20 + N
C                                                 @ FP(N)
      LTI22 = LTI21 + N
C
C *********** WORK AREA ********************************************
C
C              FOR "DATAIN"
      LWK1  = LTI22
C                                                 @ VIS (N)
      LWK2  = LWK1  + N * 2
C                                                 @ WORK (33)
      LWK3  = LWK2  + MAX0(33,N+1)
C              FOR "READY"
      LWK1  = LWK1
C                                                 @ VIS (N)
      LWK2  = LWK1  + N * 2
C                                                 @ A (N01)
      LWK4  = LWK2  + N01 * 2
C              FOR "SHEROT"
      LWK1  = LWK1
C                                                 @ A (3,N01)
      LWK5  = LWK1  + 3 * N01 * 2
C                                                 @ AL (N01)
      LWK6  = LWK5  + N01 * 2
C                                                 @ CML (N01)
      LWK7  = LWK6  + N01 * 2
C                                                 @ ICG (N)
      LWK8  = LWK7  + N
C              FOR "AMNOSY" AND "AMNOSA"
      LWK1  = LWK1
C                                                 @ RPR (13)
      LWK9  = LWK1  + 13 * 2
C                                                 @ BM (3,12)
      LWK10 = LWK9  + 3 * 12 * 2
C                                                 @ TAUSTR (N)
      LWK11 = LWK10 + N * 2
C                                                 @ WORK (5*N01)
C                                   5*(N+1) FOR SHEROT, (N+1) FOR CORACC
      LWK12 = LWK11 + 5 * N01 * 2
C                                                 @ IWORK
C                                   N FOR SHEROT, 11 FOR RALVES
      LWK13 = LWK12 + MAX0(N,11)
C              FOR "RIEPER"
      LWK1  = LWK1
C                                                 @ Q (NW4)
      LWK14 = LWK1  + NW4 * 2
C                                                 @ R(NW4)
      LWK15 = LWK14 + NW4 * 2
C              FOR "UPDATE"
      LWK1  = LWK1
C                                                 @ WORK (NW41)
      LWK16 = LWK1  + NW41 * 2
C
      LMAX = MAX0(LWK3,LWK4,LWK8,LWK13,LWK15,LWK16)
C                MAX AREA FOR CALCULATION
C
      WRITE(6,600) LMAX
C
      RETURN
C
  600 FORMAT(//,' Max area used for calculation =',I8)
C
      END
C=======================================================================
      SUBROUTINE AMNOSY (IKKK, GMAX, G, RHO, DZ, PR, PHAI, TAUF, TAU0,
     1                    TAU1, GAMR, LRFTYP, ROSTRN, ROBETA, COHE,
     2                    GAM0, GAM1, SIGV0, U0, V0, A0, U1, V1, A1,
     3                    ALM, CMLM, K, C, M, MI, GAMREV, TAUREV,
     4                    CMAX, GAMOR, RPR, BM, TAUSTR, WORK, TIME,
     5                    DT, MS, AB0, VB0, UB0, AB, VB, UB,
     6                    IC, ICC, ICGM, IWORK, N, NW, NREV, JFLAG,
     7                    CA, CB, CC, K0)
C=======================================================================
C
C     "TAU0"     (N)        INITIAL STRESS
C     "TAU1"     (N)        CALCULATED STRESS
C     "GAM0"     (N)        INITIAL STRAIN
C     "GAM1"     (N)        CALCULATED STRAIN
C     "GAMOR"    (N)        STRAIN AT THE ORIGIN
C     "U0"       (N+1)      INITIAL DISPLACEMENT
C     "V0"       (N+1)      INITIAL VELOCITY
C     "A0"       (N+1)      INITIAL ACCELERATION
C     "U1"       (N+1)      CALCULATED DISPLACEMENT
C     "V1"       (N+1)      CALCULATED VELOCITY
C     "A1"       (N+1)      CALCULATED ACCELERATION
C     "K"        (3N+3)     RIGIDITY  MATRIX
C     "C"        (3N+3)     VISCOSITY MATRIX
C     "M"        (3N+3)     MASS      MATRIX
C     "MI"       (3N+3)     DATA FOR (M)**-1
C     "ALM"      (N+1)      DATA FOR (M)**-1
C     "CMLM"     (N+1)      DATA FOR (M)**-1
C     "ICGM"     (N)        DATA FOR (M)**-1
C     "RPR"      (13)       CUTS OF "DT"
C     "BM"      (3,12)      BASE MOTION AT RPR(J), J>1
C     "TAUSTR"   (N)        STORAGE FOR "TAU0"
C     "COHE"     (N)        COHESION
C     "LRFTYP"   (N)        TYPES OF RESTORING FORCE CHARACTERISTICS
C     "ROBETA"   (N)        BETA OF R.O. MODEL
C     "ROSTRN"   (N)        UNTIMATE STRAIN OF R.O. MODEL
C     "DT"                    TIME INCREMENT (VARIABLE)
C     "IREV"                  ORIGINAL "DT" IS DIVIDED INTO "IREV" PARTS
C
C     "AMNOSY" SUBDIVIDES "DT" AND PERFORMS STRAIN-REVERSAL CALCULATION.
C
      IMPLICIT REAL*8 (A-H, O-Z)
      COMMON /KABA/  P1, P2, P3, BTN, BTNC
     1       /KUMA/  NM1, NP1, NP2
     2       /ITACHI/  UND, UND2, INDELA
     3       /ROBA/  NLD, NUD, MP, N34, N45, NW45, N50
     4       /SARU/  ALPHA, BETA, IDAMP, IRNM
      REAL*8  K(3,*), M(3,*), MI(3,*), MS, K0
      DIMENSION  G(*), RHO(*), DZ(*), PR(*), TAUF(*), TAU0(*),
     1           TAU1(*), GAMR(*), GAM0(*), GAM1(*), SIGV0(*),
     2           U0(*), V0(*), A0(*), U1(*), V1(*), A1(*),
     3           ALM(*), CMLM(*), C(3,*), GAMREV(NREV,*),
     4           TAUREV(NREV,*),CMAX(14,*) ,RPR(13),BM(3,12),
     5           TAUSTR(*), GMAX(*), GAMOR(*), PHAI(*), WORK(*),
     6           IC(*), ICC(*), ICGM(*), IWORK(*),
     7           LRFTYP(*), ROSTRN(*), ROBETA(*), COHE(*),
     8           CA(*), CB(*), CC(*), K0(*)
C
C     IS STRAIN-REVERSAL CALCULATION NECESSARY?
C
      IF (IKKK .EQ. 2) GO TO 1
      JFLAG = 0
      CALL RALVES (GAM0,V0,V1,GAMREV,RPR,DT,IC,IWORK,N,NREV,IREV)
      IF (IREV .EQ. 0) GO TO 220
      GO TO 10
C
C     IN CASE OF TOO MUCH DEPARTURE FROM NON-LINEAR STRESS-STRAIN CURVE
C
    1 JFLAG = 0
      IREV = 4
      RPR(1) = 0.D0
      RPR(2) = 0.25D0 * DT
      RPR(3) = 0.50D0 * DT
      RPR(4) = 0.75D0 * DT
      RPR(5) = DT
C
C     ORIGINAL VALUES STORED AND PREPARATION FOR CALCULATION
   10 DO 20  J = 1, N
   20 TAUSTR(J) = TAU0(J)
      TIME = TIME - DT
      DTSTR = DT
      P1STR = P1
      P2STR = P2
      P3STR = P3
      IRNMST = IRNM
      BM(1, IREV) = AB
      BM(2, IREV) = VB
      BM(3, IREV) = UB
      IRNM = 0
      Q3 = 1.D0 / DT
      Q1 = Q3 * (AB - AB0)
      Q2 = Q3 * (VB - VB0)
      Q3 = Q3 * (UB - UB0)
      DO 30  I = 1, IREV - 1
          QMAX = RPR(I + 1)
          BM(1, I) = AB0 + Q1 * QMAX
          BM(2, I) = VB0 + Q2 * QMAX
          BM(3, I) = UB0 + Q3 * QMAX
   30 CONTINUE
      Q1 = TAU0(1) - G(1) * GAM0(1)
      A1(1) = Q1
      IF (N .EQ. 1)  GO TO 50
      DO 40  J = 2, N
          Q2 = TAU0(J) - G(J) * GAM0(J)
          A1(J) = Q2 - Q1
          Q1 = Q2
   40 CONTINUE
   50 IF (INDELA .EQ. 1)  A1(NP1) = -Q1
      ICL = 1
C
C     CYCLES OF STRAIN-REVERSAL CALCULATION
  100 DT = RPR(ICL + 1) - RPR(ICL)
      P1 = 0.5D0 * DT
      P3 = DT * DT
      P2 = P3 * BTN
      P3 = P3 * BTNC
      TIME = TIME + DT
      CALL SHEROT (DZ,GAM1,U0,V0,A0,U1,V1,A1,K,C,M,WORK(1),WORK(N34),
     1              WORK(N45),BM(3,ICL),BM(2,ICL),BM(1,ICL),DT,IWORK,N)
      CALL HDLOOP (TAUF, GAMR, TAU0, GAM0, TAU1, GAM1, TAUREV, GAMREV,
     1              GAMOR, LRFTYP, ROSTRN, ROBETA, IC, ICC, N, NREV)
      QMAX = 0.D0
      DO 110  J = 1, N
  110 QMAX = MAX(QMAX, ABS(TAU1(J)-TAU0(J)-G(J)*(GAM1(J)-GAM0(J)))
     1                 /TAUF(J))
      IF (QMAX .LT. CMAX(3, NP1))  GO TO 120
      CMAX(3, NP1) = QMAX
      CMAX(4, NP1) = TIME
  120 CALL REPONT (TAU0, GAM0, TAUREV, GAMREV, GAMOR, IC, ICC, N, NREV)
      IF (ICL .EQ. IREV)  GO TO 200
C
C     UPDATING
      DO 130  J = 1, N
          GAM0(J) = GAM1(J)
          TAU0(J) = TAU1(J)
          U0(J) = U1(J)
          V0(J) = V1(J)
  130 CONTINUE
      U0(NP1) = U1(NP1)
      V0(NP1) = V1(NP1)
      CALL RENEW (GMAX, G, PR, PHAI, TAUF, GAMR, GAM0, SIGV0, GAMREV,
     1             GAMOR, Q1, Q2, Q3, IC, N, -1 ,
     2             TAUREV, TAU1, LRFTYP, ROSTRN, ROBETA, COHE,
     3             NREV, CA, CB, CC, K0)
C
C     DYNAMIC EQUILIBRIUM
      CALL CORACC (GMAX,G,RHO,DZ,TAU0,GAM0,A0,V0,A1,ALM,CMLM,K,C,M,
     1              MI,WORK,BM(2,ICL),BM(1,ICL),MS,ICGM,N,NW)
      ICL = ICL + 1
      GO TO 100
C
C     CALCULATION COMPLETED
  200 DO 210  J = 1, N
  210 TAU0(J) = TAUSTR(J)
      DT = DTSTR
      P1 = P1STR
      P2 = P2STR
      P3 = P3STR
      IRNM = IRNMST
      JFLAG = 1
C
C     NO REVERSAL
  220 RETURN
      END
C=======================================================================
      SUBROUTINE COPPER (CS, DU, PRD, WD0, WV0, WA0, WA1, RHO, RHOS,
     1                    DZ, PR, SIGV0, WORK, DT, WTRSF, RHOW, ACG,
     2                    NL, NW, NW4, IWL, KUBC)
C=======================================================================
C
C     CS    (NW)            LATERALLY CONFINED COMPRESSIBILITY
C     DU    (NW)            EXCESS PORE WATER PRESSURE GENERATED IN
C                            LAST "DT"
C     PRD   (4NW)           POROSITY IN SUBLAYERS
C     WD0   (4NW+1)         OLD VERTICAL VELOCITY OF SOIL PARTICLES
C     WV0   (4NW+1)         (D/DT) * WD0
C     WA0   (4NW+1)         (D/DT)**2 * WD0
C     WA1   (4NW+1)         NEW WA
C     SIGV0 (N)             EFFECTIVE VERTICAL STRESS
C     WORK  (4NW+1)         WORKING REGION
C     WTRSF                   LOCATION OF GROUND WATER SURFACE
C     NW4                     NW4 = 4 * NW
C     IWL                     GROUND WATER SURFACE IS IN LAYER-IWL.
C     KUBC                    UPPER BOUNDARY CONDITION
C                                 =1 : UNDRAINED
C                                 =2 : FREE WATER SURFACE
C
C     "COPPER" UPDATES PRD, WD0, WV0, SIGV0, WTRSF AND IWL.
C
      IMPLICIT REAL*8 (A-H, O-Z)
      COMMON  /INU/  C1, C2, C3, E1M, NDRY
     1        /KABA/ P1, P2, P3, BTN, BTNC
      DIMENSION  CS(*),DU(*),PRD(*),WD0(*),WV0(*),WA0(*),WA1(*),RHO(*),
     1           RHOS(*),DZ(*),PR(*),SIGV0(*),WORK(*),NL(*)
      F(D, V, A, A1) = DT * D + C1 * V + C2 * A + C3 * A1
C
      J = NW4 + 1
      WA1(J) = 0.D0
      WORK(J) = 0.D0
      IF (KUBC .EQ. 1)  GO TO 10
      WORK(1) = F(WD0(1), WV0(1), WA0(1), WA1(1))
      GO TO 20
   10 WA1(1) = 0.D0
      WORK(1) = 0.D0
   20 DO 30 J = 2, NW4
   30 WORK(J) = F(WD0(J), WV0(J), WA0(J), WA1(J))
C
C     ABOVE THE GROUND WATER SURFACE
C
      IWL = NL(1)
      IF (KUBC .EQ. 1 .OR. NDRY .EQ. 0)  GO TO 100
      WTRSF = WTRSF + E1M * WORK(1)
      IF (WTRSF .GT. 0.D0)  IWL = IWL - 1
      IF (WTRSF .GT. DZ(NDRY))  IWL = IWL - 1
      X = WV0(1) + P1 * (WA0(1) + WA1(1))
      X = (ACG - X) / (ACG - WV0(1))
      DO 40  J = 1, NDRY
   40 SIGV0(J) = SIGV0(J) * X
C
C     POROSITY AND MASS DENSITY UPDATED
C
  100 DO 110 J = 1, NW4
          NLJ = 1 + (J - 1) / 4
          NLJ = NL(NLJ)
          PRD(J) = PRD(J) + 4.D0 * (1.D0 - PRD(J))
     1             * (WORK(J+1) - WORK(J)) / DZ(NLJ)
  110 CONTINUE
      DO 120  J = 1, NW4, 4
          NLJ = 1 + J / 4
          NLJ = NL(NLJ)
          PR(NLJ) = 0.25D0 * (PRD(J)+PRD(J+1)+PRD(J+2)+PRD(J+3))
          RHO(NLJ) = RHOW * PR(NLJ) + RHOS(NLJ) * (1.D0 - PR(NLJ))
  120 CONTINUE
C
C     VELOCITIES ETC. AT NODAL POINTS
C
      DO 130  J = 1, NW4
          WD0(J) = WD0(J) + DT * WV0(J) + P2 * WA1(J)
     1             + P3 * WA0(J)
          WV0(J) = WV0(J) + P1 * (WA0(J) + WA1(J))
  130 CONTINUE
      J = NW4 + 1
      WD0(J) = 0.D0
      WV0(J) = 0.D0
C
C     EFFECTIVE VERTICAL STRESS
C
      DO 140  J = 1, NW
          NLJ = NL(J)
          K = 4 * J
          SIGV0(NLJ) = SIGV0(NLJ) - DU(J) + 2.D0 * (WORK(K-2)
     1                 - WORK(K)) / (CS(J) * DZ(NLJ))
  140 CONTINUE
      RETURN
      END
C=======================================================================
      SUBROUTINE CORACC (GMAX, G, RHO, DZ, TAU0, GAM0, A0, V0, A1,
     1                    ALM, CMLM, K, C, M, MI, A, VB0, AB0, MS,
     2                    ICGM, N, NW)
C=======================================================================
C
C     "TAU0"     (N)        CURRENT STRESS
C     "GAM0"     (N)        CURRENT STRAIN
C     "A0"       (N+1)      CURRENT ACCELERATION
C     "V0"       (N+1)      CURRENT VELOCITY
C     "VB0"                   CURRENT BASEROCK MOTION
C     "AB0"                   CURRENT BASEROCK MOTION
C     "A1"       (N+1)      DATA FOR NEXT TIME STEP
C     "MI"       (3, N+1)   DATA FOR (M)**-1
C     "ALM"      (N+1)      DATA FOR (M)**-1
C     "CMLM"     (N+1)      DATA FOR (M)**-1
C     "ICGM"     (N)        DATA FOR (M)**-1
C
C     "CORACC" CALCULATES CURRENT ACCELERATION (DYNAMIC EQUILIBRIUM).
C
      IMPLICIT REAL*8 (A-H, O-Z)
      COMMON /KUMA/  NM1, NP1, NP2
     1       /SARU/  ALPHA, BETA, IDAMP, IRNM
     2       /ITACHI/  UND, UND2, INDELA
     3       /ROBA/  NLD, NUD, MP, N34, N45, NW45, N50
      REAL*8  K(3,*), M(3,*), MI(3,*), MS
      DIMENSION  GMAX(*), G(*), RHO(*), DZ(*), TAU0(*), GAM0(*), A0(*),
     1           V0(*), A1(*), ALM(*), CMLM(*), C(3,*), A(*),
     2           ICGM(*)
      IF (INDELA .EQ. 1)  GO TO 100
C
C****************** R I G I D   B A S E R O C K ***********************
C     (M), (C) AND (K) CORRECTED
C
      IF (IRNM .EQ. 1)  CALL MISET (RHO, DZ, A, M, MS, N)
      IF (IDAMP .NE. 2)  GO TO 10
      IF (NW .EQ. 0 .AND. IRNM .EQ. 0)  GO TO 10
      CALL MASET (GMAX, DZ, A, K, N)
      CALL RAKSAN (M, C, K, N)
   10 CALL MASET (G, DZ, A, K, N)
C
C     NEW RIGHT-HAND MEMBERS
C
      W1 = TAU0(1) - G(1) * GAM0(1)
      A1(1) = W1
      A0(1) = TAU0(1)
      IF (N .EQ. 1)  GO TO 30
      DO  20  J = 2, N
          W2 = TAU0(J) - G(J) * GAM0(J)
          A1(J) = W2 - W1
          W1 = W2
          A0(J) = TAU0(J) - TAU0(J - 1)
   20 CONTINUE
   30 A0(N) = A0(N) + VB0*(C(1,N)+C(2,N))
     1        + AB0*M(2,NP1)
      IF (IDAMP .EQ. 1)  GO TO 50
      CALL PREMUL (C, V0, A, N, NLD, NUD)
      DO  40  J = 1, N
   40 A0(J) = A0(J) - A(J)
C
C     CORRECTED ACCELERATION
C
   50 IF (IRNM .EQ. 1)  GO TO 60
      CALL ELSOL (2, MI, A0, CMLM, ALM, ICGM, N, NLD, MP)
      A0(NP1) = AB0
      RETURN
   60 DO  80  J = 1, N
      DO  70  I = 1, 3
          MI(I, J) = M(I, J)
   70 CONTINUE
   80 CONTINUE
      CALL ELSOL  (1, MI, A0, CMLM, ALM, ICGM, N, NLD, MP)
      A0(NP1) = AB0
      RETURN
C
C****************** E L A S T I C   B A S E R O C K *******************
C     (M), (C) AND (K) CORRECTED
C
  100 IF (IRNM .EQ. 0)  GO TO 110
      CALL MISET (RHO, DZ, A, M, MS, N)
  110 IF (IDAMP .NE. 2)  GO TO 150
      IF (NW .EQ. 0 .AND. IRNM .EQ. 0)  GO TO 150
      CALL MASET (GMAX, DZ, A, K, N)
      IF (NM1)  120, 120, 130
  120 A(1) = GMAX(1) / DZ(1)
      K(2, 1) = -A(1)
      GO TO 140
  130 K(3, N) = -A(N)
  140 K(1, NP1) = -A(N)
      K(2, NP1) =  A(N)
      CALL RAKSAN (M, C, K, NP1)
      C(2, NP1) = C(2, NP1) + UND
  150 CALL MASET (G, DZ, A, K, N)
      IF (NM1)  160, 160, 170
  160 A(1) = G(1) / DZ(1)
      K(2, 1) = -A(1)
      GO TO 180
  170 K(3, N) = -A(N)
  180 K(1, NP1) = -A(N)
      K(2, NP1) =  A(N)
C
C     NEW RIGHT-HAND MEMBERS
C
      W1 = TAU0(1) - G(1) * GAM0(1)
      A1(1) = W1
      A0(1) = TAU0(1)
      IF (N .EQ. 1)  GO TO 200
      DO 190  J = 2, N
          W2 = TAU0(J) - G(J) * GAM0(J)
          A1(J) = W2 - W1
          W1 = W2
          A0(J) = TAU0(J) - TAU0(J - 1)
  190 CONTINUE
  200 A1(NP1) = -W1
      A0(NP1) = -TAU0(N) + UND2 * VB0
      IF (IDAMP .EQ. 1)  GO TO 90
      CALL PREMUL (C, V0, A, NP1, NLD, NUD)
      DO 210  J = 1, NP1
  210 A0(J) = A0(J) - A(J)
      GO TO 230
   90 A0(NP1) = A0(NP1) - C(2, NP1) * V0(NP1)
C
C     CORRECTED ACCELERATION
C
  230 IF (IRNM .EQ. 1)  GO TO 240
      CALL ELSOL (2, MI, A0, CMLM, ALM, ICGM, NP1, NLD, MP)
      RETURN
  240 DO 260  J = 1, NP1
      DO 250  I = 1, 3
          MI(I, J) = M(I, J)
  250 CONTINUE
  260 CONTINUE
      CALL ELSOL  (1, MI, A0, CMLM, ALM, ICGM, NP1, NLD, MP)
      RETURN
      END
C=======================================================================
      SUBROUTINE CROSS (EL, SRA, ELW, SIGV0, TAUEND, AKAP)
C=======================================================================
C
C     EXPLANATION ABOUT ARGUMENTS IS OMITTED.  SEE SUBROUTINE "DILATE".
C
C     "CROSS" FINDS EFFECTIVE VERTICAL STRESS AT THE YIELD POINT (EL).
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DATA  EPSRN, MINCR / 1.D-4, 10 /
      U0 = SIGV0
      I = 1
      ERN = EPSRN * SIGV0
   10 X = SRA * U0 - TAUEND
      Y = EXP(EL - ELW * X * X)
      DF = 1.D0 + 2.D0 * ELW * SRA * X * Y
      F = U0 - AKAP - Y
      U1 = U0 - F / DF
      IF (ABS(U1-U0) .LT. ERN)  GO TO 30
      IF (I .GE. MINCR)  GO TO 20
      I = I + 1
      U0 = U1
      GO TO 10
   20 U0 = ABS(U1-U0) / SIGV0
      WRITE(6,600)  U0
   30 EL = U1
      RETURN
C
  600 FORMAT (' ### Insufficient convergence in finding yield point',
     1        '   (', 1PD11.4, ')  -Subroutine CROSS-  ###',/)
      END
C=======================================================================
      SUBROUTINE DAAPRE (TAU0, GAM0, SIGV0, SIGV00, U0, V0, A0, FA, FT,
     1                    FG, FS, FP, FV, FD, CMAX, TIME, QMAX, N,
     2                    N01)
C=======================================================================
C
C     "SIGV00"   (N)   INITIAL EFFECTIVE VERTICAL STRESS
C
C     "FA"                    DATA OF ACCELERATION
C     "FT"                    DATA OF SHEARING STRESS
C     "FG"                    DATA OF SHEARING STRAIN
C     "FS"                    DATA OF EFFECTIVE VERTICAL STRESS
C     "FP"                    DATA OF EXCESS PORE WATER PRESSURE
C     "FV"                    DATA OF VELOCITY
C     "FD"                    DATA OF DISPLACEMENT
C
C     "CMAX"     (14, N01)   ABSOLUTELY MAXIMUM VALUES AND ITS TIME
C     "QMAX"                  DEGREE OF CONVG. OF INITIAL STRESS METHOD
C
C     "DAAPRE" DETERMINES "FA" ETC. AND "CMAX".
C
      IMPLICIT REAL*8 (A-E,G,H,O-Z)
      DIMENSION  TAU0(*),GAM0(*),SIGV0(*),SIGV00(*),V0(*),A0(*),FA(*),
     1           FT(*),FG(*),FS(*),FP(*),FV(*),CMAX(14,*),U0(*),FD(*)
C
      DO  60  J = 1, N
      FA(J) = SNGL(A0(J))
      IF (ABS(A0(J)) .LE. ABS(CMAX(1, J)))  GO TO 10
      CMAX(1, J) = A0(J)
      CMAX(2, J) = TIME
   10 FT(J) = SNGL(TAU0(J))
      IF (ABS(TAU0(J)) .LE. ABS(CMAX(3,J)))  GO TO 20
      CMAX(3, J) = TAU0(J)
      CMAX(4, J) = TIME
   20 FG(J) = SNGL(GAM0(J))
      IF (ABS(GAM0(J)) .LE. ABS(CMAX(5,J)))  GO TO 30
      CMAX(5, J) = GAM0(J)
      CMAX(6, J) = TIME
   30 FS(J) = SNGL(SIGV0(J))
      D = SIGV00(J) - SIGV0(J)
      FP(J) = SNGL(D)
      IF (ABS(D) .LE. ABS(CMAX(7,J)))  GO TO 40
      CMAX(7, J) = D
      CMAX(8, J) = TIME
   40 D = SNGL(V0(J) - V0(N01))
      FV(J) = SNGL(D)
      IF (ABS(D) .LE. ABS(CMAX(9,J)))  GO TO 50
      CMAX(9, J) = D
      CMAX(10, J) = TIME
   50 D = SNGL(U0(J) - U0(N01))
      FD(J) = SNGL(D)
      IF (ABS(D) .LE. ABS(CMAX(11,J)))  GO TO  60
      CMAX(11, J) = D
      CMAX(12, J) = TIME
   60 CONTINUE
C
  100 FA(N01) = SNGL(A0(N01))
      IF (ABS(A0(N01)) .LE. ABS(CMAX(1, N01)))  GO TO 110
      CMAX(1, N01) = A0(N01)
      CMAX(2, N01) = TIME
  110 IF (QMAX .LE. CMAX(3,N01))  GO TO 120
      CMAX(3, N01) = QMAX
      CMAX(4, N01) = TIME
  120 FV(N01) = SNGL(V0(N01))
      IF (ABS(FV(N01)) .LE. ABS(CMAX(9, N01)))  GO TO 130
      CMAX(9, N01) = FV(N01)
      CMAX(10, N01) = TIME
  130 FD(N01) = SNGL(U0(N01))
      IF (ABS(FD(N01)) .LE. ABS(CMAX(11, N01)))  GO TO 140
      CMAX(11, N01) = FD(N01)
      CMAX(12, N01) = TIME
  140 CONTINUE
C
      WRITE(24) SNGL(TIME)     , (FA(I),I=1,N01), (FV(I),I=1,N01),
     1          (FD(I),I=1,N01), (FT(I),I=1,N ), (FG(I),I=1,N ),
     2          (FS(I),I=1,N ), (FP(I),I=1,N )
      END
C=======================================================================
      SUBROUTINE DATAIN (BP, CS, WA1, PRD, PHT, ELW, GMAX, G, VIS, RHO,
     1                    RHOS, DZ, PR, SEP, PHAI, K0, CA, CB, CC, TAUF,
     2                    GAMR, GAM0, SIGV0, GAMREV, AKAP, DIF, SIGV99,
     3                    SIGV00, GAMOR, DT, TIME, UB0, VB0, UB, VB, AB,
     4                    TMS, NL, IC, IREVUD, IWRITE, IWL, N, NW,
     5                    NREV, INDIBM, NNW , TAUREV, TAU1,
     6                    LRFTYP, ROSTRN, ROBETA, COHE, IFGEF, WORK)
C=======================================================================
C
C      "DATAIN" READS DATA AND CALCULATES SOIL PARAMETERS.
C
      IMPLICIT REAL*8 (A-H, O-Z)
      REAL*8  MS, K0(N)
      COMMON /KUMA/  L00, L01, L02
     1       /SARU/  ALPHA, BETA, IDAMP, IRNM
     2       /ITACHI/  RHOBAS, GBAS, INDELA
     3       /INU/  Q30, Q31, Q32, Q33, L30
     4       /ROBA/  L40, L41, L42, L43, L44, L45, L46
     5       /KABA/  P1, P2, P3, BTN, BTNC
     6       /TANUKI/  Q70, IRCAL
     7       /BUTA/  Q80, IPHI
      COMMON /RISU1/ ACG,RHOW,MS,DTT,PMAX,
     1              IRVMAX,IWRMAX,IGRMAX,KUBC,ISTP,NWW,NW4,N01,INDIBN
      DIMENSION  BP(*), CS(*), PHT(*), GMAX(*), G(*),
     1           VIS(*), RHO(*), RHOS(*), DZ(*), PR(*), SEP(*), PHAI(*),
     2           CA(*), CB(*), CC(*), TAUF(*), GAMR(*), GAM0(*),
     3           SIGV0(*), GAMREV(NREV,*), ELW(*), DIF(*),
     4           AKAP(*), SIGV99(*), SIGV00(*), GAMOR(*), WA1(*),
     5           PRD(*), NL(*), IC(*) ,
     6           TAUREV(NREV,*),TAU1(*),LRFTYP(*), ROSTRN(*), ROBETA(*)
     7          ,COHE(*), IFGEF(*), WORK(*)
      DIMENSION DUMVAL(8)
      COMMON / RRR / IND3
      CHARACTER *4 SPS, RESTF(2), SOILTP(2)
      DATA APAI / 1.745329251994330D-2 /
      DATA SPS / '    ' /
      DATA RESTF,SOILTP / 'H.D.' , 'R.O.' , 'Sand' , 'Clay' /
      FGMAX(A,B,C,AK0,SIGV) = A * ((B-E)**2/(1.D0+E))
     1    * (3.333333333333333D-1 * SIGV * (1.D0+AK0+AK0)) ** C
C     STATEMENT FUNCTIONS FOR "GMAX" AND "TAUF"
C
      REWIND 23
      WRITE(24) N,NW,INDIBM,SNGL(DT)
      WD = MS * ACG
      WRITE(25,7000) N,NW,MS,ACG
C
C *************** R E A D   S O I L   D A T A **********************
C
      TMS = MS * ACG
      NDRY = N - NW
      IWL = MAX0(1,NDRY)
      DO 100 I = 1,N
      J = I - NDRY
      READ(5,5000) LRFTYP(I),IFGLY,IFGEF(I),IFGST,IFGGT,
     1             DZ(I),RHO(I),COHE(I),PHAI(I),K0(I),CA(I),CB(I),CC(I),
     2             ROBETA(I),ROSTRN(I),PR(I),SEP(I),SIGV99(I),VIS(I)
C                  IFGLY : LAYER NUMBER FOR DEFAULT VALUE
C                  IFGST : 0    SHEAR STRENGTH IS GIVEN AS "PHAI"
C                          1    COHESION AND FRICTION ANGLE
C                          NOTE! COHESION SHOULD BE ZERO ,IF GENERATION
C                          PORE PRESSURE IS TAKEN INTO ACCOUNT.
C                  IFGGT : 0    SHEAR MODULUS IS GIVEN AS "CA"
C                          1    SHEAR MODULUS IS CALCULATED FROM "CA"'S
C
      IF (ISTP .EQ. 1) THEN
        IF(NW .GT. 0 .AND. J .GT. 0)
     1       READ(5,5010) BP(J),CS(J),PHT(J),ELW(J),AKAP(J)
      ENDIF
C
      IF(DZ(I) .LE. 0. .AND. IFGLY .GT. 0) DZ(I) = DZ(IFGLY)
      IF(DZ(I) .LE. 0. .AND. IFGLY .LE. 0) DZ(I) = DZ(I-1)
      IF(RHO(I) .LE. 0. .AND. IFGLY .GT. 0) RHO(I) = RHO(IFGLY)
      IF(RHO(I) .LE. 0. .AND. IFGLY .LE. 0) RHO(I) = RHO(I-1)
      IF(PR(I) .LE. 0. .AND. IFGLY .GT. 0) PR(I) = PR(IFGLY)
      IF(PR(I) .LE. 0. .AND. IFGLY .LE. 0) PR(I) = PR(I-1)
C
C%%
      RHO(I) = RHO(I) / ACG
C%%
C
      WU = WD
      IF(NW .GT. 0 .AND. J .GT. 0) GO TO 10
C       SOILS ABOVE THE UNDERGROUND WATER
      RHOS(I) = RHO(I) / (1.D0 - PR(I))
      WD = WU + RHO(I) * DZ(I) * ACG
      SIGV0(I) = 0.5D0 * (WU + WD)
      GO TO 20
C       SOILS UNDER THE UNDERGROUND WATER
   10 IF(J .EQ. 1) TMS = WD
      RHOS(I) = (RHO(I) - RHOW * PR(I)) / (1. - PR(I))
      WD = WU + DZ(I) * (RHO(I) - RHOW) * ACG
      SIGV0(I) = 0.5D0 * (WU + WD)
      K = 4 * J
      WA1(K - 2) = 0.5D0 * (WU + SIGV0(I))
      WA1(K - 1) = SIGV0(I)
      WA1(K   ) = 0.5D0 * (WD + SIGV0(I))
      WA1(K + 1) = WD
      PRD(K - 3) = PR(I)
      PRD(K - 2) = PR(I)
      PRD(K - 1) = PR(I)
      PRD(K   ) = PR(I)
C
      IF(PHT(J) .LE. 0. .AND .IFGLY .GT. 0) PHT(J) = PHT(IFGLY - NDRY)
      NL(J) = I
      IF(BP(J) .LE. 0. .AND .IFGLY .GT. 0) BP(J) = BP(IFGLY - NDRY)
      IF(AKAP(J).LE.0. .AND .IFGLY.GT.0) AKAP(J) = AKAP(IFGLY - NDRY)
      DIF(J) = 1.0D0
      IF(CS(J) .LE. 0. .AND .IFGLY .GT. 0) CS(J) = CS(IFGLY - NDRY)
      IF(CS(J) .LE. 0. .AND .IFGLY .LE. 0) THEN
        IF (J .EQ. 1) THEN
          CS(J) = 1.D-5
        ELSE
          CS(J) = CS(J - 1)
        ENDIF
        IF (ISTP .EQ. 0 .AND. KUBC .GT. 0) THEN
          WRITE(6,*) ' Compressibility is necessary for drainage'
          WRITE(6,*) ' Temporary value is used for ', J, '-th layer'
          WRITE(6,*) ' from the water table.   Value = ', CS(J)
        ENDIF
      ENDIF
C
   20 IF(SIGV99(I) .LE. 0.) SIGV99(I) = 0.03
      SIGV99(I) = SIGV99(I) * SIGV0(I)
      SIGV00(I) = SIGV0(I)
C
C        TAUF
C
      IF(COHE(I) .LE. 0. .AND .IFGLY .GT. 0) COHE(I) = COHE(IFGLY)
      IF(J .GT. 0 .AND. IFGEF(I) .EQ. 0) COHE(I) = 0.
      IF(PHAI(I) .LT. 0.) GO TO 40
      IF(IFGST .EQ. 1) GO TO 30
      TAUF(I) = PHAI(I)
      PHAI(I) = ATAN((TAUF(I) - COHE(I)) / SIGV0(I)) / APAI
      GO TO 50
   30 TAUF(I) = COHE(I) + SIGV0(I) * TAN(APAI * PHAI(I))
      GO TO 50
   40 IF(IFGLY .GT. 0) TAUF(I) = TAUF(IFGLY)
      IF(IFGLY .LE. 0) TAUF(I) = TAUF(I - 1)
      IF(IFGLY .GT. 0) PHAI(I) = PHAI(IFGLY)
C
C        GMAX
C
   50 IF(IFGLY .GT. 0) GO TO 80
      IF(I .EQ. 1 .AND. CB(I) .LE. 0.) CB(I) = 2.17
      IF(I .GT. 1 .AND. CB(I) .LE. 0.) CB(I) = CB(I-1)
      IF(I .EQ. 1 .AND. CC(I) .LE. 0.) CC(I) = 0.4
      IF(I .GT. 1 .AND. CC(I) .LE. 0.) CC(I) = CC(I - 1)
      IF(CA(I) .GT. 0) GO TO 60
      IF(I .EQ. 1) CA(I) = 14000.D0
      IF(I .GT. 1) CA(I) = CA(I - 1)
      IFGGT = 1
   60 IF(IFGGT .EQ. 1) GO TO 70
      E = PR(I) / (1.D0 - PR(I))
      CA(I) = CA(I) / FGMAX(1.D0,CB(I),CC(I),K0(I),SIGV0(I))
   70 E = PR(I) / (1.D0 - PR(I))
      GMAX(I) = CA(I) * (CB(I) - E) ** 2 / (1.D0 + E)
     1        * ((1.D0 + 2.D0 * K0(I)) / 3.D0 * SIGV0(I)) ** CC(I)
      GO TO 90
   80 IF(CA(I) .LE. 0.) CA(I) = CA(IFGLY)
      IF(CB(I) .LE. 0.) CB(I) = CB(IFGLY)
      IF(CC(I) .LE. 0.) CC(I) = CC(IFGLY)
      GO TO 70
   90 CONTINUE
      WRITE(25,7010) DZ(I), RHO(I), PR(I), SEP(I), PHAI(I),
     1               TAUF(I), GMAX(I), LRFTYP(I), ROBETA(I),
     2               ROSTRN(I), COHE(I), VIS(I), K0(I), CA(I), CB(I),
     3               CC(I)
      IF(NW .GT. 0 .AND. J .GT. 0) WRITE(25,7020)
     1               BP(J), CS(J), PHT(J), ELW(J), AKAP(J)
  100 CONTINUE
      IF(INDELA .EQ. 1) THEN
        READ(5,5010) RHOBAS,GBAS
C%%
        RHOBAS = RHOBAS / ACG
C%%
      ENDIF
      IF(NW .GT. 0) WA1(1) = TMS
C
C ************** W R I T E   I N P U T   D A T A *******************
C
      NPRNT = (N - 1) / 8 + 1
C
C%%
      DO 101 I = 1, N
        RHO(I) = RHO(I) * ACG
  101 CONTINUE
      MS = MS * ACG
      RHOW = RHOW * ACG
C%%
      WRITE(6,6060) ACG,MS,RHOW
C%%
      MS = MS / ACG
      RHOW = RHOW / ACG
C%%
      DO 140 II = 1,NPRNT
        IS = 8 * II - 7
        IE = 8 * II
        IF(IE .GT. N) IE = N
        WRITE(6,6000) (SPS,J,J=IS,IE)
        WRITE(6,6010) (RESTF(LRFTYP(J)),J=IS,IE)
        CALL DATAP(002,'Thickness      ',IFGEF,    DZ,IS,IE,NDRY)
        CALL DATAP(002,'Unit weight    ',IFGEF,   RHO,IS,IE,NDRY)
        CALL DATAP(000,'Ulti. strength ',IFGEF,  TAUF,IS,IE,NDRY)
        DO 110 J = IS,IE
          IF(LRFTYP(J) .EQ. 2) GO TO 120
  110   CONTINUE
        GO TO 130
  120   CALL DATAP(100,'Beta           ',IFGEF,ROBETA,IS,IE,NDRY)
C##      CALL DATAP(100,'Ultimate strain',IFGEF,ROSTRN,IS,IE,NDRY)
        CALL DATAP(100,'Alpha          ',IFGEF,ROSTRN,IS,IE,NDRY)
  130   CALL DATAP(000,'Initial modulus',IFGEF,  GMAX,IS,IE,NDRY)
        CALL DATAP(100,'Cohesion       ',IFGEF,  COHE,IS,IE,NDRY)
        CALL DATAP(102,'Friction angle ',IFGEF,  PHAI,IS,IE,NDRY)
        CALL DATAP(102,'Co. earth pre. ',IFGEF,    K0,IS,IE,NDRY)
        CALL DATAP(100,'Coef.  CA      ',IFGEF,    CA,IS,IE,NDRY)
        CALL DATAP(100,'Coef.  CB      ',IFGEF,    CB,IS,IE,NDRY)
        CALL DATAP(100,'Coef.  CC      ',IFGEF,    CC,IS,IE,NDRY)
        CALL DATAP(100,'Elastic modulus',IFGEF,  GMAX,IS,IE,NDRY)
        JJ = 0
        DO 131 J = IS, IE
          JJ = JJ + 1
          DUMVAL(JJ) = SQRT(GMAX(J) * ACG / RHO(J))
  131   CONTINUE
        CALL DATAP(100,'Wave velocity  ',IFGEF,DUMVAL, 1,JJ,NDRY)
        WRITE(6,6050) (SOILTP(IFGEF(J)+1),J=IS,IE)
        IF(IDAMP .EQ. 3)
     1  CALL DATAP(100,'Damp. Coef.    ',IFGEF,   VIS,IS,IE,NDRY)
        IF(ISTP .EQ. 0) GO TO 140
        CALL DATAP(002,'Porosity       ',IFGEF,    PR,IS,IE,NDRY)
        CALL DATAP(000,'Permeability   ',IFGEF,   SEP,IS,IE,NDRY)
        IF (ISTP .EQ. 0) GO TO 140
        IF(NW .LE. 0) GO TO 140
        CALL DATAP(100,'Min. nor. pre. ',IFGEF,SIGV99,IS,IE,NDRY)
        CALL DATAP(010,'Bp             ',IFGEF,    BP,IS,IE,NDRY)
        CALL DATAP(010,'Volume comp.   ',IFGEF,    CS,IS,IE,NDRY)
        CALL DATAP(012,'Phase tran. ang',IFGEF,   PHT,IS,IE,NDRY)
        CALL DATAP(010,'Bu             ',IFGEF,   ELW,IS,IE,NDRY)
        CALL DATAP(010,'Kappa          ',IFGEF,  AKAP,IS,IE,NDRY)
  140 CONTINUE
C%%
      DO 141 I = 1, N
        RHO(I) = RHO(I) / ACG
  141 CONTINUE
      IF(INDELA .EQ. 0) WRITE(6,6020)
C%%
      RHOBAS = RHOBAS * ACG
C%%
      IF(INDELA .EQ. 1) WRITE(6,6030) RHOBAS,GBAS
C%%
      RHOBAS = RHOBAS / ACG
C%%
      WRITE(25,7020) RHOBAS, GBAS
C
C ********* C H A N G E   D A T A   F O R   C A L U C U L A T I O N ***
C
      WORK(1) = MS * ACG
      SV0 = WORK(1)
      TMS = TMS / ACG
      DO 150 I = 1,N
        PHAI(I) = TAN(APAI * PHAI(I))
        DUM = DZ(I) * RHO(I) * ACG
        WORK(I+1) = SV0 + DUM / 2.D0
        SV0 = SV0 + DUM
        J = I - NDRY
        IF(NW .LE. 0 .OR. J .LE. 0) GO TO 150
        SV0 = SV0 - DUM
        DUM = DZ(I) * (RHO(I) - RHOW) * ACG
        WORK(I+1) = SV0 + DUM / 2.D0
        SV0 = SV0 + DUM
        PHT(J) = TAN(APAI * PHT(J))
        ELW(J) = ELW(J) / (2.D0 * SIGV0(I) ** 2)
        AKAP(J) = AKAP(J) * SIGV0(I)
  150 CONTINUE
      WRITE(24) (SNGL(DZ(I)),I=1,N)
      WRITE(24) (SNGL(WORK(I)),I=1,N+1)
C
          TIME = 0.D0
          UB0  = 0.D0
          VB0  = 0.D0
          IREVUD = IRVMAX - 1
          IWRITE = IWRMAX - 1
          IND3   = INDIBM * 3
C
C     ELASTIC BASEROCK
C
      IF (INDELA .NE. 1)  GO TO 160
      RHOBAS = SQRT(RHOBAS * GBAS)
      GBAS = RHOBAS * 2.D0
C
  160 CONTINUE
C     CONSTANTS
C
  170 Q30 = 0.5D0 * DT * DT
      Q32 = DT**3 / 3.D0
      Q31 = BTNC * Q32
      Q32 = Q32 * BTN
      Q33 = 0.D0
      IF (NDRY .GE. 1)  Q33 = -1.D0 + 1.D0 / PR(NDRY)
      Q70 = DT / 12.D0
      Q80 = 8.D0 / 5.D0
      L00 = N - 1
      L01 = N + 1
      L02 = N + 2
      L30 = NDRY
      L40 = 1
      IF (N + INDELA .EQ. 1)  L40 = 0
      L41 = L40
      L42 = MIN(N+INDELA, L40+L41+1)
      L43 = 4 + 3 * N
      L44 = 5 + 4 * N
      L45 = 5 + 4 * NW
      L46 = N + 50
C
      CALL RENEW (GMAX, G, PR, PHAI, TAUF, GAMR, GAM0, SIGV0,
     1            GAMREV, GAMOR, UB, VB, AB, IC, N, NNW ,
     2            TAUREV, TAU1, LRFTYP, ROSTRN, ROBETA, COHE,
     3            NREV, CA, CB, CC, K0)
      IF (ISTP .EQ. 0) NW = 0
C
      RETURN
C
 5000 FORMAT(4X,I1,I5,3I1,7X,6F10.0,/,(8F10.0))
 5010 FORMAT(8F10.0)
 6000 FORMAT(///,16X,8(A1,1X,5H-----,I3,5H ----))
 6010 FORMAT(' R. F. Charact. ',8(8X,A4,3X))
 6020 FORMAT(' Rigid baserock ')
 6030 FORMAT(///,' Elastic baserock      Unit weight =',F9.3,5X,
     1       'Shear modulus =',D12.5)
 6050 FORMAT(' Soil type      ',8(6X,A4,5X))
 6060 FORMAT(//,'----- Constants -----',/,
     1       ' Acceleration of gravity -------- ',F12.3,/
     2       ' Weight at ground surface ------- ',F12.3,/
     3       ' Unit weight of water ----------- ',F12.3)
 7000 FORMAT(2I10,2E15.8)
 7010 FORMAT(5E15.8,/,2E15.8,I4,3E15.8,/,(5E15.8))
 7020 FORMAT((5E15.8))
C
      END
C=======================================================================
      SUBROUTINE RENEW (GMAX, G, PR, PHAI, TAUF, GAMR, GAM0, SIGV0,
     1              GAMREV, GAMOR, UB, VB, AB, IC, N, NNW ,
     2              TAUREV, TAU1, LRFTYP, ROSTRN, ROBETA, COHE,
     3              NREV, CA, CB, CC, K0)
C=======================================================================
C
C      "RENEW" (ENTRY STATEMENT) UPDATES "GMAX", "G", "TAUF", AND
C      "GAMR", AND SUPPLIES BASEROCK MOTION DATA; "AB", "VB", AND "UB".
C
      IMPLICIT REAL*8 (A-H, O-Z)
      REAL*8 K0
      DIMENSION  GMAX(*), G(*), PR(*), PHAI(*), TAUF(*), GAMR(*),
     1           GAM0(*), SIGV0(*), GAMREV(NREV,*), GAMOR(*),
     2           IC(*), TAUREV(NREV,*),TAU1(*),LRFTYP(*), ROSTRN(*),
     3           ROBETA(*), COHE(*), CA(*), CB(*), CC(*), K0(*)
      DIMENSION BSMOTN(255)
      COMMON / RRR / IND3
C
      DATA  IBMNRD, IDBSMT, IDBMAX / 2*0, 255 /
C
      FGMAX(A,B,C,AK0,SIGV) = A * ((B-E)**2/(1.D0+E))
     1    * (3.333333333333333D-1 * SIGV * (1.D0+AK0+AK0)) ** C
  200 DO 230  J = 1, N
          IF (NNW .LE. 0)  GO TO 210
          E = PR(J) / (1.D0 - PR(J))
          GMAX(J) = FGMAX(CA(J), CB(J), CC(J), K0(J), SIGV0(J))
          TAUF(J) = COHE(J) + SIGV0(J) * PHAI(J)
          GAMR(J) = TAUF(J) / GMAX(J)
  210 G(J) = TAU1(J)
          IF (IC(J) .GT.0)  GO TO 220
          CALL SSTMOD(LRFTYP(J),G(J),GAM0(J),TAUF(J),GAMR(J),0.D0,
     1           GAMOR(J),1.0,ROBETA(J),ROSTRN(J))
          GO TO 230
  220     CALL SSTMOD(LRFTYP(J),G(J),GAM0(J),TAUF(J),GAMR(J),
     1       TAUREV(IC(J),J),GAMREV(IC(J),J),0.5,ROBETA(J),ROSTRN(J))
  230 CONTINUE
      IF (NNW .EQ. -1)  RETURN
C ************ READ EARTHQUAKE WAVE *******************************
C
      IF (IDBSMT .NE. IDBMAX   .AND.   IDBSMT .NE. 0)  GO TO 260
      J = IND3 - IBMNRD
      IF (J .LT. IDBMAX)  GO TO 240
      READ(23)  BSMOTN
      GO TO 250
  240 READ(23)  (BSMOTN(I),  I = 1, J)
  250 IDBSMT = 0
C
  260 AB = BSMOTN(IDBSMT + 1)
      VB = BSMOTN(IDBSMT + 2)
      UB = BSMOTN(IDBSMT + 3)
      IDBSMT = IDBSMT + 3
      IBMNRD = IBMNRD + 3
C
      RETURN
C
      END
C=======================================================================
      SUBROUTINE DATAP(IT,NAME,IFGEF,DATA,IS,IE,NDRY)
C=======================================================================
C
C     IT : TYPES OF DATA
C          100TH DIGIT : IF UNITY, ZERO IS NOT PRINTED
C           10TH DIGIT : IF UNITY, CONSTANTS FOR EFFECTIVE STRESS A.
C           UNIT DIGIT : IF UNITY, DATA IS INTEGER
C                        IF ZERO, DRAW "E-TYPE"
C                        IF TWE , DRAW "F-TYPE"
C     NAME : HEADING NAME
C     IFGEF : FLAG ON EXCESS P.P. GENERATION
C     DATA : DATA TO BE PRINTED
C     IS,IE : DATA NUMBER TO BE PRINTED
C     NDRY : NUMBER OF LAYER ABOVE THE UNDERGROUND WATER
C     WORKDD : WORK AREA FOR PRINT
C
      IMPLICIT REAL*8 (A-H,O-Z)
      CHARACTER *15 NAME
      CHARACTER *15 A
      CHARACTER *137 WORKDD
      DIMENSION DATA(*), IFGEF(*)
C
      WORKDD = '                 '
      IT1 = MOD(IT,10)
      IT10 = MOD(IT/10,10)
      IT100 = IT / 100
      WORKDD(2:16) = NAME
C
      DO 100 I = IS,IE
      J = 17 + (I - IS) * 15
      IF(IT10 .EQ. 0) GO TO 10
      IF(I - NDRY .GT. 0) GO TO 20
      GO TO 100
   10 K = I
      GO TO 30
   20 IF(IFGEF(I) .EQ. 1) GO TO 100
      K = I - NDRY
   30 IF(IT1 .NE. 1) GO TO 40
      IF(IT100 .EQ. 1 .AND. DATA(K) .EQ. 0.) GO TO 100
      WRITE(A,'(4X,I5,6X)') DATA(K)
      GO TO 50
   40 IF(IT100 .EQ. 1 .AND. DATA(K) .EQ. 0.) GO TO 100
      IF(IT1 .EQ. 2) GO TO 50
      WRITE(A,'(1X,1PG14.5)') DATA(K)
      GO TO 60
   50 WRITE(A,'(1X, F14.3)') DATA(K)
   60 WORKDD(J:J+14) = A
  100 CONTINUE
      WRITE(6,600) WORKDD
      RETURN
C
  600 FORMAT(A137)
      END
C=======================================================================
      SUBROUTINE DILATE (BP, ELW, AKAP, DU, PHT, TAUEND, DIF, SRA,
     1                    GMAX, PHAI, TAU0, TAU1, GAM1, SIGV0, SIGV99,
     2                    LRFTYP,ROSTRN,ROBETA,IFGEF,LQTIME,
     3                    GAMOR, TIME, LOC, IC, NL, IFL, NW)
C=======================================================================
C
C     "BP"       (NW)       PARAMETER FOR PORE PRESSURE GENERATION
C     "ELW"      (NW)       PARAMETER FOR PORE PRESSURE GENERATION
C     "AKAP"     (NW)       PARAMETER FOR PORE PRESSURE GENERATION
C     "DU"       (NW)       PORE PRESSURE INCREMENT IN UNDRAINED
C                            CONDITIONS
C     "PHT"      (NW)       TAN(ANGLE OF PHASE TRANSFORMATION)
C     "TAUEND"   (NW)       STRESS AT THE LATE REVERSAL POINT
C     "DIF"      (NW)       DIRECTION OF STRESS CHANGE
C     "SRA"      (2, NW)    MAXIMUM AND MINIMUM STRESS RATIOS
C     "GMAX"     (N)        INITIAL TANGENT MODULUS
C     "PHAI"     (N)        TAN(FRICTION ANGLE)
C     "TAU0"     (N)        OLD SHEARING STRESS
C     "TAU1"     (N)        NEW SHEARING STRESS
C     "GAM1"     (N)        NEW SHEARING STRAIN
C     "SIGV0"    (N)        OLD EFFECTIVE VERTICAL STRESS
C     "SIGV99"   (N)        MINIMUM VALUE OF VERTICAL STRESS
C     "LRFTYP"   (N)        TYPES OF RESTORING FORCE CHARACTERISTICS
C     "ROSTRN"   (N)        ULTIMATE STRAIN IN R.O. MODEL
C     "ROBETA"   (N)        BETA IN R.O. MODEL
C     "IFGEF"    (N)        FLAG OF TAKING INTO ACCOUNT GENERATION PP
C     "GAMOR"    (N)        STRAIN AT THE ORIGIN
C     "TIME"                  TIME
C     "LOC"      (NW)       TYPE OF PORE PRESSURE GENERATION
C     "NL"       (NW)       NL(J) = N - NW + J
C     "IC"       (N)        NUMBER OF HISTERESIS LOOP
C     "IFL"      (N)        IF LAYER-J HAS LIQUEFIED, IFL(J) = 1.
C     "NW"                    NUMBER OF SATURATED LAYERS
C     "INCLL"                 NUMBER OF COMPLETELY LIQUEFIED LAYERS
C
C      "DILATE" CALCULATES PORE PRESSURE INCREMENT "DU" IN UNDRAINED
C
C     CONDITIONS.
C
      IMPLICIT REAL*8 (A-H, O-Z)
      COMMON /SAI/ INCLL
     1       /BUTA/ PM, IPHI
      DIMENSION  BP(*), ELW(*), AKAP(*), DU(*), PHT(*),
     1           TAUEND(*), DIF(*), SRA(2,*), TAU0(*), TAU1(*),
     2           SIGV0(*), PHAI(*), GAM1(*), GAMOR(*), GMAX(*),
     3           SIGV99(*), LOC(*), NL(*), IFL(*), IC(*),
     4           LRFTYP(*), ROSTRN(*), ROBETA(*), IFGEF(*)
      REAL *8 LQTIME(2,*)
      DO 230  J = 1, NW
      IOM = 0
      NLJ = NL(J)
      IF(IFGEF(NLJ) .NE. 0) GO TO 220
      IF (ABS(LOC(J)) - 2)  1, 90, 160
    1 IF (ELW(J) .LE. 0.D0)  GO TO 50
      IF (SIGV0(NLJ) .LE. AKAP(J))  GO TO 40
C
C     BEFORE INITIAL LIQUEFACTION WITH ENOUGH LARGE EFFECTIVE VERTICAL
C     STRESS
C
      EL = TAU1(NLJ) - TAU0(NLJ)
      IF (EL * DIF(J) .GE. 0.D0)  GO TO 10
      TAUEND(J) = TAU0(NLJ)
      DIF(J) = EL
   10 EL = LOG(SIGV0(NLJ)-AKAP(J)) 
     1       + ELW(J) * (TAU0(NLJ) -TAUEND(J))**2
      SIGV1 = AKAP(J) + EXP(EL-ELW(J)*(TAU1(NLJ)-TAUEND(J))**2)
      EM = TAU1(NLJ) / SIGV1
      IF (EM .GT. SRA(1,J))  GO TO 30
      IF (EM .LT. SRA(2,J))  GO TO 20
      LOC(J) = 0
      GO TO 190
C
C     YIELD POINT
C
   20 CALL CROSS(EL, SRA(2,J), ELW(J), SIGV0(NLJ), TAUEND(J), AKAP(J))
      GO TO 70
   30 CALL CROSS(EL, SRA(1,J), ELW(J), SIGV0(NLJ), TAUEND(J), AKAP(J))
      GO TO 60
C
C     BEFORE PHASE TRANSFORMATION
C
   40 ELW(J) = -1.D0
   50 EL = SIGV0(NLJ)
      EM = TAU1(NLJ) / EL
      IF (EM .GT. SRA(1,J))  GO TO 60
      IF (EM .LT. SRA(2,J))  GO TO 70
      DU(J) = 0.D0
      LOC(J) = 0
      GO TO 230
C
   60 EM = EL * (0.5D0 + SQRT(0.25D0 + BP(J) * SRA(1,J)**2))
      EL = BP(J) / EM
      SIGV1 = EM - EL * TAU1(NLJ)**2
      IF (TAU1(NLJ) - SIGV1*PHT(J) .GE. 0.D0)  GO TO 80
      SRA(1,J) = TAU1(NLJ) / SIGV1
      LOC(J) = 1
      GO TO 190
C
   70 EM = EL * (0.5D0 + SQRT(0.25D0+BP(J)*SRA(2,J)**2))
      EL = BP(J) / EM
      SIGV1 = EM - EL * TAU1(NLJ)**2
      IF (TAU1(NLJ) + SIGV1*PHT(J) .LE. 0.D0)  GO TO 80
      SRA(2,J) = TAU1(NLJ) / SIGV1
      LOC(J) = -1
      GO TO 190
   80 LOC(J) = 2
      IOM = 1
      LQTIME(1,NLJ) = TIME
      EM = PHT(J) **2
      EL = (-0.5D0 + SQRT(0.25D0 + BP(J) * EM)) / (EL * EM)
C%%
      IF (IPHI .EQ. 0) THEN
        PHAI(NLJ) = PM * PHT(J)
      ENDIF
C%%
      EM = (EL**2) * (1.D0-(PHT(J)/PHAI(NLJ))**2)
      PHT(J) = 1.D0 / PHAI(NLJ) **2
      GO TO 180
C
C     AFTER PHASE TRANSFORMATION
C
   90 IF (TAU1(NLJ) * TAU0(NLJ) .GE. 0.D0)  GO TO 130
      IF (LOC(J) .EQ. 2)  BP(J) = TAU0(NLJ)*PHT(J)/SIGV0(NLJ)
      EL = SIGV0(NLJ) - TAU0(NLJ) * BP(J)
      LOC(J) = 2
      IF (EL .GT. SIGV99(NLJ))  GO TO 120
C
C     COMPLETELY LIQUEFIED!!
C
  100 IF (IFL(NLJ) .EQ. 1)  GO TO 110
      LQTIME(2,NLJ) = TIME
      INCLL = INCLL + 1
  110 LOC(J) = 3
      IFL(NLJ) = 1
      EL = SIGV99(NLJ)
C
  120 EM = EL * EL
      GO TO 180
  130 IF (TAU1(NLJ)**2 .GE. TAU0(NLJ)**2)  GO TO 150
      IF (LOC(J) .EQ. -2)  GO TO 140
      LOC(J) = -2
      BP(J) = TAU0(NLJ) * PHT(J) / SIGV0(NLJ)
  140 SIGV1 = SIGV0(NLJ) + BP(J)*(TAU1(NLJ)-TAU0(NLJ))
      IF (SIGV1 .LE. SIGV99(NLJ))  GO TO 100
      GO TO 190
  150 IF (LOC(J) .EQ. -2)  LOC(J) = 2
  160 EM = SIGV0(NLJ)**2 - PHT(J) * TAU0(NLJ)**2
      EL = SIGV99(NLJ) **2
      IF (EM .LE. 1.2D0 * EL .OR. LOC(J) .NE. 3)  GO TO 170
      LOC(J) = 2
      GO TO 90
  170 IF (EM .LE. 0.D0)  EM = EL
  180 SIGV1 = SQRT(EM + PHT(J) * TAU1(NLJ)**2)
C
  190 DU(J) = SIGV0(NLJ) - MAX(SIGV1, SIGV99(NLJ))
      IF (IOM .EQ. 0)  GO TO 230
C%%
      IF (IPHI .EQ. 1) GO TO 230
C%%
      EL = SIGV1 * PHAI(NLJ)
      IF(LRFTYP(NLJ) .EQ. 1) GO TO 200
C  ----- RAMBERG OSGOOD MODEL
C##      GAMOR(NLJ) = GAM1(NLJ) - TAU1(NLJ) / GMAX(NLJ) *
C##     1           (1. + (ROSTRN(NLJ) / (EL / GMAX(NLJ)) - 1.)
C##     2           * DABS(TAU1(NLJ) / EL) ** (ROBETA(NLJ) - 1.D0))
      GAMOR(NLJ) = GAM1(NLJ) - TAU1(NLJ) / GMAX(NLJ) *
     1           (1. + ROSTRN(NLJ)
     2           * DABS(TAU1(NLJ) / EL) ** (ROBETA(NLJ) - 1.D0))
      GO TO 210
C  ----- HARDIN DRNEVICH MODEL
  200 EM = GMAX(NLJ) * (EL - DABS(TAU1(NLJ)))
      GAMOR(NLJ) = GAM1(NLJ) - EL * TAU1(NLJ) / EM
  210 IC(NLJ) = 0
      GO TO 230
  220 DU(J) = 0.
  230 CONTINUE
      RETURN
      END
C=======================================================================
      SUBROUTINE ELSOL (IFLG, A, B, CML, AL, ICG, ND, NLD, MP)
C=======================================================================
C
C     "A"        (MP*ND)    BAND MATRIX OF COEFFICIENTS
C     "B"        (ND)       RIGHT-HAND SIDE VECTOR
C     "CML"      (ND)       SCALING FACTORS
C     "AL"       (NLD*ND-NLD*(NLD+1)/2)
C                             WORKING REGION
C     "ICG"      (ND-1)     WORKING REGION
C     "ND"                    ORDER OF MATRIX
C     "NLD"                   NUMBER OF SUB-DIAGONAL LINES OF NON-ZERO
C                            ELEMENTS OF "A"
C     "NUD"                   NUMBER OF SUPER-DIAGONAL LINES OF NON-ZERO
C                            ELEMENTS OF "A"
C     "MP"                    MP = MIN(ND, NLD+NUD+1)
C
C      "ELSOL" SOLVES SIMULTANEOUS LINEAR EQUATIONS, A*X=B, OF BAND
C     MATRIX BY GAUSSIAN ELIMINATION.  "A" IS ASSUMED NON-SINGULAR.
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION  A(1), B(ND), CML(ND), AL(1), ICG(ND)
      LT(KARASU) = MP * (KARASU - 1)
C
      IF (NLD .EQ. 0)  GO TO 200
      IF (IFLG .EQ. 2) GO TO 131
C
C     SCALING OF "A"
C
      I = 1 - MP
      J = 0
      DO  30  IDB = 1, ND
          I = I + MP
          J = J + MP
          W = 0.D0
          DO  10  K = I, J
   10     W = MAX(W, ABS(A(K)))
          W = 1.D0 / W
          CML(IDB) = W
          DO  20  K = I, J
   20     A(K) = A(K) * W
   30 CONTINUE
C
C     DETERMINATION OF A PIVOT (PARTIAL PIVOTTING)
C
      IN0 = NLD
      IAL = 0
C
      DO 130  IDB = 1, ND - 1
C
      J = 1 + LT(IDB)
      PIVOT = A(J)
      IPIVOT = IDB
      ABMAX = ABS(PIVOT)
      IN0 = MIN(ND, IN0 + 1)
      DO  40  I = IDB + 1, IN0
          J = J + MP
          IF (ABS(A(J)) .LE. ABMAX)  GO TO  40
          PIVOT = A(J)
          ABMAX = ABS(PIVOT)
          IPIVOT = I
   40 CONTINUE
      ICG(IDB) = IPIVOT
C
C     EXCHANGING OF ROWS
C
      MEND = MIN(MP - 1, ND - IDB)
      IF (IDB .EQ. IPIVOT)  GO TO 100
      I = 1 + LT(IDB)
      J = I + MEND
      K = LT(IPIVOT)
      DO  50  L = I, J
          K = K + 1
          W = A(L)
          A(L) = A(K)
          A(K) = W
   50 CONTINUE
C
C     ELIMINATION AND SETTING A(1+LT(I)) = THE FIRST OF NON-ZERO
C                                           ELEMENTS IN ROW-I
  100 I = 2 + LT(IDB)
      JJ = I - 1
      IZERO = 1
      IF (MEND .LT. ND - IDB)  IZERO = 0
      DO 120  K = IDB + 1, IN0
          JJ = JJ + MP
          J = JJ
          W = -A(J) / PIVOT
          L = I
          IAL = IAL + 1
          AL(IAL) = W
          DO 110  M = 1, MEND
              A(J) = A(J+1) + A(L) * W
          J = J + 1
          L = L + 1
  110     CONTINUE
          IF (IZERO .EQ. 0)  A(J) = 0.D0
  120 CONTINUE
C
  130 CONTINUE
C
  131 CONTINUE
C
C     SCALING OF "B"
C
      DO 140  IDB = 1, ND
  140 B(IDB) = B(IDB) * CML(IDB)
C
C     EXCHANGING AND MULTIPLYING OF "B"
C
      IAL = 0
      IN0 = NLD
C
      DO 170  IDB = 1, ND - 1
          IN0 = MIN(ND, IN0 + 1)
          W = B(ICG(IDB))
          IF (IDB .EQ. ICG(IDB))  GO TO 150
          B(ICG(IDB)) = B(IDB)
          B(IDB) = W
C
  150     DO 160  K = IDB + 1, IN0
              IAL = IAL + 1
              B(K) = B(K) + W * AL(IAL)
  160     CONTINUE
  170 CONTINUE
C
C     SOLUTION "B"
C
  200 I = 1 + LT(ND)
      B(ND) = B(ND) / A(I)
      IF (ND .EQ. 1)  RETURN
      DO 230  J = 1, ND - 1
          IDB = ND - J
          I = 1 + LT(IDB)
          MEND = MIN(J, MP - 1)
          W = 0.D0
          IF (MEND .EQ. 0)  GO TO 220
          DO 210  M = 1, MEND
  210     W = W + A(I+M) * B(IDB+M)
  220     B(IDB) = (B(IDB) - W) / A(I)
  230 CONTINUE
C
      RETURN
      END
C=======================================================================
      SUBROUTINE HDLOOP (TAUF, GAMR, TAU0, GAM0, TAU1, GAM1,
     1                    TAUREV, GAMREV, GAMOR, LRFTYP, ROSTRN,
     2                    ROBETA, IC, ICC, N, NREV)
C=======================================================================
C
C     "TAUF"     (N)        SHEARING STRENGTH OF LAYERS
C     "GAMR"     (N)        REFERENCE STRAIN
C     "TAU0"     (N)        OLD VALUES OF SHEARING STRESS IN LAYERS
C     "GAM0"     (N)        OLD VALUES OF SHEARING STRAIN IN LAYERS
C     "TAU1"     (N)        SHEARING STRESS CORRESPONDING TO "GAM1"
C                                                 (COMPUTED HERE)
C     "GAM1"     (N)        NEW VALUES OF SHEARING STRAIN IN LAYERS
C     "TAUREV"   (NREV,N)   STRESS AT REVERSAL POINTS IN STRAIN HISTOR
C     "GAMREV"   (NREV,N)   STRAIN AT REVERSAL POINTS IN STRAIN HISTOR
C     "GAMOR"    (N)        STRAIN AT THE ORIGIN
C     "IC"       (N)        THE NAME OF THE HISTERESIS LOOP WITH "GAM0
C     "ICC"      (N)        THE NAME OF THE HISTERESIS LOOP WITH "GAM1
C                                                 (COMPUTED HERE)
C     "N"                     NUMBER OF LAYERS
C     "NREV"                  LIMIT OF THE NUMBER OF REVERSAL POINTS
C
C     "GAM1" GIVEN, THIS SUBROUTINE DETERMINES ON WHICH HISTERESIS LOOP
C    (GAM1, TAU1) IS, AND COMPUTES "TAU1".
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION  TAUF(*), GAMR(*), TAU0(*), GAM0(*), TAU1(*), GAM1(*),
     1           TAUREV(NREV,*), GAMREV(NREV,*), GAMOR(*),
     2           IC(*), ICC(*), LRFTYP(*), ROSTRN(*), ROBETA(*)
C
      DO 200 J=1,N
      ICC(J) = IC(J)
      IF (ICC(J) .GE. 2)  GO TO 100
C
C     IN VIRGIN LOADING
C
      Q1 = GAM1(J) - GAMOR(J)
      IF (DABS(Q1) .GE. DABS(GAM0(J) - GAMOR(J)))  GO TO 10
C
C     REVERSAL HAS HAPPENED.
C
      ICC(J) = 2
      CALL SSCURV (LRFTYP(J),TAU1(J),GAM1(J),TAUF(J),GAMR(J),
     1             TAU0(J),GAM0(J),0.5,ROBETA(J),ROSTRN(J))
      GO TO 200
C
C     'TAU1' IN VIRGIN LOADING
C
   10 CALL SSCURV (LRFTYP(J),TAU1(J),GAM1(J),TAUF(J),GAMR(J),
     1             0.D0,GAMOR(J),1.0,ROBETA(J),ROSTRN(J))
      GO TO 200
C
C     IN HISTERESIS LOOP
C
  100 Q1 = GAM0(J) - GAM1(J)
      Q2 = GAM1(J) - GAMREV(ICC(J),J)
      IF (Q1 * Q2 .GT. 0.D0)  GO TO 110
      Q1 = Q1 + Q2
      IF (Q1 * Q2 .LE. 0.D0)  GO TO 130
      ICCM = ICC(J) - 1
      Q1 = GAM0(J) - GAMREV(ICCM,J)
      Q2 = GAMREV(ICCM,J) - GAM1(J)
      IF (Q1 * Q2 .GE. 0.D0)  GO TO 160
C
C     "ICC" PRESERVED
C
      CALL SSCURV (LRFTYP(J),TAU1(J),GAM1(J),TAUF(J),GAMR(J),
     1             TAUREV(ICC(J),J),GAMREV(ICC(J),J),0.5,
     2             ROBETA(J),ROSTRN(J))
      GO TO 200
C
C     REVERSAL HAS HAPPENED.
C
  110 ICC(J) = ICC(J) + 1
      IF (ICC(J) .GT. NREV)  GO TO 120
      CALL SSCURV (LRFTYP(J),TAU1(J),GAM1(J),TAUF(J),GAMR(J),
     1             TAU0(J),GAM0(J),0.5,ROBETA(J),ROSTRN(J))
      GO TO 200
C
C     "ICC" .GT. "NREV"
C
  120 ICC(J) = ICC(J) - 2
      CALL SSCURV (LRFTYP(J),TAU1(J),GAM1(J),TAUF(J),GAMR(J),
     1             TAUREV(ICC(J),J),GAMREV(ICC(J),J),0.5,
     2             ROBETA(J),ROSTRN(J))
      GO TO 200
C
C     "GAMREV(ICC)" BETWEEN "GAM1" AND "GAM0"
C
  130 IF (ICC(J) .GE. 3)  GO TO 140
C
C     INTO THE SKELETON CURVE
C
      ICC(J) = 0
      CALL SSCURV (LRFTYP(J),TAU1(J),GAM1(J),TAUF(J),GAMR(J),
     1             0.D0,GAMOR(J),1.0,ROBETA(J),ROSTRN(J))
      GO TO 200
C
C     IN LOOP
C
  140 Q1 = GAMREV(ICC(J),J) - GAM1(J)
      Q2 = GAM1(J) - GAMREV(ICC(J) - 2,J)
      IF (Q1 * Q2 .GT. 0.D0  .OR.  GAM1(J) .EQ.
     1     GAMREV(ICC(J),J))  GO TO 150
      ICC(J) = ICC(J) - 2
      GO TO 130
C
  150 ICC(J) = ICC(J) - 1
      CALL SSCURV (LRFTYP(J),TAU1(J),GAM1(J),TAUF(J),GAMR(J),
     1             TAUREV(ICC(J),J),GAMREV(ICC(J),J),0.5,
     2             ROBETA(J),ROSTRN(J))
      GO TO 200
C
C     "GAMREV(ICC-1)" BETWEEN "GAM0" AND "GAM1"
C
  160 IF (ICC(J) .GE. 4)  GO TO 170
C
C     INTO THE SKELETON CURVE
C
      ICC(J) = 0
      CALL SSCURV (LRFTYP(J),TAU1(J),GAM1(J),TAUF(J),GAMR(J),
     1             0.D0,GAMOR(J),1.0,ROBETA(J),ROSTRN(J))
      GO TO 200
C
C     IN LOOP
C
  170 Q1 = GAMREV(ICC(J) - 1,J) - GAM1(J)
      Q2 = GAM1(J) - GAMREV(ICC(J) - 3,J)
      IF (Q1 * Q2 .GT. 0.D0  .OR.  GAM1(J) .EQ.
     1     GAMREV(ICC(J) - 1,J))  GO TO 180
      ICC(J) = ICC(J) - 2
      GO TO 160
C
  180 ICC(J) = ICC(J) - 2
      CALL SSCURV (LRFTYP(J),TAU1(J),GAM1(J),TAUF(J),GAMR(J),
     1             TAUREV(ICC(J),J),GAMREV(ICC(J),J),0.5,
     2             ROBETA(J),ROSTRN(J))
C
  200 CONTINUE
      RETURN
      END
C=======================================================================
      SUBROUTINE MASET (B, DZ, A, P, N)
C=======================================================================
C
C     "B"        (N)        1  TANGENT MODULUS
C                             2  VISCOSITY
C     "DZ"       (N)        THICKNESS OF LAYERS
C     "A"        (14N)      WORKING REGION
C     "P"        (3, N)     1  MATRIX "K"
C                             2  MATRIX "C"       (COMPUTED HERE)
C     "N"                     NUMBER OF LAYERS
C
C     WITH "B" AND "DZ" GIVEN, THIS SUBROUTINE COMPUTES THE VALUES OF TH
C
C    MEMBERS OF "P".
C
      IMPLICIT REAL*8 (A-H, O-Z)
      COMMON /KUMA/  NM1, NP1, NP2
      DIMENSION  B(*), DZ(*), A(*), P(3,*)
C
      IF (N-2)  10, 20, 30
C
   10 P(1, 1) = B(1) / DZ(1)
      RETURN
C
   20 A(1) = B(1) / DZ(1)
      A(2) = B(2) / DZ(2)
      P(1, 1) = A(1)
      P(2, 1) = - A(1)
      P(1, 2) = - A(1)
      P(2, 2) = A(1) + A(2)
      RETURN
C
   30 A(1) = B(1) / DZ(1)
      P(1, 1) = A(1)
      P(2, 1) = - A(1)
      DO 40  J = 2, N - 1
      A(J) = B(J) / DZ(J)
      P(1, J) = - A(J-1)
      P(2, J) = A(J-1) + A(J)
      P(3, J) = - A(J)
   40 CONTINUE
      A(N) = B(N) / DZ(N)
      P(1, N) = - A(NM1)
      P(2, N) = A(NM1) + A(N)
      RETURN
      END
C=======================================================================
      SUBROUTINE MISET (RHO, DZ, A, M, MS, N)
C=======================================================================
C
C     "RHO"      (N)        MASS DENSITY
C     "DZ"       (N)        THICKNESS OF LAYERS
C     "A"        (N)        WORKING REGION
C     "M"        (3,N+1)    LUMPED MASS MATRIX
C     "MS"                    POINT MASS AT THE SURFACE
C
C     "MISET" CALCULATES LUMPED MASS MATRIX.
C
      IMPLICIT REAL*8 (A-H, O-Z)
      REAL*8  M(3,1), MS
      COMMON /KUMA/  NM1, NP1, NP2
      DIMENSION  RHO(*), DZ(*), A(*)
      DO  10  J = 1, N
   10 A(J) = 0.5D0 * RHO(J) * DZ(J)
      M(1, 1) = MS + A(1)
      M(2, NP1) = A(N)
      IF (N .EQ. 1)  RETURN
      DO  20  J = 2, N
   20 M(2, J) = A(J) + A(J - 1)
      RETURN
      END
C=======================================================================
      SUBROUTINE PREMUL (A, X, Y, ND, NLD, NUD)
C=======================================================================
C
C     "A"      (MP*ND)      BAND MATRIX
C     "X"       (ND)        VECTOR PREMULTIPLIED BY "A"
C     "Y"       (ND)        Y = A * X
C     "ND"                    DIMENSION
C     "NLD"                   SIZE OF "A"
C     "NUD"                   SIZE OF "A"
C
C     "PREMUL" CALCULATES "Y" = (A) * "X".
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION  A(*), X(*), Y(*)
      MP = MIN(ND, NLD + NUD + 1)
      IS = 1 - MP
      DO  20  IDB = 1, ND
          IS = IS + MP
          JS = MAX( 1, IDB - NLD)
          JE = MIN(ND, IDB + NUD)
          I = IS
          W = 0.D0
          DO  10  J = JS, JE
              W = W + A(I) * X(J)
              I = I + 1
   10     CONTINUE
          Y(IDB) = W
   20 CONTINUE
      RETURN
      END
C=======================================================================
      SUBROUTINE PROK(XM, DZ, GAMREV, AMAX, N, NW, NREV, INCLL,
     1                 TITLE, LDTTL, DT, LQTIME)
C=======================================================================
C
C     "XM"      MAXIMUM VALUES AND TIME
C               1,2  ABSOLUTE ACCELERATION
C               3,4  SHEAR STRESS
C               5,6  SHEAR STRAIN
C               7,8  EXCESS PORE PRESSURE
C               9,10  RELATIVE VELOCITY
C               11,12 RELATIVE DISPLACEMENT
C               13,14 MINIMUM OF (GMAX)
C               IN "N01"TH LAYER, (3,4) ARE RELATIVE ERROR IN STRESS AND
C                     (5,6) ARE TIME WHERS MAX. VALUES ARE LOOKED FOR.
C
C     "DZ"      DEPTH OF LAYER
C     "GAMREV"  STRAIN REVERSAL POINT
C     "AMAX"    MAXIMUM INPUT ACCELARATION
C     "INCLL"   NUMBER OF LAYERS WHICH WAS COMPLETELY LIQUEFIED
C     "TITLE"   TITLE OF THIS CALCULATION
C     "LDTTL"   EARTHQUAKE NAME
C     "DT"      TIME INCREMENT FOR TIME INTEGRATION
C     "LQTIME"  TIME AT PHASE TRANSFORM OR COMPLETE LIQUEFACTION
C
C     "PROK" PRINT ABSOLUTELY MAXIMUM VALUE FOR FINAL RESULT, AND
C     "PROKC" PRINT ABSOLUTELY MAXIMUM VALUES IN CALCULATION
C
      REAL *8  XM(14,1),DZ(1),GAMREV(NREV,1),AMAX,DT,LQTIME(2,1)
      CHARACTER*60 TITLE
      CHARACTER *20 LDTTL
      CHARACTER *1  SPS
      DATA MNREV ,SPS  / 0 , 1H   /
C
      WRITE(6,680) TITLE,LDTTL,XM(6,N+1),AMAX,DT
      WRITE(6,690)
C
      DEP = 0
C
      DO 10 J = 1,N
      WRITE(6,620) J,DEP,XM(1,J),XM(2,J),(XM(K,J),K=9,12)
      DEP = DEP + DZ(J)
   10 CONTINUE
      J = N + 1
      WRITE(6,640) J,DEP,XM(1,J),XM(2,J),(XM(K,J),K=9,12)
      WRITE(6,680) TITLE,LDTTL,XM(6,N+1),AMAX,DT
      IF(NW .GT. 0) WRITE(6,740)
      IF(NW .LE. 0) WRITE(6,745)
      DO 50 J = 1,N
C
C     CHECK OF MAXIMUM NUMBER OF STRAIN REVERSALS
      I = NREV
   20 IF(DABS(GAMREV(I,J)) .LT. 1.D-20) GO TO 30
      MNREV = MAX0(MNREV,I)
      GO TO 40
   30 I = I - 1
      IF(I .GE. 2) GO TO 20
C
   40 IF(NW .LE. 0) GO TO 45
      I = 0
      IF(LQTIME(1,J) .GT. 0.D0) I = 1
      IF(LQTIME(2,J) .GT. 0.D0) I = 2
      IF(I .EQ. 0) WRITE(6,750) J,DZ(J),(XM(K,J),K=3,8),
     1                          (XM(K,J),K=13,14)
      IF(I .GT. 0) WRITE(6,750) J,DZ(J),(XM(K,J),K=3,8),
     1                 (XM(K,J),K=13,14),(LQTIME(K,J),K=1,I)
      GO TO 50
   45 WRITE(6,750) J,DZ(J),(XM(K,J),K=3,6),(XM(K,J),K=13,14)
   50 CONTINUE
      WRITE(6,760) XM(3,N+1),XM(4,N+1)
C
      WRITE(6,700) MNREV
      IF(MNREV .EQ. NREV) WRITE(6,710) NREV
C
      IF(NW .GT. 0) WRITE(6,720) INCLL
C
      WRITE(25,800) (XM(1,J),XM(11,J),(XM(K,J),K=3,7,2),J=1,N)
      J = N + 1
      WRITE(25,800) XM(1,J),XM(11,J)
      WRITE(24) (SNGL(XM(1,J)),SNGL(XM(11,J)),
     1          (SNGL(XM(K,J)),K=3,7,2),J=1,N)
      J = N + 1
      WRITE(24) SNGL(XM(1,J)),SNGL(XM(11,J))
      RETURN
C
C-----------------------------------------------------------------------
      ENTRY PROKC(XM, DZ, N, NW)
C-----------------------------------------------------------------------
C
      WRITE(6,600) XM(5,N+1),XM(6,N+1)
      IF(NW .GT. 0) WRITE(6,610) SPS
      IF(NW .LE. 0) WRITE(6,610)
      IF(NW .GT. 0) LL = 8
      IF(NW .LE. 0) LL = 6
      DEP = 0.
      DO 110 I = 1,N
      WRITE(6,630) I,DEP,XM(1,I),XM(9,I),XM(11,I),
     1             I,DZ(I),(XM(J,I),J=3,LL,2)
      WRITE(6,650) XM(2,J),XM(10,J),XM(12,J),(SPS,XM(K,J),K=4,LL,2)
      DEP = DEP + DZ(I)
  110 CONTINUE
      I = N + 1
      WRITE(6,660) I,DEP,XM(1,I),XM(9,I),XM(11,I)
      WRITE(6,670) XM(2,I),XM(10,I),XM(12,I),XM(3,I),XM(4,I)
      RETURN
C
  600 FORMAT(///,1X,40(1H*),' Maximum values (',F7.3,2H -,F7.3,
     1       ' second) ',49(1H*))
  610 FORMAT(/,'    NP    Depth    Aba Acc     Rel Vel     Rel Dis ',2X,
     1       '   LN   Thick     Stress      Strain ',A1,'  Excess PP ')
  620 FORMAT(1X,I5,F9.3,3(E12.4,F7.3,2X))
  630 FORMAT(1X,I5,F9.3,3E12.4E2,I7,F8.3,3E12.4E2)
  640 FORMAT(1X,I5,F9.3,3(E12.4,F7.3,2X),'(Absolute value)')
  650 FORMAT(16X,3(3H  (,F7.3,2H) ),15X,3(A1,2H (,F7.3,2H) ))
  660 FORMAT(1X,I5,F9.3,3E12.4E2,2X,'(Absolute value)')
  670 FORMAT(16X,3(3H  (,F7.3,2H) ),10X,
     1      'Relative error in stress',E12.4E2,'  AT ',F8.3,' Second')
  680 FORMAT(/////,5X,'Maximum response values',17X,
     1       'TITLE',5X,A60,/,46X,'Earthquake',
     2       3X,A20,5X,'Duration =',F10.3,' Second',/
     3       59X,'Max =',E12.4,8X,'time increment =',F6.3,' Second')
  690 FORMAT(/,' Nodal            Absolute             Relative ',
     1       '            Relative          ',/
     2       ' Point    Depth   Accel.      Time     Velocity ',
     3       '   Time     Displ.      Time',/)
  700 FORMAT(//,'    Maximum number of strain reversals =',I4)
  710 FORMAT('    Max. area of prepared strain reversals (',I4,
     1       ') may be too small ')
  720 FORMAT(I4,' Layers are completely liquefied')
  740 FORMAT(/,' Layer  Thick-    Shear                Shear   ',
     1       '             Excess          ',
     2      '     Minimum               Phase    Compl.',/
     3       '   No.   ness     Stress     Time      Strain  ',
     4       '   Time      Ppre P.    Time ',
     5      '     Gmax       Time      Transf.    Liqu.',/)
  745 FORMAT(/,' Layer  Thick-    Shear                Shear   ',
     1      '             Minimum ',/
     2       '   No.   ness     Stress     Time      Strain  ',
     3      '   Time      Gmax       Time   ',/)
  750 FORMAT(1X,I5,F8.3,4(E12.4,F7.3,2X),2X,2F9.3)
  760 FORMAT(/,' Relative error in stress',E12.4E2,'  at ',F8.3,
     1      ' Second')
  800 FORMAT((5E15.8))
      END
C=======================================================================
      SUBROUTINE PROP (GMAX, PR, TAUF, GAMR, FA, FT, FG, FS, FP,
     1                  FV, FD, TIME, WTRSF, NW, N, N01, IWL)
C=======================================================================
C
C     "FA"                    ACCELERATION
C     "FT"                    SHEARING STRESS
C     "FG"                    SHEARING STRAIN
C     "FS"                    EFFECTIVE VERTICAL STRESS AT THE MIDDLE
C     "FP"                    EXCESS PORE WATER PRESSURE AT THE MIDDLE
C     "FV"                    RELATIVE VELOCITY
C     "FD"                    RELATIVE DISPLACEMENT
C                             PRINTED.
C
C     "PROP" PRINTS CONDITIONS AT "TIME".
C
      IMPLICIT REAL*8 (A-E,G,H,O-Z)
      DIMENSION  GMAX(*), PR(*), TAUF(*), GAMR(*), FA(*), FT(*), FG(*),
     1           FS(*),   FP(*),  FV(*),  FD(*)
      IF(NW .GT. 0) WRITE(6,600)  TIME
      IF(NW .LE. 0) WRITE(6,610)  TIME
      DO 100  J = 1, N
      IF(NW .GT. 0) WRITE(6,620)  J,FA(J),FV(J),FD(J),J,FT(J),
     1              TAUF(J),FG(J),GAMR(J),FP(J),FS(J),GMAX(J),PR(J)
      IF(NW .LE. 0) WRITE(6,620)  J,FA(J),FV(J),FD(J),J,FT(J),
     1              TAUF(J),FG(J),GAMR(J)
  100 CONTINUE
      WRITE(6,630)  N01, FA(N01), FV(N01), FD(N01)
      IF (NW .GT. 0)  WRITE(6,640)  IWL, WTRSF
      RETURN
C
  600 FORMAT(//,' ',50(1H*),F8.3,8H Second ,65(1H*),//,
     1      '    NP   Abs Acc    Rel Vel    Rel Did    LN    Tau    ',
     2      '    Tauf      Gamma       Gamr     Excess PP   Sigv''   ',
     3      '    Gmax     Porosity ')
  610 FORMAT(//,' ',50(1H*),F8.3,8H Second ,65(1H*),//,
     1      '    NP   Abs Acc    Rel Vel    Rel Dis    LN    Tau    ',
     2      '    Tauf      Gamma       Gamr ')
  620 FORMAT(1X,I5,3E11.4E2,I5,8E11.4E2)
  630 FORMAT(1X,I5,3E11.4E2,2X,'(Absolute value)')
  640 FORMAT (1H+,80X,4HIWL=, I2,9H  WTRSF =,1PD12.3)
      END
C=======================================================================
      SUBROUTINE QUAKE (N, NREV, BP, CS, PHT, ELW, AKAP ,DIF, TAUEND,
     1             DU , SRA , LOC , IFL , NL , GMAX , G , RHO , RHOS ,
     2             DZ , PR , SEP , PHAI , TAUF , GAMR , K0 , CA , CB ,
     3             CC , GAM0 , GAM1 , TAU0 , TAU1 , SIGV0, SIGV00 ,
     4             SIGV99 , GAMOR , GAMREV , TAUREV , IC , ICC , ICGM ,
     5             LRFTYP , ROSTRN , ROBETA , COHE , IFGEF , U0 , V0 ,
     6             A0 , U1 , V1 , A1 , CMAX , TMAX , FA , FV , FD ,
     7             FT , FG , FS , FP , Z , WORK, NW)
C=======================================================================
C
C     BP(NW)        PARAMETER FOR PARABOLIC TAU-SIGMA CURVE
C     CS(NW)        CONSTRAINED COMPRESSIBILITY OF SAND SKELETON IN
C                  UNLOADING  *1.5
C     DU(NW)        PORE-PRESSURE-INCREMENT IN UNDRAINED CONDITIONS
C     PHT(NW)       TANGENT OF THE ANGLE OF PHASE TRANSFORMATION
C     ELW(NW)       PARAMETER FOR PORE WATER GENERATION IN CYCLIC
C                  SHEARING
C     AKAP(NW)      PARAMETER FOR PORE WATER GENERATION IN CYCLIC
C                  SHEARING
C     TAUEND(NW)    PARAMETER FOR PORE WATER GENERATION IN CYCLIC
C                  SHEARING
C     DIF(NW)       PARAMETER FOR PORE WATER GENERATION IN CYCLIC
C                  SHEARING
C     GMAX(N)       INITIAL TANGENT SHEARING MODULUS
C     G(N)          TANGENT SHEARING MODULUS
C     VIS(N)        VISCOSITY OF SOIL
C     RHO(N)        UNIT DENSITY OF SOIL
C     RHOS(N)       UNIT DENSITY OF SOLID PARTICLES
C
C     DZ(N)         THICKNESS OF LAYERS
C     PR(N)         POROSITY
C     SEP(N)        PERMEABILITY
C     PHAI(N)       FRICTION ANGLE
C     K0(N)         COEFFICIENT OF EARTH PRESSURE AT REST
C     CA(N)         PARAMETERS FOR
C     CB(N)                THE DETERMINATION
C     CC(N)                            OF "GMAX"
C     TAUF(N)       SHEARING STRENGTH
C     GAMR(N)       REFERENCE STRAIN
C     GAMOR(N)      STRAIN OF THE ORIGIN
C     LRFTYP(N)     TYPE OF RESTORING FORCE CHARACTERISTICS
C                   1 FOR HARDIN DRNEVICH MODEL
C                   2 FOR RAMBERG OSGOOD MODEL
C     ROSTRN(N)     ULTIMATE STRAIN IN R. O. MODEL
C     ROBETA(N)     BETA FOR R. O. MODEL
C
C     TAU0(N)       INITIAL SHEARING STRESS
C     GAM0(N)       OLD VALUE OF SHEARING STRAIN
C     TAU1(N)       NEW VALUE OF SHEARING STRESS
C     GAM1(N)       NEW VALUE OF SHEARING STRAIN
C     SIGV0(N)      EFFECTIVE VERTICAL STRESS
C     SIGV00(N)     INITIAL SIGV0
C     SIGV99(N)     MINIMUM OF EFFECTIVE VERTICAL STRESS
C     SRA(2,NW)     MAXIMUM AND MINIMUM STRESS RATIO
C
C     A0(N+1)       OLD ACCELERATION
C     A1(N+1)       NEW ACCELERATION
C     V0(N+1)       OLD VELOCITY
C     V1(N+1)       NEW VELOCITY
C     U0(N+1)       OLD DISPLACEMENT
C     U1(N+1)       NEW DISPLACEMENT
C
C     CMAX(8,N+1)   ABS. MAX. VALUES IN A SHORT PERIOD
C     TMAX(8,N+1)   ABS. MAX. VALUES THROUGHOUT THE CALCULATION
C     TAUREV        SHEARING STRESS AT STRAIN REVERSAL
C        (NREV,N)
C     GAMREV        SHEARING STRAIN AT STRAIN REVERSAL
C        (NREV,N)
C
C     K(3,N+1)      RIGIDITY MATRIX                        Z(LTI6)
C     C(3,N+1)      VISCOSITY MATRIX                       Z(LTI5)
C     M(3,N+1)      MASS MATRIX                            Z(LTI3)
C     MI(3,N+1)     DATA FOR (M)**-1                       Z(LTI4)
C     ALM(N+1)      DATA FOR (M)**-1                       Z(LTI1)
C     CMLM(N+1)     DATA FOR (M)**-1                       Z(LTI2)
C
C     WD0(NW4+1)    VERTICAL                               Z(LPW1)
C     WV0(NW4+1)       MOTION OF                           Z(LPW2)
C     WA0(NW4+1)          SOIL                             Z(LPW3)
C     WA1(NW4+1)             PARTICLES                     Z(LPW4)
C
C     PK(3,NW4)                                            Z(LPW5)
C     PC(3,NW4)                                            Z(LPW6)
C     PM(3,NW4)                                            Z(LPW7)
C     TO(3,NW4)                                            Z(LPW10)
C     CPM(NW4)                                             Z(LPW8)
C     APM(NW4)      MATRICES FOR                           Z(LPW9)
C     CTO(NW4)           PRESSURE CALCULATION              Z(LPW11)
C     ATO(NW4)                                             Z(LPW12)
C     IPM(NW4)                                             Z(LPW16)
C     ITO(NW4)                                             Z(LPW17)
C     DD1(NW4)                                             Z(LPW13)
C     DD2(NW4)                                             Z(LPW14)
C     PRD(NW4)                                             Z(LPW15)
C
C     FA(IFLX)                                             ACCELERATION
C     FV(IFLX)                                             VELOCITY
C     FD(IFLX)                                             DISPLACEMENT
C     FT(IFLY)      ARRAYS FOR                             SHEAR STRESS
C     FG(IFLY)         WRITING IN FILE 24                  SHEAR STRAIN
C     FP(IFLY)                                             PORE PRESSURE
C     FS(IFLY)                                             SIGV'
C
C     WORK(1)       WORKING REGION
C
C     ACG           ACCELERATION OF GRAVITY
C     AB            OLD VALUE OF THE ACCELERATION OF THE BASEROCK
C     AB0           OLD VALUE OF THE ACCELERATION OF THE BASEROCK
C     ALPHA         PARAMETERS FOR RAYLEIGH DAMPING
C     BETA              <C> = ALPHA * <M> + BETA * <K>
C     DT            TIME INCREMENT
C     MS            MASS AT THE GROUND SURFACE
C
C     RHOW          UNIT DENSITY OF WATER
C     TIME          TIME
C     TMS           TOTAL MASS ABOVE THE INITIAL GROUND WATER LEVEL
C     UB            NEW DISPLACEMENT OF THE BASEROCK
C     UB0           OLD DISPLACEMENT OF THE BASEROCK
C     UND           CONSTANT FOR ELASTIC-BASEROCK CALCULATION
C     UND2          CONSTANT FOR ELASTIC-BASEROCK CALCULATION
C     VB            NEW VELOCITY OF THE BASEROCK
C     VB0           OLD VELOCITY OF THE BASEROCK
C
C     WTRSF         PRESENT HEIGHT OF THE GROUND WATER SURFACE ABOVE
C                  THE INITIAL LEVEL
C
C     LOC(NW)       KIND OF TRANSITION OF PORE-WATER PRESSURE
C     NL(NW)        NL(J) = N - NW + J
C     IC(N)         NUMBER OF HISTERESIS LOOP OF SHEARING DEFORMATION
C     ICC(N)        PARAMETER FOR THE RENEWAL OF "IC"
C     ICGM(N)       DATA FOR (M)**-1
C     IFL(N)        IF LAYER-J HAS LIQUEFIED, IFL(J) = 1, OTHERWISE =0.
C     ILCL(3)       SIGN OF LIQUEFIED LAYERS
C     IWORK(1)      WORKING REGION
C
C     IDAMP         =1, NO VISCOSE DAMPING
C                   =2, RAYLEIGH DAMPING
C                   =3, VISCOSE DAMPING WITH "VIS"
C     IFLX          SIZE OF ARRAYS: FA, FV
C     IFLY          SIZE OF ARRAYS: FT, FG, FP, FS
C     IGRAAL        PARAMETERS FOR
C     IGRMAX             OUTPUT OF ABS. MAX. VALUES
C     INDELA        IF INDELA=1,  BASEROCK IS ELASTIC.
C     INDIBM        NUMBER OF TIME STEPS TO BE COMPUTED
C     INCLL         NUMBER OF COMPLETELY LIQUEFIED LAYERS
C     IMNCLL        UPPER LIMIT OF "INCLL"
C     IRCAL         IF IRCAL = 1, STRAIN-REVERSAL AND DYNAMIC
C                  EQUILIBRIUM CALCULATIONS ARE PERFORMED.
C     IREVUD        PARAMETERS FOR
C     IRVMAX            THE RENEWAL OF "TAUREV"
C     IRNM          IF IRNM = 1, MASS MATRIX IS RENEWED EVERY TIME STEP.
C     IWL           THE LAYER CONTAINING THE GROUND WATER SURFACE NOW
C     IWRITE        PARAMETERS FOR
C     IWRMAX             OUTPUT OF RESPONSE
C     KUBC          KIND OF UPPER BOUNDARY CONDITIONS
C
C     MNREV         MAXIMUM NUMBER OF STRAIN REVERSALS
C     MP            MP = MIN(N+INDELA, NLD+NUD+1)
C     N             NUMBER OF LAYERS
C     NDRY          NUMBER OF LAYERS ABOVE THE INITIAL GROUND WATER
C                  SURFACE
C     NLD           DATA ABOUT "K", "C" AND "M"
C     NPSTEP        PARAMETER FOR OUTPUT
C     NREV          UPPER LIMIT OF "IC"
C     NUD           DATA ABOUT "K", "C" AND "M"
C     NW            NUMBER OF WATER-SATURATED LAYERS
C
C      "QUAKE" COMPUTES THE PROCESS OF LIQUEFACTION IN DEPOSITS OF SAND,
C     CONTROLLING FOLLOWING SUBROUTINES;
C                  "DATAIN", "READY", "SHEROT", "HDLOOP", "REPONT",
C                  "DILATE", "WEIRES", "NEPRES", "UPDATE" AND "CORACC".
C
      IMPLICIT REAL*8 (A-H, O-Z)
      REAL*8  K0, MS
      CHARACTER *20     LDTTL
      COMMON /ROBA/  NLD, NUD, MP, N34, N45, NW45, N50
     1       /SAI/  INCLL
     2       /ITACHI/  UND, UND2, INDELA
     3       /TANUKI/  DTD12, IRCAL
     4       /KUMA/  L00, L01, L02
     5       /SARU/  ALPHA, BETA, IDAMP, IRNM
     6       /INU/  Q30, Q31, Q32, Q33, L30
     7       /KABA/  P1, P2, P3, BTN, BTNC
     *       /KABA1/ LDTTL
     8       /BUTA/  Q80, IPHI
      CHARACTER*64 TITLE
      COMMON /RISU/ TITLE
      COMMON /RISU1/ ACG,RHOW,MS,DT,PMAX,
     1              IRVMAX,IWRMAX,IGRMAX,KUBC,ISTP,NWWW,NW4,N01,INDIBM
      COMMON / ADRES / LPW1 ,LPW2 ,LPW3 ,LPW4 ,LPW5 ,LPW6 ,LPW7 ,
     1                 LPW8 ,LPW9 ,LPW10,LPW11,LPW12,LPW13,LPW14,
     2                 LPW15,LPW16,LPW17,LPP12,
     3                 LTI1 ,LTI2 ,LTI3 ,LTI4 ,LTI5 ,LTI6 ,
     4                 LWK1 ,LWK2 ,LWK5 ,LWK6 ,LWK7 ,LWK9 ,LWK10,
     5                 LWK11,LWK12,LWK13,LWK14,LMAX
      REAL*4  FA(1), FV(1), FD(1), FT(1), FG(1), FS(1), FP(1)
     1        ,Z(1), WORK(1)
      DIMENSION   BP(*), CS(*), PHT(*), ELW(*), AKAP(*), DIF(*),
     1  TAUEND(*), DU(*), SRA(2,*), LOC(*), IFL(*),
     2  GMAX(*), G(*), RHO(*), RHOS(*), DZ(*), PR(*), SEP(*), PHAI(*),
     3  TAUF(*), GAMR(*), K0(*), CA(*), CB(*), CC(*), GAM0(*), GAM1(*),
     4  TAU0(*), TAU1(*), SIGV0(*), SIGV00(*), SIGV99(*), GAMOR(*),
     5  U0(*), V0(*), A0(*), U1(*), V1(*), A1(*), CMAX(14,*),
     6  TMAX(14,*), GAMREV(NREV,*), TAUREV(NREV,*),
     7  NL(*), IC(*), ICC(*), ICGM(*),
     8  LRFTYP(*), ROSTRN(*), ROBETA(*), COHE(*), IFGEF(*)
      DATA  IGRAAL / -1 /
C
      NPSTEP = 0
      WTRSF = 0.D0
      INCLL = 0
      IPRT = 0
C                                                @ FLAG ON PRINT
      DO  10 I = 1, N
          TMAX(13, I) = 9.D50
   10 CONTINUE
C
C     DATA INPUT AND SETTING INITIAL CONDITIONS
C
      CALL DATAIN (BP, CS, Z(LPW4), Z(LPW15), PHT, ELW, GMAX, G,
C                           WA1      PRD
     1              WORK(LWK1), RHO,RHOS, DZ, PR, SEP, PHAI, K0, CA, CB,
C                   VIS
     2              CC, TAUF, GAMR, GAM0, SIGV0, GAMREV, AKAP, DIF,
     3              SIGV99, SIGV00, GAMOR, DT, TIME, UB0, VB0,
     4              UB, VB, AB, TMS, NL, IC, IREVUD, IWRITE, IWL,
     5              N, NW, NREV, INDIBM, 1 ,
     6              TAUREV, TAU1, LRFTYP, ROSTRN, ROBETA, COHE, IFGEF,
     7              WORK(LWK2))
C
C     SHEARING DEFORMATION; PREPARATION FOR MATRIX CALCULATION
C
      CALL READY (GMAX, WORK(LWK1), RHO, DZ, Z(LTI1), Z(LTI2),
C                        VIS                  ALM      CMLM
     1             Z(LTI5), Z(LTI6), Z(LTI3), Z(LTI4), WORK(LWK2),
C                  C        K        M        MI       WORK(A)
     2             MS, ICGM, N)
      IF (NW.GT.0 .AND. KUBC.GT.0)
     1    CALL XINGU (CS, Z(LPW4), Z(LPW13), Z(LPW14), Z(LPW5),
C                          WA1      DD1       DD2       PK
     2                 Z(LPW6), Z(LPW7), Z(LPW8), Z(LPW9), Z(LPW10),
C                      PC       PM       CPM      APM      TO
     3                 Z(LPW11), Z(LPW12), RHO, DZ, PR, SEP, SIGV00,
C                      CTO       ATO
     4                 TMS, DT, RHOW, ACG, NL, Z(LPW16), Z(LPW17),
C                                              IPM       ITO
     5                 NW, NW4, KUBC)
C
C**************** T I M E   I N T E G R A T I O N *******************
C
  100 CALL SHEROT (DZ, GAM1, U0, V0, A0, U1, V1, A1,
     1              Z(LTI6) ,Z(LTI5), Z(LTI3),
C                   K        C        M
     2              WORK(LWK1), WORK(LWK5), WORK(LWK6), UB, VB, AB, DT,
C                   A           AL          CML
     3              WORK(LWK7), N)
C                   ICG
      DO 110  J = 1, N
          IF (G(J) .GE. TMAX(13, J))  GO TO 110
          TMAX(13, J) = G(J)
          TMAX(14, J) = TIME
  110 CONTINUE
C
C     CHECK OF STRAIN REVERSAL
C
      IF (IRCAL .EQ. 1)
     1    CALL AMNOSY (1,GMAX, G, RHO, DZ, PR, PHAI, TAUF, TAU0, TAU1,
     2                  GAMR, LRFTYP, ROSTRN, ROBETA, COHE, GAM0,
     3                  GAM1, SIGV0, U0, V0, A0, U1, V1, A1,
     4                  Z(LTI1), Z(LTI2), Z(LTI6), Z(LTI5), Z(LTI3),
C                       ALM      CMLM     K        C        M
     5                  Z(LTI4), GAMREV, TAUREV, CMAX, GAMOR,WORK(LWK1),
C                       MI                                    RPR
     6                  WORK(LWK9), WORK(LWK10), WORK(LWK11), TIME, DT,
C                       BM          TAUSTR       WORK
     7                  MS, AB0, VB0, UB0, AB, VB, UB, IC, ICC, ICGM,
     8                WORK(LWK12), N, NW, NREV, JFLAG, CA, CB, CC, K0)
      IF (JFLAG .EQ. 1) GO TO 130
C
C     NEW STRESS
C
      CALL HDLOOP (TAUF, GAMR, TAU0, GAM0, TAU1, GAM1, TAUREV, GAMREV,
     1              GAMOR, LRFTYP,ROSTRN,ROBETA, IC, ICC, N, NREV)
C
C     ERROR IN STRESS
C
      QMAX = 0.D0
      DO 120 J = 1, N
  120 QMAX = MAX(QMAX, ABS(TAU1(J)-TAU0(J)-G(J)*(GAM1(J)-GAM0(J)))
     1          / TAUF(J))
      IF (QMAX .GT. 5.D-2)
     1    CALL AMNOSY (2,GMAX, G, RHO, DZ, PR, PHAI, TAUF, TAU0, TAU1,
     2                  GAMR, LRFTYP, ROSTRN, ROBETA, COHE, GAM0,
     3                  GAM1, SIGV0, U0, V0, A0, U1, V1, A1,
     4                  Z(LTI1), Z(LTI2), Z(LTI6), Z(LTI5), Z(LTI3),
C                       ALM      CMLM     K        C        M
     5                  Z(LTI4), GAMREV, TAUREV, CMAX, GAMOR,WORK(LWK1),
C                       MI                                    RPR
     6                  WORK(LWK9), WORK(LWK10), WORK(LWK11), TIME, DT,
C                       BM          TAUSTR       WORK
     7                  MS, AB0, VB0, UB0, AB, VB, UB, IC, ICC, ICGM,
     8                  WORK(LWK12), N, NW, NREV, JFLAG,CA,CB,CC,K0)
      IF (JFLAG .EQ. 1) GO TO 130
C
C     MODIFICATION OF "IC", "GAMREV", AND "TAUREV" AFTER STRAIN REVERSAL
C
      CALL REPONT (TAU0, GAM0, TAUREV, GAMREV, GAMOR, IC, ICC, N, NREV)
C
  130 IF (NW .EQ. 0)  GO TO 160
C
C     PORE-WATER-PRESSURE INCREMENT IN UNDRAINED CONDITIONS
C
      CALL DILATE (BP, ELW, AKAP, DU, PHT, TAUEND, DIF, SRA, GMAX,
     1              PHAI, TAU0, TAU1, GAM1, SIGV0, SIGV99,
     2              LRFTYP, ROSTRN, ROBETA, IFGEF,Z(LPP12),
     3              GAMOR, TIME, LOC, IC, NL, IFL, NW)
      IF (KUBC .NE. 0)  GO TO 150
      DO 140  J = 1, NW
          I = NL(J)
          SIGV0(I) = SIGV0(I) - DU(J)
  140 CONTINUE
      GO TO 160
C
C     PRESSURE-WAVE CALCULATION
C
  150 CALL RIEPER (DU ,Z(LPW1), Z(LPW2), Z(LPW3), Z(LPW4), Z(LPW13),
C                       WD0      WV0      WA0      WA1      DD1
     1              Z(LPW14), Z(LPW5), Z(LPW6), Z(LPW7), Z(LPW8),
C                   DD2       PK       PC       PM       CPM
     2              Z(LPW9), Z(LPW10), Z(LPW11), Z(LPW12), WORK(LWK1),
C                   APM      TO        CTO       ATO       Q
     3              WORK(LWK14), DT, Z(LPW16), Z(LPW17), NW, NW4)
C                   R                IPM       ITO
C
C     UPDATING
C
  160 CALL UPDATE(CS, DU, Z(LPW15), Z(LPW1), Z(LPW2), Z(LPW3),
C                         PRD       WD0      WV0      WA0
     1            Z(LPW4), PHT, SRA, G, RHO, RHOS, DZ, PR, PHAI,
C                 WA1
     2            TAUF, GAMR, TAU0, GAM0, TAU1, GAM1, LRFTYP, ROSTRN,
     3            ROBETA, IFGEF, COHE, SIGV0, SIGV99, U0, V0, U1, V1,
     4            TAUREV, GAMREV, WORK(LWK1), RHOW, ACG, DT, TIME, UB0,
     5            VB0, AB0, UB, VB, AB, GAMOR, GMAX, WTRSF, LOC, NL,
     6            IC, IFL, IRVMAX, IREVUD, KUBC, IWL, N, NW, NW4, NREV,
     7            Z(LPP12), CA, CB, CC, K0)
C
C     DYNAMIC EQUILIBRIUM
C
      IF (IRCAL .EQ. 1)
     1    CALL CORACC(GMAX, G, RHO, DZ, TAU0, GAM0, A0, V0,
     2                 A1, Z(LTI1), Z(LTI2), Z(LTI6), Z(LTI5),
C                          ALM      CMLM     K        C
     3                 Z(LTI3), Z(LTI4), WORK(LWK1),
C                      M        MI       A
     4                 VB0, AB0, MS, ICGM, N, NW)
      IF (IRCAL .EQ. 1)  GO TO 200
      DO 170  J = 1, N01
          A0(J) = A1(J)
          A1(J) = 0.D0
  170 CONTINUE
C
C     OUTPUT
C
  200 IWRITE = IWRITE + 1
      NPSTEP = NPSTEP + 1
      IGRAAL = IGRAAL + 1
      CALL DAAPRE (TAU0, GAM0, SIGV0, SIGV00, U0, V0, A0, FA, FT, FG,
     1              FS, FP, FV, FD, CMAX, TIME, QMAX, N, N01)
      IF (IWRMAX .LE. 0 .OR. IWRITE .LT. IWRMAX)  GO TO 210
      IWRITE = 0
      IPRT = IPRT + 1
      IF(IPRT .EQ. 1) WRITE(6,600)
      CALL PROP (GMAX, PR, TAUF, GAMR, FA, FT, FG, FS, FP, FV, FD,
     1            TIME, WTRSF, NW, N, N01, IWL)
  210 IF (IGRMAX .LE. 0 .OR. IGRAAL .LT. IGRMAX)  GO TO 220
      CMAX(6,N01) = TIME
      IGRAAL = 0
      IPRT = IPRT + 1
      IF(IPRT .EQ. 1) WRITE(6,600)
      CALL PROKC (CMAX, DZ, N, NW)
      CALL SOLOME (CMAX, TMAX, N, N01)
  220 CONTINUE
C
  230 IF (NPSTEP .GE. INDIBM)  GO TO 300
      TIME = TIME + DT
      GO TO 100
C
  300 IF (IGRAAL .EQ. 0)  GO TO 310
      CMAX(6,N01) = TIME
      IF(IWRMAX .LE. 0) GO TO 310
      IPRT = IPRT + 1
      IF(IPRT .EQ. 1) WRITE(6,600)
      CALL PROKC (CMAX, DZ, N, NW)
  310 CALL SOLOME (CMAX, TMAX, N, N01)
  320 TMAX(6,N01) = TIME
      CALL PROK (TMAX, DZ, GAMREV, PMAX, N, NW, NREV,
     1            INCLL, TITLE, LDTTL, DT, Z(LPP12))
  600 FORMAT(/////)
C
      RETURN
      END
C=======================================================================
      SUBROUTINE RAKSAN (M, C, K, N)
C=======================================================================
C
C     "M"        (3, N)     MASS MATRIX
C     "C"        (3, N)     DAMPING MATRIX
C     "K"        (3, N)     STIFFNESS MATRIX
C     "N"                   NUMBER OF LAYERS
C
C     "RAKSAN" CALCULATES RAYLEIGH DAMPING MATRIX "C".
C
      IMPLICIT REAL*8 (A-H, O-Z)
      REAL*8  M(3,N), C(3,N), K(3,N)
      COMMON /SARU/  ALPHA, BETA, IDAMP, IRNM
C
      DO 100  J = 1, N
          C(1, J) = ALPHA * M(1, J) + BETA * K(1, J)
          C(2, J) = ALPHA * M(2, J) + BETA * K(2, J)
          C(3, J) = ALPHA * M(3, J) + BETA * K(3, J)
  100 CONTINUE
      RETURN
      END
C=======================================================================
      SUBROUTINE RALVES (GAM0, V0, V1, GAMREV, RPR, DT, IC, IPR, N,
     1                    NREV, IREV)
C=======================================================================
C
C     "GAM0"     (N)        OLD STRAIN
C     "V0"       (N+1)      OLD VELOCITY
C     "V1"       (N+1)      NEW VELOCITY
C     "GAMREV"   (NREV,N)   REVERSAL HISTORY
C     "RPR"      (13)       SUBDIVIDED POINTS OF "DT"
C     "IPR"      (11)       WORKING REGION
C     "IREV"                  NUMBER OF SUBDIVIDING
C
C     "RALVES" SUBDIVIDES "DT" INTO "IREV" FRACTIONS FOR
C    STRAIN-REVERSAL CALCULATION.
C
      IMPLICIT REAL*8 (A-H, O-Z)
      COMMON /TANUKI/  DTD12, IRCAL
      DIMENSION  GAM0(*), V0(*), V1(*), GAMREV(NREV,*),RPR(13),IC(*),
     1           IPR(11)
      IREV = 0
      DO 10  I = 1, 11
   10 IPR(I) = 0
      DO 50  J = 1, N
          Q0 = V0(J + 1) - V0(J)
          Q1 = V1(J + 1) - V1(J)
          Q2 = GAM0(J)
          IF (IC(J) .GE. 2)  Q2 = Q2 - GAMREV(IC(J), J)
          IF (Q2 * (Q0 + Q1) .GE. 0.D0)  GO TO 50
          IF (Q0 * Q2)  20, 20, 30
   20     IPR(1) = 1
          IPR(4) = 1
          GO TO 40
   30     I = 1 + AINT(24.001D0 * Q0 / (Q0 - Q1))
          IF (I .GE. 12)  GO TO 50
          IPR(I) = 1
          IF (I .LE. 8)  IPR(I + 3) = 1
   40     IREV = 1
   50 CONTINUE
      IF (IREV .EQ. 0)  RETURN
      RPR(1) = 0.D0
      IREV = 1
      DO 60  I = 1, 11
          IF (IPR(I) .EQ. 0)  GO TO 60
          IREV = IREV + 1
          RPR(IREV) = DTD12 * DFLOAT(I)
   60 CONTINUE
      RPR(IREV + 1) = DT
      RETURN
      END
C=======================================================================
      SUBROUTINE READY (GMAX, VIS, RHO, DZ, ALM, CMLM, C, K, M, MI, A,
     1                   MS, ICGM, N)
C=======================================================================
C
C     "GMAX"     (N)        GMAX
C     "VIS"      (N)        VISCOSITY
C     "RHO"      (N)        MASS DENSITY
C     "DZ"       (N)        THICKNESS OF LAYERS
C     "ALM"      (N+1)      DATA FOR (M)**-1
C     "CMLM"     (N+1)      DATA FOR (M)**-1
C     "C"        (3, N+1)   VISCOSITY MATRIX
C     "K"        (3, N+1)   RIGIDITY  MATRIX
C     "M"        (3, N+1)   MASS      MATRIX
C     "MI"       (3, N+1)   DATA FOR (M)**-1
C     "A"        (N+1)      WORKING REGION
C     "MS"                    POINT MASS AT THE SURFACE
C     "ICGM"     (N)        DATA FOR (M)**-1
C     "IRNM"                  IF IRNM=1, MASS MATRIX IS RENEWED EVERY
C                             TIME STEP.
C     "N"                     NUMBER OF LAYERS
C     "NP1"                   NP1 = N + 1
C
C     "READY" IS THE BEGINNING OF CALCULATION OF VIBRATION.
C
      IMPLICIT REAL*8 (A-H, O-Z)
      COMMON /SARU/  ALPHA, BETA, IDAMP, IRNM
      COMMON /KUMA/  NM1, NP1, NP2
      COMMON /ITACHI/  UND, UND2, INDELA
      COMMON /ROBA/  NLD, NUD, MP, N34, N45, NW45, N50
      REAL*8  K(3,1), M(3,1), MI(3,1), MS
      DIMENSION  GMAX(*), VIS(*), RHO(*), DZ(*), ALM(*), CMLM(*),
     1           C(3,*), A(*), ICGM(*)
      IF (INDELA .EQ. 1)  GO TO 100
C
C     RIGID BASEROCK
C     MASS MATRIX AND INVERSION
C
      CALL MISET (RHO, DZ, A, M, MS, N)
      DO 20  J = 1, N
      DO 10  I = 1, 3
          MI(I,J) = M(I,J)
   10 CONTINUE
   20 CONTINUE
      CALL ELSOL (1,  MI, A, CMLM, ALM, ICGM, N, NLD, MP)
C
C     RIGIDITY AND VISCOSITY MATRIX
C
      CALL MASET (GMAX, DZ, A, K, N)
      GO TO (40, 30, 50),  IDAMP
   30 CALL RAKSAN (M, C, K, N)
   40 RETURN
   50 CALL MASET (VIS, DZ, A, C, N)
      RETURN
C
C     ELASTIC BASEROCK
C     MASS MATRIX AND INVERSION
C
  100 CALL MISET (RHO, DZ, A, M, MS, N)
      DO 120  J = 1, NP1
      DO 110  I = 1, 3
          MI(I,J) = M(I,J)
  110 CONTINUE
  120 CONTINUE
      CALL ELSOL (1,  MI, A, CMLM, ALM, ICGM, NP1, NLD, MP)
C
C     RIGIDITY AND VISCOSITY MATRIX
C
      CALL MASET (GMAX, DZ, A, K, N)
      IF (NM1)  130, 130, 140
  130 A(1) = GMAX(1) / DZ(1)
      K(2, 1 ) = -A(1)
      GO TO 150
  140 K(3, N ) = -A(N)
  150 K(1,NP1) = -A(N)
      K(2,NP1) =  A(N)
      GO TO (210, 160, 170),  IDAMP
  160 CALL RAKSAN (M, C, K, NP1)
      GO TO 210
  170 CALL MASET (VIS, DZ, A, C, N)
      IF (NM1)  180,180, 190
  180 A(1) = VIS(1) / DZ(1)
      C(2, 1 ) = -A(1)
      GO TO 200
  190 C(3, N ) = -A(N)
  200 C(1,NP1) = -A(N)
      C(2,NP1) =  A(N)
  210 C(2,NP1) = C(2,NP1) + UND
      RETURN
      END
C=======================================================================
      SUBROUTINE REPONT (TAU0, GAM0, TAUREV, GAMREV, GAMOR,
     1                    IC, ICC, N, NREV)
C=======================================================================
C
C     "TAU0"     (N)        OLD VALUES OF SHEARING STRESS IN LAYERS
C     "GAM0"     (N)        OLD VALUES OF SHEARING STRAIN IN LAYERS
C     "TAUREV"   (NREV,N)   STRESS AT REVERSAL POINTS IN STRAIN HISTOR
C     "GAMREV"   (NREV,N)   STRAIN AT REVERSAL POINTS IN STRAIN HISTOR
C     "GAMOR"    (N)        STRAIN AT THE ORIGIN
C     "IC"       (N)        THE NAME OF THE HISTERESIS LOOP WITH "GAM0
C     "ICC"      (N)        THE NAME OF THE HISTERESIS LOOP WITH "GAM1
C     "N"                     NUMBER OF LAYERS
C
C     IF "HDLOOP" HAS FOUND "IC" MUST BE CHANGED,
C    THIS SUBROUTINE CHANGES "IC", AND "TAUREV" AND "GAMREV", IF NECESSA
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION  TAU0(*), GAM0(*), TAUREV(NREV,*), GAMREV(NREV,*),
     1           GAMOR(*), IC(*), ICC(*)
      DO 30 J=1,N
      IF (ICC(J) .EQ. 2  .AND.  IC(J) .EQ. 0)  GO TO 10
      IF (ICC(J) .EQ. IC(J) + 1)  GO TO 20
      IC(J) = ICC(J)
      GO TO 30
C
   10 TAUREV(1,J) = - TAU0(J)
      GAMREV(1,J) = -GAM0(J) + 2.D0 * GAMOR(J)
   20 IC(J) = ICC(J)
      TAUREV(IC(J),J) = TAU0(J)
      GAMREV(IC(J),J) = GAM0(J)
C
   30 CONTINUE
      RETURN
      END
C=======================================================================
      SUBROUTINE RIEPER (DU, WD0, WV0, WA0, WA1, DD1, DD2, PK, PC, PM,
     1                    CPM, APM, TO, CTO, ATO, Q, R,
     2                    DT, IPM, ITO, NW, NW4)
C=======================================================================
C
C     "RIEPER" CALCULATES DYNAMICALLY EQUILIBRATED "WA0" AND THEN "WA1".
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION  DU(*),WD0(*),WV0(*),WA0(*),WA1(*),DD1(*),
     1           DD2(*),PK(3,*),PC(3,*),PM(3,*),CPM(*),
     2           APM(*),TO(3,*),CTO(*),ATO(*),Q(*),R(*),
     3           IPM(*),ITO(*)
      COMMON /KABA/  P1, P2, P3, BTN, BTNC
C     PORE WATER PRESSURE INCREMENTS
      DO 20  J = 1, NW
          K = -3 + 4 * J
          X1 = DU(J)
          DO 10  I = K, K + 3
              WA0(I) = X1 * DD1(I)
   10     CONTINUE
          IF (J .GE. 2)  WA0(K) = WA0(K) + DD2(J) * DU(J - 1)
   20 CONTINUE
C
C     DYNAMIC EQUILIBRIUM
      CALL PREMUL (PC, WV0, Q, NW4, 1, 1)
      CALL PREMUL (PK, WD0, R, NW4, 1, 1)
      DO 30  J = 1, NW4
          WA1(J) = WA0(J)
          WA0(J) = WA0(J) - (Q(J) + R(J))
   30 CONTINUE
      CALL ELSOL (2, PM, WA0, CPM, APM, IPM, NW4, 1, 3)
C
C     WA1
      DO 40  J = 1, NW4
          Q(J) = WV0(J) + P1 * WA0(J)
   40 CONTINUE
      CALL PREMUL (PC, Q, R, NW4, 1, 1)
      DO 50  J = 1, NW4
          WA1(J) = WA1(J) - R(J)
          Q(J) = WD0(J) + DT * WV0(J) + P3 * WA0(J)
   50 CONTINUE
      CALL PREMUL (PK, Q, R, NW4, 1, 1)
      DO 60  J = 1, NW4
          WA1(J) = WA1(J) - R(J)
   60 CONTINUE
      CALL ELSOL (2, TO, WA1, CTO, ATO, ITO, NW4, 1, 3)
      RETURN
      END
C=======================================================================
      SUBROUTINE SHEROT (DZ, GAM1, U0, V0, A0, U1, V1, A1, K, C, M, A,
     1                    AL, CML, UB, VB, AB, DT, ICG, N)
C=======================================================================
C
C     "DZ"       (N)        THICKNESS OF LAYERS
C     "GAM1"     (N)        NEW SHEAR STRAIN
C     "U0"       (N+1)      OLD DISPLACEMENT
C     "U1"       (N+1)      NEW DISPLACEMENT
C     "V0"       (N+1)      OLD VELOCITY
C     "V1"       (N+1)      NEW VELOCITY
C     "A0"       (N+1)      OLD ACCELERATION
C     "A1"       (N+1)      NEW ACCELERATION
C     "K"        (3,N+1)    RIGIDITY MATRIX
C     "C"        (3,N+1)    VISCOSITY MATRIX
C     "M"        (3,N+1)    MASS MATRIX
C     "A"        (3,N+1)    WORKING REGION
C     "AL"       (N+1)      WORKING REGION
C     "CML"      (N+1)      WORKING REGION
C     "ICG"      (N)        WORKING REGION
C     "UB", "VB" AND "AB"     NEW BASEROCK MOTION
C     "DT"                    TIME INCREMENT
C     "N"                     NUMBER OF LAYERS
C     "IDAMP"                 =1, NO VISCOSITY
C                             =2, RAYLEIGH DAMPING
C                             =3, VISCOSE SOIL
C
C     "SHEROT" CALCULATES NEW ACCELERATION BY NEWMARK'S BETA METHOD.
C
      IMPLICIT REAL*8 (A-H, O-Z)
      COMMON /KUMA/  NM1, NP1, NP2
      COMMON /SARU/  ALPHA, BETA, IDAMP, IRNM
      COMMON /ITACHI/  UND, UND2, INDELA
      COMMON /KABA/  P1, P2, P3, BTN, BTNC
      COMMON /ROBA/  NLD, NUD, MP, N34, N45, NW45, N50
      REAL*8  K(3,1), M(3,1)
      DIMENSION  DZ(*), GAM1(*), U0(*), V0(*), A0(*), U1(*), V1(*),
     1           A1(*), C(3,*), A(3,*), AL(*), CML(*), ICG(*)
C
      IF (INDELA .EQ. 1)  GO TO 200
C
C**************** RIGID BASEROCK *********************************
C
      A1(N) = A1(N) + UB*(K(1,N)+K(2,N)) + VB*(C(1,N)+C(2,N))
     1        + AB*M(2,NP1)
      DO 10  J = 1, N
   10 AL(J) = U0(J) + DT * V0(J) + P3 * A0(J)
      CALL PREMUL (K, AL, CML, N, NLD, NUD)
      DO 20  J = 1, N
   20 A1(J) = A1(J) - CML(J)
      IF (IDAMP .EQ. 1)  GO TO 50
      DO 30  J = 1, N
   30 AL(J) = V0(J) + P1 * A0(J)
      CALL PREMUL (C, AL, CML, N, NLD, NUD)
      DO 40  J = 1, N
   40 A1(J) = A1(J) - CML(J)
   50 DO 70  J = 1, N
      DO 60  I = 1, 3
          A(I, J) = M(I, J) + P2 * K(I, J)
   60 CONTINUE
   70 CONTINUE
      IF (IDAMP .EQ. 1)  GO TO 100
      DO 90  J = 1, N
      DO 80  I = 1, 3
          A(I, J) = A(I, J) + P1 * C(I, J)
   80 CONTINUE
   90 CONTINUE
  100 CALL ELSOL (1, A, A1, CML, AL, ICG, N, NLD, MP)
      DO 110  J = 1, N
          U1(J) = U0(J) + DT*V0(J) + P3*A0(J) + P2*A1(J)
          V1(J) = V0(J) + P1*(A0(J)+A1(J))
  110 CONTINUE
      U1(NP1) = UB
      V1(NP1) = VB
      A1(NP1) = AB
      GO TO 400
C
C**************** ELASTIC BASEROCK *******************************
C
  200 A1(NP1) = A1(NP1) + UND2 * VB
      DO 210  J = 1, NP1
  210 AL(J) = U0(J) + DT * V0(J) + P3 * A0(J)
      CALL PREMUL (K, AL, CML, NP1, NLD, NUD)
      DO 220  J = 1, NP1
  220 A1(J) = A1(J) - CML(J)
      IF (IDAMP .EQ. 1)  GO TO 250
      DO 230  J = 1, NP1
  230 AL(J) = V0(J) + P1 * A0(J)
      CALL PREMUL (C, AL, CML, NP1, NLD, NUD)
      DO 240  J = 1, NP1
  240 A1(J) = A1(J) - CML(J)
      GO TO 260
  250 A1(NP1) = A1(NP1) - C(2,NP1)*(V0(NP1)+P1*A0(NP1))
  260 DO 280  J = 1, NP1
      DO 270  I = 1, 3
          A(I, J) = M(I, J) + P2 * K(I, J)
  270 CONTINUE
  280 CONTINUE
      IF (IDAMP .EQ. 1)  GO TO 310
      DO 300  J = 1, NP1
      DO 290  I = 1, 3
          A(I, J) = A(I, J) + P1 * C(I, J)
  290 CONTINUE
  300 CONTINUE
      GO TO 320
  310 A(2, NP1) = A(2, NP1) + P1 * C(2, NP1)
  320 CALL ELSOL (1, A, A1, CML, AL, ICG, NP1, NLD, MP)
      DO 330  J = 1, NP1
      U1(J) = U0(J) + DT*V0(J) + P3*A0(J) + P2*A1(J)
          V1(J) = V0(J) + P1*(A0(J)+A1(J))
  330 CONTINUE
C
C**************** NEW SHEARING STRAIN ****************************
C
  400 DO 410  J = 1, N
  410 GAM1(J) = (U1(J+1) - U1(J)) / DZ(J)
C
      RETURN
      END
C=======================================================================
      SUBROUTINE SOLOME (CMAX, TMAX, N, N01)
C=======================================================================
C
C     "CMAX"   (14,N01)      ABS. MAX. IN LATEST PERIOD
C     "TMAX"   (14,N01)      ABS. MAX. SINCE THE BEGINNING
C
C     "N"                     NUMBER OF LAYERS
C     "N01"                   N01 = N+1
C
C     "SOLOME" UPDATES "TMAX" AND INITIALIZES "CMAX".
C
      REAL*8  CMAX(14,N01), TMAX(14,N01)
      DO 30  J = 1, N
      DO 20  I = 1, 11, 2
          IF (ABS(TMAX(I,J)) .GT. ABS(CMAX(I,J)))  GO TO 10
          TMAX(I  ,J) = CMAX(I  ,J)
          TMAX(I+1,J) = CMAX(I+1,J)
   10     CMAX(I  ,J) = 0.D0
   20 CONTINUE
   30 CONTINUE
      DO 50  I = 1, 3, 2
          IF (ABS(TMAX(I,N01)) .GT. ABS(CMAX(I,N01)))  GO TO 40
          TMAX(I  ,N01) = CMAX(I  ,N01)
          TMAX(I+1,N01) = CMAX(I+1,N01)
   40     CMAX(I  ,N01) = 0.D0
   50 CONTINUE
      DO 70  I = 9, 11, 2
          IF (ABS(TMAX(I,N01)) .GT. ABS(CMAX(I,N01)))  GO TO 60
          TMAX(I  ,N01) = CMAX(I  ,N01)
          TMAX(I+1,N01) = CMAX(I+1,N01)
   60     CMAX(I  ,N01) = 0.D0
   70 CONTINUE
      CMAX(5,N01) = CMAX(6,N01)
      RETURN
      END
C=======================================================================
      SUBROUTINE SSCURV(ITP,TAU,GAM,TAUF,GAMR,TAUB,GAMB,CA,CB,CC)
C=======================================================================
C
C     "SSCURV" CIMPUTES STRESS AND "SSTMOD" COMPUTES TANGENT MODELUS
C
C     ITP    : TYPE OF STRESS-STRAIN CURVE
C            1 = HARDIN DRNEVICH
C            2 = RAMBERG OSGOOD (MODIFIED BY HARA)
C     TAU    : "SSCURVE"
C               FIRST TRIAL VALUE FOR STRESS WHEN INPUT
C               COMPOUTED STRESS WHEN OUTPUT
C     GAM    : CURRENT STRAIN
C     TAUF   : ULTIMATE STRESS
C     GAMR   : REFERENCE STRAIN
C     TAUB   : STRESS AT UNLOADINT
C     GAMB   : STRAIN AT UNLOADING
C     CA     : COEFFICIENT   1 FOR SKELTON CURVE
C                            0.5 FOR HYSTERESIS CURVE
C     CB     : BETA IN R. O. MODEL
C##     CC     : UNLIMATE STRAIN IN R. O. MODEL
C     CC     : ALPHA IN R. O. MODEL
C
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*4  CA
C
      DATA EPS /1.D-6/ , IERR /0/
C
      G = GAM - GAMB
      GO TO (100,200),ITP
C
C-----------------------------------------------------------------------
      ENTRY SSTMOD(ITP,TAU,GAM,TAUF,GAMR,TAUB,GAMB,CA,CB,CC)
C-----------------------------------------------------------------------
C
      G = GAM - GAMB
      GO TO (110,230),ITP
C
C *************** HARDIN DRNEVICH MODEL ****************************
C
C     (TAU - TAUB) * CA = TAUF * (GAM - GAMB) * CA /
C                        (GAMR + ABS(GAM - GAMB) * CA)
C
C ******** COMPUTE STRESS
C
  100 TAU = TAUB + TAUF * G / (GAMR + CA * DABS(G))
      RETURN
C
C ******** COMPUTE TAUGENT MODULUS
C
  110 TAU = TAUF * GAMR / (GAMR + CA * DABS(G)) ** 2
      RETURN
C
C *************** RAMBERG OSGOOD MODEL *****************************
C
C ******** COMPUTE STRESS
C     STRESS FOR GIVEN STRAIN IS CAUCULATED BY THE NEWTON METHOD
C     ERROR FOR CONVERGENCY = EPS * TAUF = TEPS
C
C     D(G)/D(T) / GAMR = (1. + (GAMF/GAMR - 1) * CB *
C                        (CA * T / TAUF) ** (CB - 1)
C     WHERE G = GAM - GAMB, AND T = TAU - TAUB
C
  200 TEPS = EPS * TAUF
      S1 = TAUF / GAMR
C##      S2 = (CC / GAMR - 1.0D0) * (CA / TAUF) ** (CB - 1.D0)
      S2 = CC * (CA / TAUF) ** (CB - 1.D0)
      S3 = S2 * CB
      GG = DABS(G)
      T1 = DABS(TAU - TAUB)
C
      DO 210 I = 1,50
      SS = T1 ** (CB - 1.D0)
      DTDG = S1 / (1.D0 + S3 * SS)
      G1 = T1 / S1 * (1.D0 + S2 * SS)
      TT = (GG - G1) * DTDG
      IF(DABS(TT) .LE. TEPS) GO TO 220
      T1 = T1 + TT
  210 CONTINUE
      WRITE(6,600)
      IERR = IERR + 1
      IF(IERR .GE. 50) STOP
C
  220 IF(G .GE. 0.) TAU = TAUB + T1 + TT
      IF(G .LT. 0.) TAU = TAUB - T1 - TT
      RETURN
C
C ******** COMPUTE TANGEMT MODULUS
C
C##  230 TAU = TAUF / GAMR / (1.D0 + (CC / GAMR - 1.D0)
C##     *       * CB * DABS(CA * (TAU - TAUB)/ TAUF) ** (CB - 1.D0))
  230 TAU = TAUF / GAMR / (1.D0 + CC
     *       * CB * DABS(CA * (TAU - TAUB)/ TAUF) ** (CB - 1.D0))
      RETURN
C
  600 FORMAT(' ** WARNING **   STRESS DOES NOT FIND FOR ',
     *       'RAMBERG OSGOOD MODEL')
C
      END
C=======================================================================
      SUBROUTINE UPDATE (CS, DU, PRD, WD0, WV0, WA0, WA1, PHT, SRA,
     1                    G, RHO, RHOS, DZ, PR, PHAI, TAUF, GAMR,
     2                    TAU0, GAM0, TAU1, GAM1, LRFTYP, ROSTRN,
     3                    ROBETA, IFGEF, COHE, SIGV0, SIGV99, U0, V0,
     4                    U1, V1, TAUREV, GAMREV, WORK, RHOW, ACG, DT,
     5                    TIME, UB0, VB0, AB0, UB, VB, AB, GAMOR, GMAX,
     6                    WTRSF, LOC, NL, IC, IFL, IRVMAX, IREVUD,
     7                    KUBC, IWL, N, NW, NW4, NREV, LQTIME,
     8                    CA, CB, CC, K0)
C=======================================================================
C      "UPDATE" RENEWS "SRA", "G", "RHO", "PR", "TAUF", "GAMR", "TAU0",
C     "GAM0", "SIGV0", "U0", "V0", "ESO", "TAUREV", "UB0", "VB0", "AB0",
C     "UB", "VB", "AB", "WTRSF", "LOC", "IREVUD", "KUBC", "IWL"
C     and "INCLL".
C
      IMPLICIT REAL*8 (A-H, O-Z)
      COMMON /INU/  C1, C2, C3, E1M, NDRY
     1       /SAI/  INCLL
     2       /KUMA/  NM1, NP1, NP2
     3       /BUTA/  PM, IPHI
      REAL*8 K0
      DIMENSION  PHT(*), SRA(2,*), G(*), RHO(*), RHOS(*), DZ(*),
     1           PR(*), TAUF(*), GAMR(*), TAU0(*), GAM0(*),
     2           TAU1(*), GAM1(*), SIGV0(*), U0(*), V0(*), U1(*), V1(*),
     3           TAUREV(NREV,*), GAMREV(NREV,*), CS(*), DU(*), PRD(*),
     4           WD0(*), WV0(*), WA0(*), WA1(*), WORK(*),
     5           GAMOR(*), GMAX(*), SIGV99(*), PHAI(*),
     6           LOC(*), NL(*), IC(*), IFL(*),
     7           LRFTYP(*), ROSTRN(*), ROBETA(*), IFGEF(*), COHE(*),
     8           CA(*), CB(*), CC(*), K0(*)
      REAL *8    LQTIME(2,*)
C
C     EXISTENCE OF UNDERGROUND WATER
C
      IF (NW .EQ. 0)  GO TO 100
      IF (KUBC .EQ. 0)  GO TO 10
C
C     RENEWAL OF UPPER BOUNDARY CONDITIONS, "RHO", AND "PR".
C
      CALL COPPER (CS, DU, PRD, WD0, WV0, WA0, WA1, RHO, RHOS, DZ, PR,
     1              SIGV0, WORK, DT, WTRSF, RHOW, ACG, NL, NW, NW4, IWL,
     2              KUBC)
C
C     SIGV0 > 0?
C
   10 DO 20  J = 1, N
          IF (SIGV0(J) .GT. SIGV99(J))  GO TO 20
          SIGV0(J) = SIGV99(J)
          IF (IFL(J) .EQ. 1)  GO TO 20
          WRITE(6,600)  J, TIME
      LQTIME(2,J) = TIME
          IFL(J) = 1
          IF (J .LE. NDRY)  GO TO 20
          LOC(J - NDRY) = 3
          INCLL = INCLL + 1
   20 CONTINUE
C
C     MAXIMUM AND MINIMUM STRESS-RATIOS
C
      DO 60  J = 1, NW
          IF (ABS(LOC(J)) .GE. 2)  GO TO 60
          NLTHIS = NL(J)
      IF(IFGEF(NLTHIS) .NE. 0) GO TO 60
          IF (TAU1(NLTHIS) .GT. SIGV0(NLTHIS)*SRA(1,J))  GO TO 30
          IF (TAU1(NLTHIS) .LT. SIGV0(NLTHIS)*SRA(2,J))  GO TO 40
          LOC(J) = 0
          GO TO 60
   30     SRA(1, J) = TAU1(NLTHIS) / SIGV0(NLTHIS)
          IF (SRA(1, J) .GE. PHT(J))  GO TO 50
          LOC(J) = 1
          GO TO 60
   40     SRA(2, J) = TAU1(NLTHIS) / SIGV0(NLTHIS)
          IF (SRA(2, J) .LE. -PHT(J))  GO TO 50
          LOC(J) = -1
          GO TO 60
   50     LOC(J) = 2
      LQTIME(1,J) = TIME
C%%
          IF (IPHI .EQ. 0) THEN
            PHAI(NLTHIS) = PM * PHT(J)
          SD = SIGV0(NLTHIS) * PHAI(NLTHIS)
          SU = GMAX(NLTHIS) * (SD - ABS(TAU1(NLTHIS)))
          GAMOR(NLTHIS) = GAM1(NLTHIS) - SD * TAU1(NLTHIS) / SU
          IC(NLTHIS) = 0
          ENDIF
C%%
          PHT(J) = 1.D0 / PHAI(NLTHIS) **2
   60 CONTINUE
C
C     UPDATING "TAU0", "GAM0", "U0", "V0", "UB0", AND "VB0"
C
  100 DO 110  J = 1, N
          TAU0(J) = TAU1(J)
          GAM0(J) = GAM1(J)
          U0(J) = U1(J)
          V0(J) = V1(J)
  110 CONTINUE
      U0(NP1) = U1(NP1)
      V0(NP1) = V1(NP1)
      UB0 = UB
      VB0 = VB
      AB0 = AB
C
C     UPDATING "G", "TAUF", "GAMR", "UB", "VB", AND "AB" IN "RENEW"
C
      CALL RENEW (GMAX, G, PR, PHAI, TAUF, GAMR, GAM0, SIGV0, GAMREV,
     1             GAMOR, UB, VB, AB, IC, N, NW ,
     2             TAUREV, TAU1, LRFTYP, ROSTRN, ROBETA, COHE,
     3             NREV, CA, CB, CC, K0)
C
C     NEW "TAUREV" IN ACCORDANCE WITH NEW "TAUF" AND NEW "GAMR"
C
  200 IF (NW .EQ. 0)  RETURN
      IREVUD = IREVUD + 1
      IF (IREVUD .LT. IRVMAX)  RETURN
      IREVUD = 0
      DO 220  J = NDRY + 1, N
          IF (IC(J) .EQ. 0)  GO TO 220
           A1 = 0.
           CALL SSCURV(LRFTYP(J),A1,GAMREV(2,J),TAUF(J),GAMR(J),
     *                 0.D0,GAMOR(J),1.0,ROBETA(J),ROSTRN(J))
          TAUREV(2, J) = A1
          TAUREV(1, J) = -A1
          IF (IC(J) .LE. 2)  GO TO 220
          K = IC(J)
          DO 210  I = 3, K
           CALL SSCURV(LRFTYP(J),A1,GAMREV(I,J),TAUF(J),GAMR(J),
     *                TAUREV(I-1,J),GAMREV(I-1,J),0.5,
     *                ROBETA(J),ROSTRN(J))
           TAUREV(I,J) = A1
  210     CONTINUE
  220 CONTINUE
C
  600 FORMAT (/,' ### Effective vertical stress = 0.0 in layer',
     1         I2, ' at ', F6.3, ' seconds')
C
      RETURN
      END
C=======================================================================
      SUBROUTINE VIDEFF (A, DT, INDIBM, PMAX, LMAX)
C=======================================================================
C     "A"        (INDIBM)   BASEROCK MOTION DATA
C     "DT"                    TIME INCREMENT
C     "INDIBM"                NUMBER OF DATA "A"
C     "LMAX"                  MAX. AREA (DYNAMIC ALLOCATION)
C     "KIND"                  KIND=1, READ ACCELERATION DATA IN.
C                                 =2, READ VELOCITY DATA IN.
C                                 =3, READ DISPLACEMENT DATA IN.
C                                 =4, HARMONIC EXCITATION
C     "PMUL"                  SCALING FACTOR OF BASEROCK MOTION;
C                             SKIP IF PMAX .NE. 0.
C     "PMAX"                  BASEROCK MOTION ARE MODIFIED SO THAT
C                             MAX. = PMAX.  MAY BE SKIPPED IF PMUL .NE.
C                             0.
C     "BTN"                   NEWMARK'S BETA
C
C     "VIDEFF" GETS BASEROCK MOTION DATA.
C
      IMPLICIT REAL*8 (A-H, O-Z)
      CHARACTER *20     LDTTL
      CHARACTER *16 FORMT, P(3)
      COMMON /KABA/  P1, P2, P3, BTN, BTNC
     1       /KABA1/ LDTTL
      DIMENSION  S(255), A(*)
      DATA  P, A0, V0, U0, W / 12HAcceleration, 8HVelocity,
     1      12HDisplacement, 4*0.D0 /
      F1(U0, V0, A0, A1) = U0 + DT * V0 + P3 * A0 + P2 * A1
      F2(V0, A0, A1) = V0 + P1 * (A0 + A1)
      F3(V0, A0, V1) = -A0 + P4 * (V1 - V0)
      F4(U0, V0, A0, U1) = P5 * (U1 - U0 - DT * V0 - P3 * A0)
C
      REWIND 23
      BTN = 0.25D0
      BTNC = 0.5D0 - BTN
C
      READ(5,500) KIND,IFGPRT,IWDIV,NDATA,DT,TEND,PMUL,PMAX,FORMT,
     1            NSKIP,LDTTL
C              KIND : INPUT DATA TYPE
C                     1 = ACC, 2 = VEL, 3 = DISP, 4 = SINUSOIDAL
C             IFGPRT : FLAG ON PRINT WAVEFORM
C                     1 = PRINT
C             IWDIV : NUMBER OF DIVISION
C             NDATA : NUMBER OF WAVE DATA
C             DT    : TIME INTERVAL OF INPUT DATA
C             TEND  : TIME WHEN CALCULATION STOP
C             PMUL  : NUMTIFICATION FACTOR
C             PMAX  : MAXIMUM VALUE OF INPUT WAVE
C                NOTE! EITHER OF PMUL OR PMAX SHOULD BE ZERO
C             FORMT : FORMAT
C             NSKIP : NUMBER OF LINES TO BE SKIPPED BEFORE READING
C                     INPUT EXCITATION WHEN LDTTL IS NOT BLANK.
C             LDTTL : FILE FOR INPUT EXCITATION. WHEN BLAK, UNIT=5
C             IN CASE THAT KIND = 4, TEND AND FORMT ARE MEANINGLESS,
C             PMUL MEADS AMPLITUDE OF ACCELERATION , AND PMAX MEADS
C             NATURAL FREQUENCY.
C
      WRITE(24) LDTTL
      IF(KIND .LE. 0) KIND = 1
      IF(TEND .LE. 0.D0) TEND = NDATA * DT
      GO TO (10, 10, 10, 300),  KIND
   10 IF (LDTTL .EQ. '                    ') THEN
        WRITE(6,600)  'input through CARD  ',DT,P(KIND),NDATA
      ELSE
        WRITE(6,600)  LDTTL,DT,P(KIND),NDATA
      ENDIF
   20 INDIBM = TEND / DT + 0.000001
      LLMAX = (MAX0(NDATA,INDIBM) + 257) * 2
      IF(LLMAX .LE. LMAX) GO TO 30
   30 IF (LDTTL .EQ. '                    ') THEN
        READ(5,FORMT)  (A(I),I=1,NDATA)
        LDTTL = 'input through CARD  '
      ELSE
        OPEN (UNIT=10, FILE=LDTTL)
        IF (NSKIP .GT. 0) THEN
          DO 31 I = 1, NSKIP
            READ(10,500)
   31     CONTINUE
        ENDIF
        READ(10,FORMT)  (A(I),I=1,NDATA)
        CLOSE (UNIT=10)
      ENDIF
C
C     MAXIMUM VALUE
C
      W = 0.D0
      DO 100  I = 1, NDATA
          D = ABS(A(I))
          IF (D .LE. W)  GO TO 100
          W = D
          IM = I
  100 CONTINUE
      O = DT * DFLOAT(IM)
      WRITE(6,610)  W, O
      IF (PMUL .EQ. 0.D0)  GO TO 110
      PMAX = W * PMUL
      GO TO 120
  110 PMUL = ABS(PMAX) / W
  120 IF (PMUL .EQ. 1.D0)  GO TO 140
      WRITE(6,620)  PMAX, PMUL
C     SCALING
      DO 130  I = 1, NDATA
  130 A(I) = A(I) * PMUL
  140 WRITE(6,630) TEND
      IF(INDIBM .LE. NDATA) GO TO 160
      DO 150 I = NDATA+1,INDIBM
      A(I) = 0.D0
  150 CONTINUE
      WRITE(6,640)
      GO TO 180
  160 IF(INDIBM .EQ. NDATA) GO TO 180
      WRITE(6,650) INDIBM
      IF(O .LE. TEND) GO TO 180
C     LOOK FOR MAX. VALUE WHEN INDIBM .LT. NDATA
      W = 0.
      DO 170 I = 1,INDIBM
      D = DABS(A(I))
      IF(D .LE. W) GO TO 170
      W = D
      IM = I
  170 CONTINUE
      O = IM * DT
      WRITE(6,660) W,O
  180 IF(IFGPRT .NE. 1) GO TO 191
      WRITE(6,670) INDIBM
      IM = (INDIBM - 1) / 10 + 1
      DO 190 I = 1,IM
      IS = 10 * I - 9
      IE = 10 * I
      IF(IE .GT. INDIBM) IE = INDIBM
      J = I - 1
      WRITE(6,680) J,(A(K),K=IS,IE)
  190 CONTINUE
C
C      Divide time interval
C
  191 CONTINUE
      IF(IWDIV .LE. 1) GO TO 200
      LLMAX = (INDIBM * IWDIV + 257) * 2
      IF(LLMAX .LE. LMAX) GO TO 192
  192 DO 199 I = INDIBM , 1 , -1
      A1 = A(I)
      IF(I .GT. 1) A2 = A(I - 1)
      IF(I .EQ. 1) A2 = 0.D0
      A3 = (A1 - A2) / IWDIV
      II = (I - 1) * IWDIV
      DO 198 J = 1,IWDIV
      A(II + J) = A2 + A3 * J
  198 CONTINUE
  199 CONTINUE
      INDIBM = INDIBM * IWDIV
      DT = DT / IWDIV
      WRITE(6,710) IWDIV , DT
C
C     Integration and differentiation
C
  200 IM = 3
      P1 = 0.5D0 * DT
      P3 = DT * DT
      P2 = BTN * P3
      P3 = P3 * BTNC
      P4 = 2.D0 / DT
      P5 = 1.D0 / P2
      DO 250  I = 1, INDIBM
          GO TO (210, 220, 230),  KIND
  210     A1 = A(I)
          V1 = F2(V0, A0, A1)
          U1 = F1(U0, V0, A0, A1)
          GO TO 240
  220     V1 = A(I)
          A1 = F3(V0, A0, V1)
          U1 = F1(U0, V0, A0, A1)
          GO TO 240
  230     U1 = A(I)
          A1 = F4(U0, V0, A0, U1)
          V1 = F2(V0, A0, A1)
  240     A0 = A1
          V0 = V1
          U0 = U1
          S(IM + 1) = A1
          S(IM + 2) = V1
          S(IM + 3) = U1
          IM = IM + 3
          IF (IM .LT. 255)  GO TO 250
          WRITE(23)  S
          IM = 0
  250 CONTINUE
      IF (IM .GT. 0)  WRITE(23)  S
      GO TO 320
C
C     Harmonic excitation
C
  300 O = 1.D0 / PMAX
      WRITE(6,690)  LDTTL,DT,PMUL,PMAX,O
      PMAX = 8.D0 * DATAN(1.0D0) / PMAX
      AAMP = PMUL
      O = PMAX
      IM = 3
      PMAX = AAMP / O
      PMUL = PMAX / O
      O = O * DT
      DO 310  I = 1, INDIBM
          A0 = A0 + O
          V0 = COS(A0)
          S(IM+1) = AAMP * V0
          S(IM+2) = PMAX * SIN(A0)
          S(IM+3) = PMUL * (1.D0 - V0)
          IM = IM + 3
          IF (IM .LT. 255)  GO TO 310
          WRITE(23)  S
          IM = 0
  310 CONTINUE
      IF (IM .GT. 0)  WRITE(23)  S
C
  320 NDATA = MAX0(INDIBM , NDATA)
      DO 350  I = 1, NDATA
  350 A(I) = 0.D0
      INDIBM = INDIBM + 1
      WRITE(6,700) LLMAX
C
      RETURN
C
C **************** F O R M A T *****************
C
  500 FORMAT(2I1,I3,I5,2F5.0,2F10.0,A16,I4,A20)
C
  600 FORMAT(//,' ----- Baserock excitation -----',//,
     1   '     Name ---------------------- ',A20,/,
     2   '     Time interval ------------- ',F8.4,' Seconds',/,
     3   '     Wave type ----------------- ',A16,/,
     4   '     Number of wave data ------- ',I8)
  610 FORMAT('     Maximum value ------------- ',
     1   5X,'Input wave      ',E12.5,5X,'at time =',F8.3,' Seconds')
  620 FORMAT(38X,'for calculation ',E12.5,5X,'(Cal./Input =',F11.5,')')
  630 FORMAT('     Duration ------------------ ',F8.4,' Second')
  640 FORMAT(33X,'(Zero acceleration are assumed for residual part)')
  650 FORMAT(33X,1H(,I5,' Data from the beginning are used)')
  660 FORMAT(33X,'(Max. value in calculation =',E12.5,5X,
     1      'at time =',F8.4,' Second ')
  670 FORMAT(///,'    Waveform for calculation ',10X,
     1       'Number of data =',I8,/)
  680 FORMAT(1X,I6,10F12.5)
  690 FORMAT(//,' ----- Baserock excitation -----',/,
     1   '     Name ---------------------- ',A20,/,
     2   '     Time interval ------------- ',F8.4,' Second',/,
     3   '     Input wave ---------------- Harmonic wave',/,
     4   '     Amplitude of accel. ------- ',E12.5,/,
     5   '     Natural period ------------ ',F8.4,' Second',/
     6   '     Natural frequency --------- ',F8.3,' Herts')
  700 FORMAT(/,' Max. area requiared for earthquake wave =',I8)
  710 FORMAT(/,' Time intercal for calculation is divided by ',
     1       I3,' ,becomes',F8.5,' Second')
C
      END
C=======================================================================
      SUBROUTINE XINGU (CS, WA1, DD1, DD2, PK, PC, PM, CPM, APM, TO,
     1                   CTO, ATO, RHO, DZ, PR, SEP, SIGV00, TMS, DT,
     2                   RHOW, ACG, NL, IPM, ITO, NW, NW4, KUBC)
C=======================================================================
C      "XINGU" PRODUCES MATRICES USED IN PRESSURE-WAVE CALCULATION.
C
      IMPLICIT REAL*8 (A-H, O-Z)
      DIMENSION  CS(*),WA1(*),DD1(*),DD2(*),PK(3,*),PC(3,*),PM(3,*),
     1           CPM(*),APM(*),TO(3,1),CTO(*),ATO(*),RHO(*),DZ(*),PR(*),
     2           SEP(*),SIGV00(*),NL(*),IPM(*),ITO(*)
      COMMON /KABA/  P1, P2, P3, BTN, BTNC
      X1 = 1.D0 / (RHOW * ACG)
C     WEIGHTING FACTOR "SA" FOR UPPER BOUNDARY CONDITION
      NLJ = NL(1)
      SA = X1 * SEP(NLJ) * CS(1)**2
C
      DO 10  J = 1, NW
          K = 4 * J
          NLJ = NL(J)
          X2 = X1 * SEP(NLJ)
          DD2(K-3) = X2
          DD2(K-2) = X2
          DD2(K-1) = X2
          DD2(K  ) = X2
          X2 = X2 * CS(J)
          DD1(K-3) = X2
          DD1(K-2) = X2
          DD1(K-1) = X2
          DD1(K  ) = X2
   10 CONTINUE
      IF (KUBC .EQ. 1)  GO TO 20
      NLJ = NL(1)
      X2 = 4.D0 * SA / (CS(1) * DZ(NLJ))
      PK(1, 1) =  X2
      PK(2, 1) = -X2
      GO TO 30
   20 PK(1, 1) = 0.D0
      PK(2, 1) = 0.D0
   30 PK(3, 1) = 0.D0
      DO 60  J = 1, NW
          NLJ = NL(J)
          K = -3 + 4 * J
          IF (J .EQ. 1)  GO TO 40
          X3 = 2.D0 * (DD1(K-1) + DD1(K))
          X2 = X3 / DZ(NLJ-1)
          X3 = X3 / DZ(NLJ  )
          X5 = 1.66666666666667D-1 * (CS(J) - CS(J - 1))
          X4 = 4.D0 * X5 * (DD2(K)+2.D0*DD2(K-1)) / DZ(NLJ-1)
          X5 = 4.D0 * X5 * (DD2(K-1)+2.D0*DD2(K)) / DZ(NLJ  )
          PK(1, K) = -(X2 + X4)
          PK(2, K) = X2 + X3 + X4 - X5
          PK(3, K) = X5 - X3
   40     X2 = -4.D0 * DD1(K) / DZ(NLJ)
          X3 = -(X2 + X2)
          K = K + 1
          DO 50  I = K, K+2
              PK(1, I) = X2
              PK(2, I) = X3
              PK(3, I) = X2
   50     CONTINUE
   60 CONTINUE
      PK(3, NW4) = 0.D0
C
      DO 70  J = 1, NW
          NLJ = NL(J)
          X2 = (1.D0/12.D0) * DZ(NLJ) * CS(J)**2
          K = 4 * J
          DD1(K-3) = X2
          DD1(K-2) = X2
          DD1(K-1) = X2
          DD1(K  ) = X2
   70 CONTINUE
      IF (KUBC .EQ. 1)  GO TO 80
      PC(1, 1) = DD1(1)
      PC(2, 1) = 0.5D0 * DD1(1)
      GO TO 90
   80 PC(1, 1) = 0.D0
      PC(2, 1) = 0.D0
   90 PC(3, 1) = 0.D0
      DO 120  J = 1, NW
          NLJ = NL(J)
          K = -3 + 4 * J
          IF (J .EQ. 1)  GO TO 100
          PC(1, K) = 0.5D0 * DD1(K-1)
          PC(2, K) = DD1(K-1) + DD1(K)
          PC(3, K) = 0.5D0 * DD1(K)
  100     K = K + 1
          X2 = 0.5D0 * DD1(K)
          DO 110  I = K, K+2
              PC(1, I) = X2
              PC(2, I) = 4.D0 * X2
              PC(3, I) = X2
  110     CONTINUE
  120 CONTINUE
      PC(3, NW4) = 0.D0
C
      DO 130  J = 1, NW
          NLJ = NL(J)
          X2 = (1.D0/12.D0) * DZ(NLJ) * SEP(NLJ) * X1 * CS(J)**2
     1       * (RHO(NLJ)+RHOW*(-2.D0+1.D0/PR(NLJ)))
          K = 4 * J
          DD1(K - 3) = X2
          DD1(K - 2) = X2
          DD1(K - 1) = X2
          DD1(K    ) = X2
  130 CONTINUE
      IF (KUBC .EQ. 1)  GO TO 140
      PM(1, 1) = DD1(1) + SA * TMS
      PM(2, 1) = 0.5D0 * DD1(1)
      GO TO 150
  140 PM(1, 1) = 1.D0
      PM(2, 1) = 0.D0
  150 PM(3, 1) = 0.D0
      DO 180  J = 1, NW
          NLJ = NL(J)
          K = -3 + 4 * J
          IF (J .EQ. 1)  GO TO 160
          PM(1, K) = 0.5D0 * DD1(K-1)
          PM(2, K) = DD1(K-1) + DD1(K)
          PM(3, K) = 0.5D0 * DD1(K)
  160     K = K + 1
          X2 = 0.5D0 * DD1(K)
          DO 170  I = K, K+2
              PM(1, I) = X2
              PM(2, I) = 4.D0 * X2
              PM(3, I) = X2
  170     CONTINUE
  180 CONTINUE
      PM(3, NW4) = 0.D0
C
      DO 190  J = 1, NW
          NLJ = NL(J)
          K = 4 * J
          X2 = X1 * SEP(NLJ) * CS(J)**2
          CPM(K-3) = X2
          CPM(K-2) = X2
          CPM(K-1) = X2
          CPM(K  ) = X2
          X2 = ACG * (RHO(NLJ) - RHOW)
          APM(K-3) = X2
          APM(K-2) = X2
          APM(K-1) = X2
          APM(K  ) = X2
  190 CONTINUE
      NLJ = NL(1)
      X2 = 1.D0 / (DT * SIGV00(NLJ))
      DD1(1) = 0.D0
      IF (KUBC .EQ. 2)  DD1(1) = X2 * (SA * WA1(1) + 0.125D0
     1                             * CPM(1) * APM(1) * DZ(NLJ))
      DO 220  J = 1, NW
          K = 4 * J
          IF (J .EQ. 1)  GO TO 200
          NLJ = NL(J)
          X2 = 1.D0 / (DT * SIGV00(NLJ))
          I = K - 3
          DD1(I) = 0.5D0 * X2 * (0.25D0*CPM(I)*APM(I)*DZ(NLJ)
     1             + WA1(I)*(CPM(I-1)+CPM(I)))
          DD2(J) = -0.5D0 * (0.25D0*CPM(I)*APM(I-1)*DZ(NLJ-1)
     1             + WA1(I-1)*(CPM(I-1)+CPM(I))) / (DT*SIGV00(NLJ-1))
  200     X2 = X2 * CPM(K)
          DO 210  I = K-2, K
  210     DD1(I) = X2 * (WA1(I) - WA1(I-1))
  220 CONTINUE
C
      DO 240  J = 1, NW4
      DO 230  I = 1, 3
          TO(I, J) = PM(I, J) + P1 * PC(I, J) + P2 * PK(I, J)
  230 CONTINUE
  240 CONTINUE
      CALL ELSOL (1, PM, WA1, CPM, APM, IPM, NW4, 1, 3)
      CALL ELSOL (1, TO, WA1, CTO, ATO, ITO, NW4, 1, 3)
      RETURN
      END
